        function jdnext( start, intv, i )
C ***
C ******************************************************************************
C ***
C ***   Return newly computed Julian date incremented by i*intv.
C ***
C ***   start   > D     Julian start date
C ***   intv    > D     Interval (in Julian days)
C ***   i       > I     number of interval to return (zero based)
C ***
C ***   jdnext  < D     computed new Julian Date incremented by i*int
C ***
C ***   $Log: jdnext.f,v $
C ***   Revision 1.1  2002/12/15 14:13:48  michael
C ***   Initial version.
C ***
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***   Arguments
C ***
        double precision start, intv
        integer i
        double precision jdnext

C ***
C ***   Local variables
C ***
        integer month, day, year, hour, minute
        real sec
        logical months, years, end_of_month

C ***
C ***   Functions
C ***
        integer lastday
        double precision jd
        external lastday, jd

C ***
C ***   Code
C ***

C ***   We assume that if the interval is an integral multiple of 30
C ***   or 365 then we have to increment by months or years,
C ***   respectively.

        months = (int(intv/30.d0)*30.d0)/intv .le. 1.d-5
        years = (int(intv/365.d0)*365.d0)/intv .le. 1.d-5
        
        if( months .or. years ) then
C ***
C ***      increments are months or years
C ***
           call gdate( start, month, day, year, hour, minute, sec )
           if( month .gt. 1 ) then
              end_of_month = day .eq. lastday( month, year)
           else         ! January
              end_of_month = day .ge. lastday( 2, year )
           end  if
           if( months ) then
C ***
C ***         month interval
C ***
              month = month + i*int(intv/30.d0 + 0.5d0)
              if( month .gt. 12 ) then
                 year = year + 1
                 month = month - 1
              end if
           else
C ***
C ***         year interval
C ***
              year = year + i*int(intv/365.d0 + 0.5d0)
           end if
           if( end_of_month ) day = lastday( month, year )
           jdnext= jd( year, month, day, hour, minute, sec )
              
        else
C ***
C ***      interval is not more than a week -> can do simple arithmetic
C ***
           jdnext = start + i*intv
        end if

        end
