      SUBROUTINE LPLOT( XVALS, ZVALS, NX, top, xmax, xmin, YMAX, YMIN )
C
C     F I G _ L P L O T
C
C     Writes line identificationsin array (ZVALS) at positons in array
C     (XVALS) without overlap of labels.  Adopted from MIDAS.
C
C     Parameters -   (">" input, "<" output)
C
C     (>) XVALS    (Real array XVALS(NX) The abscissae
C                  for each point to be plotted.
C     (>) ZVALS    (Character array ZVALS(NX) The data to be plotted.
C     (>) NX       (Integer) Number of elements in XVALS and ZVALS.
C     (>) TOP      (Logical) Write identifications on top of frame.
C     (>) XMAX     (Real) The actual x-start value for the plot.
C     (>) XMAX     (Real) The actual x-end value for the plot.
C     (>) YMAX     (Real) The maximum value for the plot.
C     (>) YMIN      (Real) The minimum value for the plot.
C
C     Subroutines / functions used -
C
C     LBLOVE     (MIDAS this file) Calculate label positiosn without overlap
C
C ***	Modifications:
C ***
C ***	20-JUN-1995 16:14:36.00		ML / IoA.  Code taken from FIG_LPLOT
C
C+
	IMPLICIT NONE
C
C      	Parameters
C
	LOGICAL TOP
	INTEGER NX
	REAL XVALS(NX), xmin, xmax, ymin, ymax
	CHARACTER*(*) ZVALS(NX)
C
C	Functions
C
	integer lib$get_vm, idx
	real xcm, ycm, gen_elemf
C
C	Local variables
C
	LOGICAL MESH
	REAL SIZEX, SIZEY, XMARGIN, YMARGIN, X, VX, Y
	REAL Y2, CHSIZE, xchsize, DUMMY
	real xscale, yscale, length
	INTEGER I, NLIM, WRKPTR, ISIZEY, status


        CHSIZE = 0.4	! max character size [cm]

        XMARGIN = 0.6   ! [cm]
        YMARGIN = 0.6   ! [cm]

	xscale = abs( xcm(10.) - xcm(0.) )/10.
	yscale = abs( ycm(10.) - ycm(0.) )/10.

	xmargin = xmargin/xscale 	! [user coord]
	ymargin = ymargin/yscale	! [user coord]

	xchsize = chsize/xscale

C ***
C ***   Compute character size required to plot all lines.
C ***
	length = xmax - xmin 
        SIZEX = (length - 2.*XMARGIN)/NX
C ***
C ***   Limit it between default size and half the default.
C ***
        SIZEX = MIN( xCHSIZE, SIZEX )
        SIZEX = MAX( 0.5*xCHSIZE, SIZEX )
C ***
C ***	Update cm value CHSIZE
C ***
	chsize = sizex*xscale
C ***
C ***   Compute number of lines plotable with this new size.
C ***
        NLIM = MIN( NINT( (length - 2.*XMARGIN)/SIZEX ), NX )
C ***
C ***   Determine length of plotted string and find Y start position.
C ***
	ISIZEY = 0
        DO I = 1, nx
	   ISIZEY = MAX( ISIZEY, idx(ZVALS(I)) )
	END DO
        CALL textlk( dummy, sizey, chsize, ZVALS(1)(:ISIZEY), DUMMY,
     >               dummy )
        sizey = sizey/yscale	     ! [cm->user coord]
        Y2 = (YMAX - YMIN)/10.       !Strichlaenge
        IF( TOP ) THEN
           Y = YMAX - (SIZEY + y2 + y2) - YMARGIN
        ELSE
           Y = YMIN + YMARGIN
        END IF
C
C ... compute new label positions without overlap
C
        STATUS = LIB$GET_VM( 4*4*NX, WRKPTR )
        CALL LBLOVE( XVALS, NX, XMIN+XMARGIN, XMAX-XMARGIN,
     >               SIZEX, %VAL(WRKPTR), %VAL(WRKPTR+4*NX),
     >               %VAL(WRKPTR+2*4*NX), %VAL(WRKPTR+3*4*NX) )
        chsize = 0.8*chsize ! leave a bit more room between characters
        MESH = .TRUE.
        DO I = 1, nx
C          
C ...      plot labels
C
           X = GEN_ELEMF( %VAL(WRKPTR), I )
           VX = XVALS(I)
           IF( VX .GE. XMIN .AND. VX .LE. XMAX ) THEN
              IF( I .GT. NLIM ) THEN
                 IF( MESH ) THEN
                    MESH = .FALSE.
!                    write( 6, * ) 'Warning: Too many identifications'
!                    write( 6, * ) 'Change plotting scale '
                 END IF
              END IF
              IF( TOP ) THEN
                 CALL plot( xcm(VX), ycm(Y), 3 )
                 CALL plot( xcm(VX), ycm(Y+Y2), 2 )
                 IF (MESH) THEN
                    CALL plot( xcm(X), ycm(Y+Y2+Y2), 2 )
                    CALL symblk( xcm(X+0.5*SIZEX), ycm(Y+Y2+Y2), chsize,
     >                           zvals(i), 90., idx(zvals(i)) )
                 END IF
              ELSE
                 IF( MESH ) THEN
                    CALL symblk( xcm(X+0.5*SIZEX), ycm(Y+SIZEY), chsize,
     >                           zvals(i), 90., idx(zvals(i)) )
                    CALL plot( xcm(X), ycm(Y+SIZEY), 3 )
                    CALL plot( xcm(VX), ycm(Y+Y2+SIZEY), 2 )
                 ELSE
                    CALL plot( xcm(VX), ycm(Y+Y2+SIZEY), 3 )
                 END IF
                 CALL plot( xcm(VX), ycm(Y+Y2+SIZEY+Y2), 2 )
              END IF
           END IF
        END DO
	END

	function gen_elemf( x, i )
	implicit none

	real gen_elemf
	integer i
	real x(i)

	gen_elemf = x(i)

	end
