	subroutine analyze
C ***
C ******************************************************************************
C ***
C ***	Computes abundances, synthetic spectra or equivalent widths.
C ***
C ***	 6-MAY-1992 18:20:51.00
C ***
C ***
C ***	Modification:
C ***
C ***	19-SEP-1992 06:40:04.71	ML / UT Austin.  Added check if
C ***				observation normalized to 10,000 or 1.
C ***
C ***                           Scale FLUXOBS to 1 to reflect change
C ***                           made to RSCAN in other program parts
C ***
C ***	22-SEP-1994 19:21:14.00	ML / IoA Cambridge.  Moved lamobs,
C ***				fluxobs, mobs in common/obs/
C ***				Gives cleaner code bu allowing to
C ***				call putout but makes it slightly
C ***				worse in the rest of the system.
C ***	 7-OCT-1994 17:07:07.00	ML / IoA.  Added separate dimension to fluxobs
C ***				and lambos
C ***	23-NOV-1994 21:44:15.00	ML / IoA.  Don't reopen LIMITS.DAT if called
C ***				repeatedly.  Good idea?
C ***	26-NOV-1994 15:09:51.00	ML / IoA.  Add graphics plot if synthesis only
C ***	21-MAR-1995 16:56:27.00	ML / IoA.  Take out file opens and other init
C ***				cruft and put in sfilop.
C ***	10-MAR-1997 16:19:52.22	ML / Bamberg.  Always call SFILOP.
C ***   16-APR-1997 22:29:23.00 ML / Stegaurach.  Added blend check flag 31.
C ***   27-APR-1997 15:11:51.00 ML / Stegaurach.  Fixed computation of
C ***                           continuum points.  Had implicit assumption that
C ***                           resolution of computed spectrum is better than
C ***                           1 A.
C ***
C ******************************************************************************
C ***
	implicit none
	include 'LINLTE.INC'

C ***
C ***	Global variables
C ***
	include 'IF.INC'
	include 'FLUSS.INC'
	include 'FCON.INC'
	include 'CALCUL.INC'
	include 'LINE.INC'
	include 'PROFIL.INC'
	include 'RECT_SIZE.INC'

C ***
C ***	Local variables
C ***
	logical extra
	integer itma
	parameter( itma = 9 )
	integer i, m, nit, noblend
	real delcon, dmlam, x, z, v, log_wobs, log_wcal
	real wclgit( itma + 1 ), gflgit( itma + 1 )

	integer ist
	include 'FILES.INC'

C ***
C ***	Functions
C ***
	external null
	real cont, flux, pol, ucg

        call sfilop	! open files, make some error checks, read observation

        if( conti ) then
C ***
C ***      C O N T I N U U M  only
C ***      =================
C ***
           fclam = cont( mlam )
           call putout( 2 )

        else if( boa(4) ) then
C ***
C ***      S Y N T H E S I S 
C ***      =================
C ***

           call kapgam( 2 )

C ***
C ***	   compute continuum
C ***

           if( endlam - anflam .lt. 1. ) then ! less than 1 A syn interval
              dmlam = 0.5*( endlam + anflam ) ! do continuum in interval center
              delcon = 1.                     ! kinda dummy

           else if( dellam .ge. 1000. ) then  ! lo res spectrum (dellam in mA)
              dmlam = anflam                  ! do continuum at
              delcon = 0.001*dellam	      ! every synth point

           else                        ! syn more than 1 A, hi res
              dmlam = anflam + 0.5     ! do continuum 0.5 A from interval start
              delcon = 1.              ! at every Aangstroem

           end if
           nfc = 0

	   do m = 1, iend
              if( lascan(m) .ge. dmlam .or. m .eq. 1 ) then
                 nfc = nfc + 1
	         alam( nfc ) = dmlam
                 afclam( nfc ) = cont( dmlam )
                 dmlam = dmlam + delcon
              end if
           end do
C ***
C ***	   compute line spectrum
C ***
	   opinit = .true.
	   call synthesis( .not.boa(2) .or. boa(1) )
C ***                           output control

	   if( boa(2) ) then
C ***
C ***	      Iteration to fit synthetic spectra to observation
C ***
	      call spefit
	   else
C ***
C ***	      Make plot
C ***
              xmin = xpmin
	      xmax = xpmax
	      ymin = ypmin
              ymax = ypmax
	      call plotout( 1 )	! plot it
              if( pid ) call plotout(8)
              if( itv .eq. 2 ) call resize( null, ist )
	      call plotout( 4 )	! close plot
	   end if
           call putout( 3 )
           if( kcontr .ge. 0 ) then
	      fclam = pol( lascan(isyn), alam, afclam, 1, nfc, 1, extra)
	      call putout( 4 )
	   end if
        else
C ***
C ***      L I N E   A N A L Y S I S 
C ***      =========================
C ***
           call kapgam( 2 )
           fclam = cont( mlam )
           if( boa(3) ) then
C ***
C ***         coarse analysis
C ***
              do while( flux(0.) .gt. flux(-5.) )
                 call new( -1. )
              end do

              x = 0.95*blam( ntmin )
              v = log10( (fclam - x)/(fclam - flux(0.)) - 1. )

              do while( abs(v) .gt. 0.1 .and. abs(v) .le. 10. )
                 call new( v )
                 v = log10( (fclam - x)/(fclam - flux(0.)) - 1. )
              end do

              if( abs(v) .le. 10. ) call new( ucg(wobs) )

           end if

           call eqwidth
           it = 0
           if( boa(2) ) then
C ***
C ***         ITERATION to match equivalent widths
C ***
              do i = 1, kma
                 if( .not. blend(i) ) then
                    noblend = i
                    go to 1

                 end if
              end do
              write( stdout, * ) 'LINLTE>> All lines in blend '//
     >                           'excluded from match.'
              write( stdout, * ) '         Modify LINE.DAT flag 31'
              call exit

1             continue

              log_wobs = log10( max(wobs, 1.e-38) )
              log_wcal = log10( max(wcal, 1.e-38) )
              nit = 0
              do while( it .le. itma .and. abs(log_wobs - log_wcal) .ge.
     >                  0.87*del )
                 if( boa(1) ) call putout( 3 )
                 it = it + 1
                 call order( log_wcal, gflg(noblend), wclgit, gflgit,
     >                       nit )
                 if( it .eq. 1 ) then
                    z = ucg( wobs ) - ucg( wcal )
                 else
                    z = pol( log_wobs, wclgit, gflgit, 1, nit, 2, extra
     >                  ) - gflg( noblend )
                 end if
                 call new( z )
                 call eqwidth
                 log_wobs = log10( max(wobs, 1.e-38) )
                 log_wcal = log10( max(wcal, 1.e-38) )
              end do
           end if

C ***
C ***      output
C ***
           call putout( 3 )
           if( kcontr .ge. 0 ) call putout( 4 )
        end if

	end
