       subroutine curfit( x, y, npts, nterms, a, flambda, yfit, chisqr,
     >                    functn, mask, sigmaa )
C ***
C ******************************************************************************
C ***
C ***	Routine taken from: Philip R. Bevington, `Data Reduction and Error 
C ***                       Analysis for the Physical Sciences'
C ***                       McGraw-Hill, New York, 1969
C ***
C ***	Modified:
C ***	  19-APR-1992 20:25:23.47	ML / UT Austin.  Cleaned up code and
C ***					combined FDERIV with FUNCTN.
C ***                                   MODE removed
C ***	  24-APR-1992 00:29:00.03	FIRST added
C ***	   3-JUN-1992 23:42:59.71	FIRST removed.  Modified to fit in
C ***                                   LINLTE
C ***	   7-OCT-1994 17:12:49.00	ML / IoA.  Changed dimension of deriv
C ***					with parameter nobs instead of ipkt
C ***	  26-NOV-1994 15:25:10.00	ML / IoA.  call plotout(3) instead
C ***					of over if boa(1)
C ***    1-DEC-2003 23:27:00.00         ML / Stegaurach.  Interface changed to
C ***                                   return fit errors (sigmaa).
C ***
C ***	Purpose:
C ***     Make a least-squares fit to a non-linear function
C ***        with a linearization of the fitting function
C ***
C ***	Description of parameters
C ***	  X	  >  Array of data point for independent variable
C ***	  Y	  >  Array of data points for dependent variable
C ***	  NPTS	  >  Number of pairs of data points
C ***	  NTERMS  >  Number of parameters
C ***	  A	 ><  Array of parameters
C ***	  DELTAA  >  Array of increments for parameters a
C ***	  SIGMAA  <  Array of standard deviations for parameters A
C ***     FLAMBDA >  Proportion of gradient search included
C ***	  YFIT	  <  Array of calculated values of Y
C ***	  CHISQR  <  Reduced CHI square for fit
C ***     MASK    >  places where to compute chi^2
C ***             
C ***	Subroutines and function subprograms required
C ***	  FUNCTN( X, NPTS, A, NTERMS, DERIV, YFIT, DER )
C ***        Evaluates the fitting function for all X and evaluates
C ***        the derivatives with respect to each parameter A
C ***	  MATINV( ARRAY, NTERMS, DET )
C ***	     Inverts a symmetric two-dimensional matrix of degree NTERMS
C ***        and calculates its determinant
C ***
C ***	Comments
C ***	   Dimension statement valid for NTERMS up to 10
C ***      Set FLAMBDA = 0.001 at beginning of search
C ***
C ***
C ***	19-APR-1992 04:30:34.66
C ***
C ******************************************************************************
C ***
        implicit none
	include 'LINLTE.INC'
C ***   
C ***   Arguments
C ***   
        integer npts, nterms
        real x( npts ), y( npts ), a( nterms ), yfit( npts ),
     >       flambda, mask( npts ), sigmaa( nterms )
        double precision chisqr
	external functn

C ***
C ***	Global parameters
C ***
	include 'CALCUL.INC'
	include 'IF.INC'

C ***   
C ***   Local variables
C ***   
        integer nfree, i, j, k, iter, itmax
        real alpha( nfit, nfit ), beta( nfit ), array( nfit, nfit ),
     >    deriv( nobs, nfit ), b( nfit )
        real det
        double precision chisq1

	logical DER, NODER
	parameter( DER = .true., NODER = .false. )
	parameter( itmax = 8 )				! emergency brake
        
        nfree = npts - nterms
        if( nfree .gt. 0 ) then
C ***   
C ***      Evaluate alpha and beta matrices
C ***   
           do j = 1, nterms
              beta( j ) = 0.
              do k = 1, j
                 alpha( j, k ) = 0.
              end do
           end do
        
           call functn( x, npts, a, nterms, deriv, yfit, DER )

           do i = 1, npts
              do j = 1, nterms
                 beta( j ) = beta( j ) + 
     >                       ( y(i) - yfit(i) )*deriv( i, j )*mask( i )
                 do k = 1, j
                    alpha( j, k ) = alpha( j, k ) + deriv( i, j )*
     >                                          deriv( i, k )*mask( i )
                 end do
              end do
           end do
           do j = 1, nterms
              do k = 1, j
                 alpha( k, j ) = alpha( j, k )
              end do
           end do
C ***   
C ***      Evaluate chi square at starting point
C ***   
           chisq1 = 0.
           do i = 1, npts
              chisq1 = chisq1 + ( dble( mask(i)*(y(i) - yfit(i)) ) )**2
           end do
           chisq1 = chisq1/nfree
        
           flambda = 0.1*flambda
           chisqr = chisq1
	   iter = 0
           do while( chisqr .ge. chisq1 .and. iter .le. itmax )
C ***         
C ***         While chi square increases, increase flambda and compute
C ***         new chi square; do at least once
C ***         
	      iter = iter + 1
              flambda = 10.*flambda
C ***   
C ***         Invert modified curvature matrix to find new parameters
C ***         
              do j = 1, nterms
                 do k = 1, nterms
                    array( j, k ) = alpha( j, k )/
     >                              sqrt( alpha(j, j)*alpha(k, k) )
                 end do
                 array( j, j ) = 1. + flambda
              end do
              call matinv( array, nterms, det )
              do j = 1, nterms
                 b( j ) = a( j )
                 do k = 1, nterms
                    b( j ) = b( j ) + beta( k )*array( j, k )/
     >                       sqrt( alpha(j, j)*alpha(k, k) )
                 end do
              end do
              call functn( x, npts, b, nterms, deriv, yfit, NODER )
	      if( boa(1) ) call plotout( 3 )
              chisqr = 0.
              do i = 1, npts
                 chisqr = chisqr + 
     >                    ( dble( mask(i)*(y(i) - yfit(i)) ) )**2
              end do
              chisqr = chisqr/nfree
        
           end do
C ***   
C ***      Evaluate parameters and uncertainties
C ***   
           do j = 1, nterms
              a( j ) = b( j )
              sigmaa( j ) = sqrt( array(j, j)/alpha(j, j) )
           end do
           flambda = 0.1*flambda
        else
           chisqr = 0.
        end if
        end
