	function h_fij( in, im )
C ***
C ******************************************************************************
C ***
C ***	Returns Hydrogen f-value for transition from level in to level im.
C ***
C ***	in	>  I	principal quantum number of lower level
C ***	im	>  I	principal quantum number of upper level
C ***
C ***	Modifications:
C ***
C ***	14-JUN-1994 19:06:04.00		ML / IoA Cambridge.  Created.
C ***	23-AUG-1994 19:40:22.00		ML / IoA.  Modified expression to
C ***					avoid overflow, added double prec.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer im, in
	real h_fij

C ***
C ***	Local variables
C ***
	double precision n, m

C ***
C ***	Functions
C ***
	double precision hyp2f1

C ***
C ***	Code
C ***
	m = im
	n = in

*	h_fij = 32. * m**4 * n**2 * 
*     >                (m - n)**( 2*int(m) + 2*int(n) - 4 )/
*     >          (3. * (m + n)**( 2*int(m) + 2*int(n) + 3 )*
*     >          (Hyp2F1( -n, -m + 1., 1., (-4.*n*m)/(m - n)**2 )**2 - 
*     >           Hyp2F1( -n + 1., -m, 1., (-4.*n*m)/(m - n)**2 )**2)

	h_fij = 32.d0/3.d0 * m**4 * n**2 * 
     >                ((m - n)**( (2.d0*m + 2.d0*n - 4.d0)/
     >                            (2.d0*m + 2.d0*n + 3.d0) )/
     >                 (m + n))**( 2*int(m) + 2*int(n) + 3 )*
     >      (Hyp2F1( -n, -m + 1.d0, 1.d0, (-4.d0*n*m)/(m - n)**2 )**2 -
     >       Hyp2F1( -n + 1.d0, -m, 1.d0, (-4.d0*n*m)/(m - n)**2 )**2)

	end
