	subroutine he2_data( k )
C ***
C ******************************************************************************
C ***
C ***	Fill in line data for Helium II lines with VCSSB theory.
C ***	Wavelengths are not too accurate.
C ***
C ***	K	I  >	# of line in blend.
C ***
C ***	Modifications:
C ***
C ***	 3-NOV-1997 16:17:03.48		ML / Bamberg.  Made from h_data.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer k

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'LINDAT.INC'
	include 'LINE.INC'

C ***
C ***	Local variables
C ***
	integer i, j, nl_min, nl_max, nl, nu
	real d1, d2, dl, dll, l1, l2, n, m

	real ryd
	parameter( ryd = 3.28808691e15*1.000412714e0 )	! tweaked to give 
						! correct wavelength in he2_data

C ***
C ***	Functions
C ***
	real lamair, he2_vnm, h_fij, he2_lam


	he2_vnm(i,j)  = 4.e0*ryd*(1.e0/(i*i) - 1.e0/(j*j))
	he2_lam(i,j) = lamair( 2.997925e18/he2_vnm(i, j) )


C ***
C ***	Code
C ***
	if( lprof(k) .eq. PVCSSB ) then
C ***
C ***	   for VCS
C ***
	   if( diu(k) .eq. 0. .or. lu(k) .eq. 0 ) then
	      if( lamk(k) .eq. 0. ) then
                 write( stdout, * ) 'LINLTE>> Need either wavelength',
     >                              ' or quantum numbers for VCS',
     >                              ' Helium II line.'
	         write( stdout, * ) '         Was processing line #', k
	         call exit
              end if
C ***
C ***         find n_low, get from series limit 
C ***	      Start with candidates for n_low, series might overlap
C ***
	      do i = 1, 100
	         if( lamk(k) .lt. he2_lam(i, 1000) ) then
                    go to 1
                 end if
              end do
1	      continue

	      nl_max = max( i - 1, 1 )

C ***
C ***	      and n_up:
C ***
              dll = 1e10		! something big for test below
	      do nl = 1, nl_max		! try all possible lower n
                 l2 = 1e10		! something big
                 do i = nl + 1, 100		! loop over all upper levels
                    l1 = he2_lam( nl, i )
                    d1 = abs(lamk(k) - l1) 	! lambda is between these two
	            d2 = abs(lamk(k) - l2)
                    if( d1 .lt. d2 ) then	! find closer one
                        nu = i
	                dl = d1
                    else
                        nu = i - 1
	                dl = d2
                    end if
                    if( lamk(k) .le. l2 .and. lamk(k) .ge. l1 ) then
                       go to 2	! got it for this series; do next series
                    end if
                    l2 = l1
                 end do
2	         continue	! NU possible upper n,
				! DL distance to desired wavelength for that n

                 if( dl .lt. dll ) then	! if closer than last use this
	            lu(k) = nl		! lower n
                    diu(k) = nu		! upper n
                    dll = dl		! distance to desired wavelength
	         end if
              end do

              lamk( k ) = 0.	! and get real wavelength from below
           end if


	   if( lamk(k) .eq. 0. ) then
	      lamk( k ) = he2_lam( lu(k), int(diu(k)) )
	   end if

	   if( chik(k) .eq. 0. ) then
	      chik(k) = 54.402*( 1. - 1./lu(k)/lu(k) )
	   end if

	   if( gflg(k) .eq. 0. ) then
	      gflg( k ) = log10( 2.*lu(k)*lu(k)*
     >                           h_fij( lu(k), int(diu(k)) ) )
	   end if

	end if
	end
