      SUBROUTINE LPLOT( XVALS, ZVALS, NX, top, xmax, xmin, YMAX, YMIN )
C
C     F I G _ L P L O T
C
C     Writes line identificationsin array (ZVALS) at positons in array
C     (XVALS) without overlap of labels.  Adopted from MIDAS.
C
C     Parameters -   (">" input, "<" output)
C
C     (>) XVALS    (Real array XVALS(NX) The abscissae
C                  for each point to be plotted.
C     (>) ZVALS    (Character array ZVALS(NX) The data to be plotted.
C     (>) NX       (Integer) Number of elements in XVALS and ZVALS.
C     (>) TOP      (Logical) Write identifications on top of frame.
C     (>) XMAX     (Real) The actual x-start value for the plot.
C     (>) XMAX     (Real) The actual x-end value for the plot.
C     (>) YMAX     (Real) The maximum value for the plot.
C     (>) YMIN      (Real) The minimum value for the plot.
C
C     Subroutines / functions used -
C
C     LBLOVE     (MIDAS this file) Calculate label positiosn without overlap
C
C ***	Modifications:
C ***
C ***	20-JUN-1995 16:14:36.00		ML / IoA.  Code taken from FIG_LPLOT
C ***	 3-JUN-1996 14:34:47.14		ML / Bamberg.  LIB$FREE_VM...
C ***	26-JUL-1996 16:24:02.97		ML / Bamberg.  Changed call plot to
C ***	                                over & mark non-fitted lines with
C ***	                                dashed lines -- symblk still needs
C ***                                   to be replaced, also textlk
C ***	17-NOV-1997 03:22:55.36		ML / Bamberg.  Add CALL PLTSYN even
C ***					though this should be handled by plot
C ***                                   system...
C ***	20-FEB-1998 14:02:55.91		ML / Bamberg.  Handling of molecule
C ***					labels.
C ***	26-FEB-1998 12:38:19.91		ML / Bamberg.  Molecule names slightly 
C ***					improved.
C ***	 5-MAR-1998 20:23:21.71		ML / Bamberg.  Replaced exit with goto
C
C+
	IMPLICIT NONE
C
C      	Parameters
C
	LOGICAL TOP
	INTEGER NX
	REAL XVALS(NX), xmin, xmax, ymin, ymax
	CHARACTER*(*) ZVALS(NX)
C
C	Functions
C
	integer lib$get_vm, idx, lib$free_vm
	real xcm, ycm, gen_elemf
C
C	Local variables
C
	LOGICAL MESH
	REAL SIZEX, SIZEY, XMARGIN, YMARGIN, X, VX, Y
	REAL Y2, CHSIZE, xchsize, DUMMY
	real xscale, yscale, length
        real xline(3), yline(3), style
	INTEGER I, j, NLIM, WRKPTR, ISIZEY, status, ielem, is
	character lastz*8

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'IF.INC'
	include 'CALCUL.INC'


	call pltsyn( .true. )

        CHSIZE = 0.4	! max character size [cm]

        XMARGIN = 0.6   ! [cm]
*        YMARGIN = 0.6   ! [cm]
        YMARGIN = -0.4   ! [cm]

	xscale = abs( xcm(10.) - xcm(0.) )/10.
	yscale = abs( ycm(10.) - ycm(0.) )/10.

	xmargin = xmargin/xscale 	! [user coord]
	ymargin = ymargin/yscale	! [user coord]

	xchsize = chsize/xscale

C ***
C ***   Compute character size required to plot all lines.
C ***
	length = xmax - xmin 
        SIZEX = (length - 2.*XMARGIN)/NX
C ***
C ***   Limit it between default size and half the default.
C ***
        SIZEX = MIN( xCHSIZE, SIZEX )
        SIZEX = MAX( 0.5*xCHSIZE, SIZEX )
C ***
C ***	Update cm value CHSIZE
C ***
	chsize = sizex*xscale
C ***
C ***   Compute number of lines plotable with this new size.
C ***
        NLIM = MIN( NINT( (length - 2.*XMARGIN)/SIZEX ), NX )
C ***
C ***   Determine length of plotted string and find Y start position.
C ***
	ISIZEY = 0
        is = 1000
        DO I = 1, nx
	   do j = 1, len(zvals(i))
	      if( zvals(i)(j:j) .ne. ' ' ) then  ! find leading spaces
	         is = min( is, j )
	         is = min( 5, is ) ! but keep at least 4 numbers
	         go to 1
	      end if
	   end do
1          continue
	   ISIZEY = MAX( ISIZEY, idx(ZVALS(I)) )
	END DO
        CALL textlk( dummy, sizey, chsize, ZVALS(1)(is:ISIZEY), DUMMY,
     >               dummy )
        sizey = sizey/yscale	     ! [cm->user coord]
        Y2 = (YMAX - YMIN)/10.       !Strichlaenge
        IF( TOP ) THEN
           Y = YMAX - (SIZEY + y2 + y2) - YMARGIN
        ELSE
           Y = YMIN + YMARGIN
        END IF
C
C ... compute new label positions without overlap
C
        STATUS = LIB$GET_VM( 4*4*NX, WRKPTR )
!        if( .not.status ) call lib$signal(%val(status))
        CALL LBLOVE( XVALS, NX, XMIN+XMARGIN, XMAX-XMARGIN,
     >               SIZEX, %VAL(WRKPTR), %VAL(WRKPTR+4*NX),
     >               %VAL(WRKPTR+2*4*NX), %VAL(WRKPTR+3*4*NX) )
        chsize = 0.8*chsize ! leave a bit more room between characters
        MESH = .TRUE.
        lastz = 'xxxx'
        style = 0.	! for synthesis only
        DO I = 1, nx
C          
C ...      plot labels
C
           if( zvals(i)(:8) .ne. lastz .and. boa(2) ) then
              read( zvals(i), '(i8)' ) ielem
              style = 0.1	! non-fitted line
              do j = 1, mfit
                 if( ielem .eq. fit_item(j) ) then
                    style = 0.	! fitted line
                    go to 2
                 end if
              end do
2	      continue
	   end if
           lastz = zvals(i)(:8)

           X = GEN_ELEMF( %VAL(WRKPTR), I )
           VX = XVALS(I)
           IF( VX .GE. XMIN .AND. VX .LE. XMAX ) THEN
              IF( I .GT. NLIM ) THEN
                 IF( MESH ) THEN
                    MESH = .FALSE.
!                    write( 6, * ) 'Warning: Too many identifications'
!                    write( 6, * ) 'Change plotting scale '
                 END IF
              END IF
              is = 1
	      do j = 1, len(zvals(i))
	         if( zvals(i)(j:j) .ne. ' ' ) then ! find leading spaces
	            is = j
                    is = min( 5, is )	! keep at least 4 numbers
	            go to 3
	         end if
	      end do
3             continue
              IF( TOP ) THEN
                 xline(1) = vx
                 yline(1) = y
                 xline(2) = vx
                 yline(2) = y + y2
                 xline(3) = x
                 yline(3) = y + y2 + y2
                 if( .not.mesh ) then
                    call over( xline, yline, 2, style )
                 else
                    call over( xline, yline, 3, style )
                    CALL symblk( xcm(X+0.5*SIZEX), ycm(Y+Y2+Y2), chsize,
     >                           zvals(i)(is:), 90., idx(zvals(i))-is-1)
                 END IF
              ELSE
                 xline(1) = x
                 yline(1) = y + sizey
                 xline(2) = vx
                 yline(2) = y + y2 + sizey
                 xline(3) = vx
                 yline(3) = y + y2 + y2 + sizey
                 IF( MESH ) THEN
                    CALL symblk( xcm(X+0.5*SIZEX), ycm(Y+SIZEY), chsize,
     >                           zvals(i)(is:), 90., idx(zvals(i))-is-1)
                    call over( xline, yline, 3, style )
                 ELSE
                    call over( xline(2), yline(2), 2, style )
                 END IF
              END IF
           END IF
        END DO
        STATUS = LIB$FREE_VM( 4*4*NX, WRKPTR )
!        if( .not.status ) call lib$signal(%val(status))
	call pltsyn( .false. )
	END

	function gen_elemf( x, i )
	implicit none

	real gen_elemf
	integer i
	real x(i)

	gen_elemf = x(i)

	end
