     	subroutine plotout( mode )
C ***
C ******************************************************************************
C ***
C ***	Routine to plot spectrum/fit.  Modeled after putout.
C ***
C ***	MODE	> I	which plot to make -- see comments in code
C ***
C ***
C ***	Modifications:
C ***
C ***	22-SEP-1994 21:01:14.00		ML / IoA.  Created.
C ***	 8-OCT-1994 14:14:23.00		ML / IoA.  Added call to new_plot for
C ***					better X windows handling.
C ***	18-NOV-1994 18:01:31.00		ML / IoA.  Hardcopy support, plotout(7).
C ***	21-NOV-1994 18:06:46.00		ML / IoA.  Changed PLTEND calls to PEND
C ***	26-NOV-1994 15:13:57.00		ML / IoA.  Changed plotout(1) slightly
C ***					to allow for synthesis only plot.
C ***	20-MAY-1996 15:15:45.36		ML / Bamberg.  Changed sequence in which
C ***					fit parameters are plotted
C ***	31-MAY-1996 19:51:51.64		ML / Bamberg.  Added plot of line ids
C ***   10-MAY-1997 21:31:50.00         ML / Bamberg.  graph was set wrong.
C ***                                   Added boa(2) check.  Was implicitly
C ***                                   assumed to be .true..  Broke with
C ***                                   synthesis only plot.
C ***	18-DEC-1997 17:23:53.25		ML / Bamberg.  iflimits .eq. 3
C ***	20-FEB-1998 14:16:58.84		ML / Bamberg.  Handling of molecule ids
C ***    1-DEC-2003 23:27:00.00         ML / Stegaurach.  Output of fit errors.
C ***
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer mode

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'CALCUL.INC'
	include 'PROFIL.INC'
	include 'TEFF.INC'
	include 'IF.INC'
	include 'PRINT.INC'
	include 'NLTE.INC'
	include 'RECT_SIZE.INC'
	include 'OBS.INC'
	include 'FIT.INC'
	include 'ERASURE.INC'
	include 'LINE.INC'
	include 'LINDAT.INC'

C ***
C ***	Local variables
C ***

	logical graph, plotted, hard, refresh, do_rect
	integer i, j, k, status
	real xinc, yinc, x(4), y(4), x0, y0, size, xlab(nkma)
	character xtext*19, ytext*9, zeile*132, plab(nkma)*18, lab*18

C ***
C ***	Functions
C ***
	integer new_plot

	save plotted, hard, refresh

	data plotted /.false./

        graph = boa(1) .or. itv .gt. 1 .or.
     >          ( (iflimits .eq. 1 .or. iflimits .eq. 3) .and. boa(2) )

	if( .not.graph ) return

	if( mode .eq. 1 ) then
C ***
C ***	   Make new plot
C ***
C ***	   pass plot size via xmin, xmax, ymin, ymax in common /rect_size/
C ***
C ***	   Stays in grafics mode on exit (or whatever your grafix package
C ***	   does after something has been plotted)
C ***
	   if( plotted ) then
              status = new_plot()
	      if( status .eq. failed ) then
	         call pend
                 if( hard ) call phard
                 plotted = .false.
	      else if( status .eq. no_refresh ) then
	         refresh = .false.
                 return

	      end if
	   end if
	   if( .not.plotted ) then
	      call pbegin( 26., 20. )
              hard = itv .eq. 3 .and.
     >               (iflimits .eq. 1 .or. iflimits .eq. 3)
	      plotted = .true.
              refresh = .true.
	   end if

           xinc  = 0.
           yinc  = 0.
	   if( .not. ifwave ) then
	      xtext = 'Wavelength [A]'
	   else
	      xtext = 'Wavenumber [cm&H-1]'
	   end if
	   if( ifsurf .eq. 1 ) then
	      ytext = 'Flux'
	   else
	      ytext = 'Intensity'
	   end if
	   if( ihead .gt. 0 .and. head(:16) .ne. ' ATM-FEINANALYSE' )
     >     then
	      zeile = head(:ihead)//
     >                ' T&teff&m=xxxxx, log g=x.xx; '//date
              j = ihead - 8
	   else
	      zeile = ' LINFIT: T&teff&m=xxxxx, log g=x.xx; '//date
              j = 0
	   end if
	   write( zeile(j+19:j+24), '(i5)' ) nint(teff)
	   write( zeile(j+32:j+35), '(f4.2)' ) glog

	   call plotscale
	   if( boa(2) ) then
     	      call diagr( 4.5, 3., 20., 15., xmin, xmax, ymin, ymax,
     >                    xinc, yinc, xtext, ytext, zeile, 1, 1, lamobs,
     >                    fluxobs, mobs, 0. )
	      call over( lamobs, yfit, mobs, 0.1 )
	   else
     	      call diagr( 4.5, 3., 20., 15., xmin, xmax, ymin, ymax,
     >                    xinc, yinc, xtext, ytext, zeile, 1, 1, lascan,
     >                    rstreu, mma, 0. )
	   end if

	else if( .not.plotted ) then
C ***
C ***	   Programming error.  Must have opened plot first for 
C ***	   rest of the functions.
C ***
	   write( stdout, * ) 'Illegal call of PLOTOUT - must ',
     >                        'call PLOTOUT(1) first.'
           call exit
	else if( mode .eq. 2 ) then
C ***
C ***	   plot fit_mask
C ***
C ***	   no change of grafix/text mode =>
C ***	   on entry: be in grafix mode
C ***	   on exit:  grafix mode
C ***
           if( .not.refresh ) then
	      refresh = .true.
	      return
	   end if

	   x( 1 ) = lamobs( 1 ) !minmax
           y( 1 ) = ymin
	   x( 2 ) = x( 1 )
           y( 2 ) = ymax - 0.1*(ymax - ymin)
           do_rect = fit_mask(1) .eq. 0.
           do j = 2, mobs
	      if( fit_mask(j) .lt. fit_mask(j-1) ) then
C ***
C ***		 Mask 1 -> 0 => start of rectangle
C ***
	         x( 1 ) = lamobs( j )
                 y( 1 ) = ymin
	         x( 2 ) = x( 1 )
                 y( 2 ) = ymax - 0.1*(ymax - ymin)
                 do_rect = .true.
	      else if( fit_mask(j) .gt. fit_mask(j-1) ) then
C ***
C ***            Mask 0 -> 1 => end of rectangle
C ***
                 x( 3 ) = lamobs( j - 1 )
	         y( 3 ) = ymax - 0.1*(ymax - ymin)
	         x( 4 ) = x( 3 )
	         y( 4 ) = ymin
	         call over( x, y, 4, 0. )
                 do_rect = .false.
	      end if
           end do
           if( do_rect ) then		! catch last point
              x( 3 ) = lamobs( mobs )
	      y( 3 ) = ymax - 0.1*(ymax - ymin)
	      x( 4 ) = x( 3 )
	      y( 4 ) = ymin
	      call over( x, y, 4, 0. )
	   end if
	else if( mode .eq. 3 ) then
C ***
C ***	   overplot new fit
C ***
C ***	   switches text/grafix/text
C ***	   on entry; text 1 plane
C ***	   on exit:  text 1 plane
C ***
	   call gplane( 3, noErase, Single_plane )
	   call over( lamobs, yfit, mobs, 0.1 )
	   call tplane( noErase, Single_plane )
	else if( mode .eq. 4 ) then
C ***
C ***	   finish plot
C ***
C ***	   on entry: grafix
C ***	   on exit:  text (or whatever your plot package does on finish)
C ***
	   if( boa(2) ) call over( lamobs, yfit, mobs, 0. )
	   call pend
           plotted = .false.
	else if( mode .eq. 7 ) then
C ***
C ***	   write current fit parameters into plot (like putout(7))
C ***
!		Darn, this can't be done independent of plot package.
!		Need a text plotting function (e.g. symblk).
!
	   size = (ymax - ymin)/60.	   ! character size
	   x0 = xmin + (xmax - xmin)/50.   ! start position in x 
	   y0 = ymin + 3.*size		   ! start position in y
           if( ifnlte .eq. 1 ) then
	      call ptext( x0, y0, size, 'NLTE '//ident )
           else
	      call ptext( x0, y0, size, 'LTE' )
           end if
           y0 = y0 + 1.6*size
	   write( zeile, '(a, 1p, g24.15e3)' ) '#c#&h2&m: ',
     >       chisqr
	   call ptext( x0, y0, size, zeile )
	   y0 = y0 + 1.6*size
           do i = mfit, 1, -1
              if( fit_item(i) .eq. 13 ) then
	         write( zeile, '(a, 1p, g15.7, a, g15.7)' ) '#x#&Tmacro&m: ',
     >             fit_value( i ), '+/-', fit_err(i)
              else if( fit_item(i) .eq. 14 ) then
	         write( zeile, '(a, 1p, g15.7, a, g15.7)' ) 'v sini: ',
     >             fit_value( i ), '+/-', fit_err(i)
	      else if( fit_item(i) .eq. 55 ) then
	         write( zeile, '(a, 1p, g15.7, a, g15.7)' ) '#x#&Tmicro&m: ',
     >             fit_value( i ), '+/-', fit_err(i)
	      else
	         write( zeile, '(i10, a, 1p, g15.7, a, g15.7)' )
     >             fit_item( i ), ': ',
     >             start_value( i ) + fit_value( i ), '+/-', fit_err(i)
	      end if
	      call ptext( x0, y0, size, zeile )
	      y0 = y0 + 1.6*size
	   end do
	else if( mode .eq. 8 ) then
C ***
C ***	   Plot line identifications
C ***
           i = 0
	   do k = 1, kma
              if( import(k) .and. 
     >            lamk(k) .ge. xmin .and.
     >            lamk(k) .le. xmax )
     >        then
                 write( lab, '(i8,1x,f9.3)' ) namk(k), lamk(k)
                 call corder( lamk(k), lab, xlab, plab, i )
              end if
           end do
           if( i .gt. 0 ) then
              call lplot( xlab, plab, i, .true., xmax, xmin, ymax,
     >                    ymin )
           end if
	else

	   write( stdout, * ) 'Illegal argument to PLOTOUT', mode
	   call exit

	end if

	end
