	subroutine rotcon( lamin, datin, datout, n, vsini, beta, fwhm, 
     >	  width, opt )
	implicit none
C ***
C ******************************************************************************
C ***
C ***	Faltet Array datin mit Rotations- oder Gauprofil.
C ***
C ***	Input:
C ***
C ***	lamin:	Wellenlngenskala
C ***	datin:	Zu faltendes Spektrum
C ***	n:	Anzahl Spektrumspunkte
C ***	vsini:	Rotationsgeschwindigkeit
C ***	beta:	Randverdunkelungsparameter (Standard: 1.5)
C ***	fwhm:	Halbwertsbreite in 
C ***	width:	Integrationsbreite in width units (Standard: 3.5)
C ***	opt:	1: Gauprofil; 2: Rotationsprofil
C ***
C ***
C ***	11-JUN-1992 00:47:38.71
C ***
C ***	Modified:
C ***
C ***	15-JUN-1992 23:50:16.52		ML / UT Austin
C ***                                   Modified for equidistant wavelength
C ***                                   scale.
C ***                                   Treat start and end of spectrum by
C ***                                   assuming data outside defined range is
C ***                                   equal to 1st resp. last data point.
C ***
C ******************************************************************************
C ***
C ***	Arguments
C ***
	integer n, opt
	real lamin(n), datin(n), datout(n), vsini, beta, fwhm, width

C ***
C ***	Local variables
C ***
	integer rot, gauss
	parameter( GAUSS = 1, ROT = 2 )

	real clight, pi
	parameter( clight = 2.99793e5, pi = 3.1415926535897932 )

	integer ista, iend, idel, jdx, i
	real gx1, gx2, gx3, gw, gxgx
	real del
	real d, z, dwid, sum, profil, xx
	real l1, l2, dat

	if( opt .ne. ROT .and. opt .ne. GAUSS ) then
	   write( 6, * ) 'ROTCON: Option ', opt, ' not recognized ', 
     >	     'as profile.'
	   return
	end if
           
	z = vsini/clight
	d = fwhm*0.60056

C ***
C ***	Schleifenindizes fr Profilbreite bestimmen
C ***
	del = (lamin(n) - lamin(1))/(n - 1)
	if( opt .eq. GAUSS ) then
	   dwid = 2.*width*d
	   ista = -int( 0.5*dwid/del )
	else 
	   dwid = 2.*z*lamin(1)
	end if                                
	idel = int( dwid/del )
        
	if( idel .lt. 4 ) then
	   write( 6, * )
     >       'ROTCON: Too few lambda steps.  No convolution done.'
	   do i = 1, n
	      datout( i ) = datin( i )
	   end do
	   return
	end if
        
	do jdx = 1, n 			

	   if( opt .eq. ROT ) then
	      dwid = 2.*z*lamin(jdx)
	      idel = int( dwid/del ) - 1
              ista = int( (lamin(jdx) - 0.5*dwid - lamin(1))/del ) + 2
	   else
	      ista = ista + 1
	   end if

	   iend = ista + idel

	   if( ista .le. 0 ) then
	      l1 = lamin(1) + (ista-1)*del
	      l2 = lamin(2) + (ista-1)*del
	      dat = datin(1)
	   else if( iend .gt. n ) then
	      l1 = lamin( 1 ) + ( ista - 1 )*del
	      l2 = lamin( 2 ) + ( ista - 1 )*del
	      dat = datin( n )
	   else
	      l1 = lamin(ista)
	      l2 = lamin(ista+1)
	      dat = datin( ista )
	   end if
           
C ***      
C ***	   Integrationsstellen GXi
C ***      
	   gx2 = ( l1 - lamin(jdx) )/dwid*2.
	   gx3 = ( l2 - lamin(jdx) )/dwid*2.
C ***      
C ***	   Integrationsgewicht
C ***      
	   gw = 0.25*(gx3 - gx2) + 0.5*(gx2 + 1.)
C ***      
C ***	   Profilberechnung
C ***      
C ***      
C ***	   First point
C ***      
           gxgx = gx2*gx2
	   if( opt .eq. GAUSS ) then
              xx = exp( -gxgx*width*width )
	   else if( opt .eq. ROT ) then
	      gxgx = max( 0., gxgx )
	      gxgx = min( 1., gxgx )
              xx = 4./pi*sqrt( 1. - gxgx ) + beta*( 1. - gxgx )
	   end if
           profil = gw*xx*dat
           sum = gw*xx
           
	   do i = ista + 1, iend - 1
C ***      
C ***	      other points
C ***      
	      if( ista .le. 0 ) then
	         l2 = lamin( 2 ) + ( i - 1 )*del
	         dat = datin( min(max(1,i),n) )
	      else if( iend .gt. n ) then
	         l2 = lamin( 2 ) + ( i - 1 )*del
	         dat = datin( max(min(n,i),1) )
	      else
	         l2 = lamin( i + 1 )
	         dat = datin( i )
	      end if
        
	      gx1 = gx2
	      gx2 = gx3
              gx3 = (l2 - lamin(jdx))/dwid*2.
           
	      gw = 0.25*( gx3 - gx1 )
           
	      gxgx = gx2*gx2
	      if( opt .eq. GAUSS ) then
	         xx = exp( -gxgx*width*width )
	      else
	         gxgx = max( 0., gxgx )
	         gxgx = min( 1., gxgx )
	         xx = 4./pi*sqrt( 1. - gxgx ) + beta*(1. - gxgx)
	      end if
	      profil = profil + gw*xx*dat
	      sum = sum + gw*xx
	   end do
C ***      
C ***	   last point
C ***      
	   if( ista .le. 0 ) then
	      dat = datin( min(max(1,iend),n) )
	   else if( iend .gt. n ) then
	      dat = datin( max(min(n,iend),1) )
	   else
	      dat = datin( iend )
	   end if
           
	   gw = 0.25*( gx3 - gx2 ) + 0.5*( 1. - gx3 )
           gxgx = gx3*gx3
	   if( opt .eq. GAUSS ) then
              xx = exp( -gxgx*width*width )
	   else if( opt .eq. ROT ) then
	      gxgx = max( 0., gxgx )
	      gxgx = min( 1., gxgx )
              xx = 4./pi*sqrt( 1. - gxgx ) + beta*( 1. - gxgx )
	   end if
           profil = profil + gw*xx*dat
           sum = sum + gw*xx
C ***      
C ***      
C ***	   done
C ***      
	   profil = profil/sum
	   datout(jdx) = profil
	end do

	end
