	function vcs( dlam, t, ne, kma )
C ***
C ******************************************************************************
C ***
C ***	Interpolate VCS tables.
C ***
C ***	DLAM	>  R	delta lambda from line center []
C ***	T	>  R	temperature [K]
C ***	ne	>  R	electron density [cm^-1]
C ***	kma	>  I	number of blend line
C ***
C ***	Expects lower and upper quantum numbers in LU and DIU from
C ***	COMMON /LINDAT/.
C ***
C ***	Modifications:
C ***
C ***	 8-JUN-1994 14:37:20.00		ML / IoA Cambridge.  Created.
C ***	14-JUN-1994 18:27:02.00		ML / IoA.  Use abs(dlam).
C ***	15-JUN-1994 17:26:08.00		ML / IoA.  Use LU and DIU instead of
C ***					DRRCA and C4LG.
C ***	23-AUG-1994 16:00:41.00		ML / IoA.  New table formats etc.
C ***	14-OCT-1994 15:16:15.00		ML / IoA.  Fixed wrong T/ne interpol-
C ***					ation criterion
C ***	25-JUN-1996 19:18:33.44		ML / Bamberg.  PMT=7
C ***	26-JUN-1996 13:45:46.67		ML / Bamberg.  MLINE=40
C ***	10-MAR-1997 16:10:42.62		ML / Bamberg.  Move OPEN(VCS.DAT) to
C ***                                   SFILOP for earlier error message.
C ***   27-JUN-1997 14:30:53.00         ML / Bamberg.  No extrapolation in ne.
C ***    8-JUL-1997 23:50:36.00         ML / Bamberg.  Fix f0 for no extrapol-
C ***                                   ation in ne.
C ***	18-AUG-1997 15:58:04.05		ML / Bamberg.  Cosmetics in error
C ***					messages.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
        real dlam, t, ne, vcs
	integer kma

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'FILES.INC'
	include 'LINDAT.INC'

C ***
C ***	Local variables
C ***
	integer pmt, pmne, pmp, mline, mij
	parameter( pmt = 7,	! # of temp points in VCS table
     >             pmne = 17,   ! # of density points in VCS table
     >             pmp = 54,    ! # of profile points in VCS table
     >             mline = 40,  ! # of H lines in VCS table
     >             mij = 22     ! max principal quantum number of VCS table
     >           )

	integer ij(mij,mij)		    ! x-ref to identify line
	integer mp(mline), mne(mline), mtemp(mline)
     	real log_alpha0(mline), log_ne0(mline), log_t0(mline),
     >       log_alpha_inc(mline), log_ne_inc(mline), log_t_inc(mline),
     >       ne0   

        real svcs(pmt, pmne, 0:pmp, mline)	! actual VCS table
	real prof(pmp)				! T/ne interpolated VCS profile

	real f0, dl

	integer i, j, k, line, nline, nl(mline), nu(mline), nnl, nnu

	integer old_line
	real old_t, old_ne 
	data old_t /-1./, old_ne /-1./, old_line /-1/

	integer itemp, ine, ialp
	real log_t, log_ne, log_alpha, btemp, bne, balp
	real wttemp, wtne, wt

	real x

	logical first 
	data first /.true./

C ***
C ***	Functions
C ***
	real exp10

	save first, old_t, old_ne, old_line, prof, svcs, ij
	save log_alpha0, log_ne0, log_t0,
     >       log_alpha_inc, log_ne_inc, log_t_inc,
     >       mp, mne, mtemp
        save ne0

        exp10( x ) = exp( 2.30258509299405E0*x )

C ***
C ***	Code
C ***
	if( first ) then
	   first = .false.
!	   open( vcs_dat, file = 'VCS.DAT', status = 'OLD' )
	   read( vcs_dat, * ) nline
	   if( nline .gt. mline ) then
	      write( stdout, * ) 'LINLTE>> Too many lines in VCS table.'
	      write( stdout, * ) '         Not more than ', mline,
     >                           ' please.'
	      call exit
	   end if
	   do i = 1, mij
	      do j = 1, mij
	         ij( j, i ) = 0
	      end do
	   end do
	   do i = 1, nline
	      read( vcs_dat, * ) nl(i), nu(i),
     >           log_alpha0(i), log_ne0(i), log_t0(i),
     >           log_alpha_inc(i), log_ne_inc(i), log_t_inc(i),
     >           mp(i), mne(i), mtemp(i)
	      if( mp(i)     .gt. pmp .or.
     >            mne(i)    .gt. pmne .or.
     >            mtemp(i)  .gt. pmt ) then
	         write( stdout, * ) 'LINLTE>> VCS table too big in ',
     >                              'one of these:'
	         write( stdout, * ) '         mp: ', mp(i), ' >', pmp
	         write( stdout, * ) '         mne: ', mne(i), ' >', pmne
	         write( stdout, * ) '         mt: ', mtemp(i), ' >', pmt
 	         call exit
	      end if
	      ij( nl(i), nu(i) ) = i
	   end do
           do line = 1, nline
	      read( vcs_dat, '(3x,i2,4x,i2)' ) nnl, nnu
              if( nnu .ne. nu(line) .or. nnl .ne. nl(line) ) then
                 write( stdout, * ) 'LINLTE>> Inconsistency in ',
     >                              'VCS table.'
	         write( stdout, '(1x,a,i2.2,a,i2.2)' )
     >                              '         Found nl=', nnl,
     >                              ' nu=', nnu
	         write( stdout, '(1x,a,i2.2,a,i2.2)' )
     >                              '         expected nl=', nl(line),
     >                              ' nu=', nu(line)
	         call exit
	      end if
	      read( vcs_dat, * ) (( (svcs(i,j,k,line), k = 0, mp(line)),
     >                           i = 1, mtemp(line)),
     >                             j = 1, mne(line))
	   end do
	   close( vcs_dat )
	   write( stdout, * ) 'VCS.DAT  OK!'
	end if ! first


	line = ij( lu(kma), int(diu(kma)) )	! ij(nlow, nup)	
	if( line .eq. 0 ) then
	   write( stdout, * ) 'LINLTE>>  No VCS data for line #',
     >                        kma
	   call exit
	end if

C ***
C ***	Temperature and electron density interpolation
C ***
	if( t .ne. old_t .or. ne .ne. old_ne .or.
     >      line .ne. old_line) then
           old_t = t
	   old_ne = ne
           old_line = line
           log_t = log10( t )
           ne0 = exp10( log_ne0(line) )
           btemp = ( log_t - log_t0(line) )/log_t_inc(line) + 1.
           itemp = max( min(int(btemp), mtemp(line)-1), 1 )
           wttemp = btemp - itemp

           log_ne = log10( ne )
           bne = ( log_ne - log_ne0(line) )/log_ne_inc(line) + 1.
           ine = max( min(int(bne), mne(line)-1), 1 )
           wtne = bne - ine
           if( wtne .lt. 0. ) wtne = 0.    ! no extrapolation
           if( wtne .gt. 1. ) wtne = 1.

	   if( svcs( itemp,     ine,     0, line ) .lt. 0. .or.
     >         svcs( itemp + 1, ine,     0, line ) .lt. 0. .or.
     >         svcs( itemp,     ine + 1, 0, line ) .lt. 0. .or.
     >         svcs( itemp + 1, ine + 1, 0, line ) .lt. 0. ) then
	      write( stdout, * ) 'LINLTE>> Warning: bad VCS table',
     >                           ' entries for T/ne interpolation'
	      write( stdout, * ) '         for H line ', nl(line),
     >                                           ' -> ', nu(line)
	      write( stdout, * ) '         T = ', T
	      write( stdout, * ) '         ne = ', ne
	      write( stdout, * ) '         badness: ', 
     >                           svcs( itemp,     ine,     0, line ),
     >                           svcs( itemp + 1, ine,     0, line ),
     >                           svcs( itemp,     ine + 1, 0, line ),
     >                           svcs( itemp + 1, ine + 1, 0, line )
	   end if

           do i = 1, mp(line)
              prof( i ) = ( 1. - wtne )*( 1. - wttemp )*
     >                    svcs( itemp, ine, i, line ) +
*
     >                    ( 1. - wtne )*wttemp*
     >                    svcs( itemp + 1, ine, i, line ) + 
*
     >                    wtne*( 1. - wttemp )*
     >                    svcs( itemp, ine + 1, i, line ) +
*
     >                    wtne*wttemp*
     >                    svcs( itemp + 1, ine + 1, i, line )
	   end do
	end if

C ***	alpha/lambda interpolation

        f0 = 1.25e-9*max(ne,ne0)**0.666666666666666666e0  ! fix at table limit
	dl = abs( dlam )
        if( dl .ne. 0. ) then
           log_alpha = log10( dl/f0 )
           balp = ( log_alpha - log_alpha0(line) )/log_alpha_inc(line) +
     >            1.
           ialp = balp
           if( ialp .lt. 1 ) then
              vcs = exp10( prof(1) )
           else if( ialp .gt. mp(line) - 1 ) then
              vcs = exp10( prof(mp(line)) +
     >                     2.5*( log_alpha0(line) +
     >                           (mp(line)-1)*log_alpha_inc(line) -
     >                           log_alpha )
     >                   )
!	type *, 'VCS>  Extrapolated'
           else
              wt = balp - ialp
              vcs = exp10( (1. - wt)*prof(ialp) +
     >                     wt*prof(ialp + 1) )
           end if
        else
           vcs = exp10( prof(1) )
        end if

	vcs = vcs/f0
!	vcs = vcs*1.7724539	! normalize like Voigt function (=sqrt(pi))

        end
