      SUBROUTINE FECSC
	
C***********************************************************************
C	 ******************************************************
C	 *						      *
C	 *   READIN, DEFAULT VALUES AND CALCULATION OF	      *
C	 *   MAXIMUM COLLISION CROSS-SECTIONS		      *
C	 *   ELECTRON AND NEUTRAL HYDROGEN COLLISIONS	      *
C	 *   for Iron.					      *
C	 *						      *
C	 *		  8-MAY-1991 17:14:45.18	      *
C	 *						      *
C	 ******************************************************
C ***
C ***	13-MAY-1995 17:33:47.00		ML / IoA.  Include files.
C ***	 6-MAR-1996 22:48:03.75		ML / Bamberg.  Fixed initialization
C ***                                   of MC_C array.
C ***	29-APR-1996 16:46:19.45		ML / Bamberg.  New ion stuff.
C ***                                   We still don't allow for Fe III.
C ***	17-MAY-1996 13:37:38.70		ML / Bamberg.  Get rid of COMMON V
C ***	29-NOV-1999 16:26:34.00		ML / McA.  Format x -> 1x
C ***
C ******************************************************************************
C ***

      INCLUDE 'MOD_SIZ.INC'
      include 'MC_MULTI.INC'
      include 'NNS.INC'
      include 'LEVEL.INC'
      INCLUDE 'COLIS.INC'

      integer ICLOW(M_COLL), ICUP(M_COLL)
      real CCS(m_coll)
*      COMMON V(20000)
*      EQUIVALENCE (V(1),ICLOW(1)), (V(5000),ICUP(1)), (V(10000),CCS(1))
      EQUIVALENCE (FH1BB,FACCEM), (FH1BF,FACCHI)

C*************************************************

C *** OMEGA: HARMONIC AVERAGE OF MEAN SQUARE RADII
C *** FOR DEFAULT CROSS-SECTIONS (FOR EXPERIMENTAL USE ONLY)

      OMEGA(RR1,RR2) = (RR1*RR2)/(RR1 + RR2)*2.
      XP(X) = EXP( X*2.30258509 )

C*************************************************

C *** INITIALIZE COLL ARRAY

      NC = (NL*NK)/2
      DO IC = 1, NC
         COLL(IC) = 0.
      END DO
      do id = 1, ndepth
         DO J = 1, NK
            DO I = 1, NK
               MC_C(I,J,ID) = 0.
            END DO
         END DO
      end do
      IC = 0
      EIH = 13.605

C *** READIN SCALE: SCALING FACTOR FOR DEFAULT EXCITATION CROSS-SECT.

      READ( 1, * ) SCALE
C ***
C ******************************************************************************
C ***
C *** DEFAULT ELECTRON COLLISION CROSS-SECTIONS:
C ***
C *** everything equal to pi*a0^2 * SCALE, assuming optically forbidden 
C *** transition
C ***
C ******************************************************************************
C ***
      DO I = 1, NL
	 DO J = I + 1, NK
	    IC = IC + 1
	    IF( I .LT. NXTION(1) ) THEN
C ***
C ***          FIRST IONIZATION STAGE
C ***
	       IF( J .LT. NXTION(1) ) THEN
C ***
C ***             CALCULATE EXCITATION CROSS SECTIONS
C ***
c	          EIJ = EPS(J) - EPS(I)
C ***             FOR EXPERIMENTAL USE ONLY 
	          COLL(IC) = -1.*SCALE
    	       ELSE IF( J .EQ. NXTION(I) ) THEN
C ***
C ***             CALCULATE IONIZATION CROSS-SECTIONS
C ***
	          EI = CHI_J(0) - EPS(I) + DI(I) + EPS(J)
	          IF( I .EQ. 1 ) THEN
C ***
C ***                FE I GROUND STATE: TWO EQUIVALENT ELECTRONS
C ***
	             XXI = 2.
                  ELSE
	             XXI = 1.
                  END IF
	          COLL(IC) = (EIH/EI)**2*0.665*XXI
	       END IF
            ELSE
C ***
C ***          SECOND IONIZATION STAGE
C ***
    	       IF( J .LT. NXTION(I) ) THEN
C ***
C ***             CALCULATE EXCITATION CROSS SECTIONS
C ***
c	          EIJ = EPS(J) - EPS(I)
C ***             FOR EXPERIMENTAL USE ONLY 
	          COLL(IC) = +1.*SCALE
               ELSE
C ***
C ***             CALCULATE IONIZATION CROSS SECTIONS
C ***
	          EI = CHI_J(1) - EPS(I) + DI(I)
	          XXI = 1.
C ***                        FE II GROUND STATE: ONE EQUIVALENT ELECTRON
	          COLL(IC) = (EIH/EI)**2*0.665*XXI
               END IF
            END IF
         END DO
      END DO

C***********************************************************************
C---READIN OF ELECTRON COLLISION CROSS SECTIONS-------------------------
C***********************************************************************

C*************************************************
C----1. OPTICALLY ALLOWED (LS) TRANSITIONS (LOWER AND UPPER LEVEL), WHICH
C----	ARE NOT READ IN IN THE NEXT SECTION (USING FOR THIS DEFAULT VALUES)
C----	I .GT. 0, J .GT. 0
C*************************************************

      READ( 1, * ) N1
      NCC = N1
      IF( NCC .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), IC = 1, NCC)
      DO IC = 1, NCC
	 I = ICLOW(IC)
	 J = ICUP(IC)
	 ICC = NK*(I - 1) - I*(I-1)/2 + J - I
	 COLL(ICC) = ABS( COLL(ICC) )
	 CCS(IC) = COLL(ICC)
      END DO

C *** READIN OF OVERALL SCALING FACTOR (ELECTRONS)

      READ( 1, * ) FACCEM

C ******************************************************************************
C----2. MAXIMUM CROSS-SECTIONS FOR
C----  A) OPTICALLY ALLOWED TRANSITIONS (LOW AND UP LEVEL, MAX CROSS-
C----	  SECT.) I .GT. 0, J .GT. 0
C----  B) OPTICALLY ALLOWED TRANSITIONS (LOW AND UP LEVEL, LOG GF-VALUE)
C----	  I .GT. 0, J .LT. 0, LOG GF FOR DIPOL APPROXIMATION
C----  C) OPTICALLY DISALLOWED TRANSITIONS (LOW AND UP LEVEL, MAX CROSS-
C----	  SECT.) I .LT. 0, J .GT. 0
C----3. MAXIMUM CROSS-SECTIONS FOR IONIZATION (LOW AND UP LEVEL, MAX
C----	CROSS-SECT.) I .GT. 0, J .GT. 0
C ******************************************************************************

      READ( 1, * ) N2
      NCC = NCC + N2

      IF( N2 .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), CCS(IC), IC =
     >	N1 + 1, NCC)
      DO IC = N1 + 1, NCC
	 I = ABS( ICLOW(IC) )
	 J = ABS( ICUP(IC) )
	 ICC = NK*(I - 1) - (I*(I - 1))/2 + J - I
	 IF( ICUP(IC) .GT. 0 ) THEN
C ***
C ***       MAX CROSS SECTION = READIN (NEGATIVE IF OPTICALLY DISALLOWED)
C ***
	    COLL(ICC) = CCS(IC)*ISIGN( 1, ICLOW(IC) )
	 ELSE
C ***
C ***       DIPOL-APPROXIMATION FOR OPTICALLY ALLOWED TRANSITIONS
C ***
C ***       READIN = LOG GF
C ***       IF COLL .LT. 0.1: USE DEFAULT VALUE
C ***
	    EIJ = EPS(J) - EPS(I)
	    CC = (EIH/EIJ)**2*XP( CCS(IC) )/G(I)
	    IF( CC .LT. 0.1 ) THEN
	       CC = ABS( COLL(ICC) )
	       ICUP(IC) = J
	    END IF
	    COLL(ICC) = CC
	    CCS(IC) = CC
	 END IF
      END DO

C***********************************************************************
C---DEFAULT 'REDUCED' HI-COLLISION CROSS-SECTIONS = ELECTRON CROSS-SECT.
C***********************************************************************

      DO IC = 1, NC
         COLNEU(IC) = ABS( COLL(IC) )
      END DO
C ***
C *** Find minimum and maximum cross section for output
C ***
      CMIN = 1.E99
      CMAX = 0.
      IC = 0
      DO I = 1, NL
	 DO J = I + 1, NK
	    IC = IC + 1
	    IF( (I .GE. NXTION(1)) .OR. (J .LT. NXTION(1)) .OR. 
     >	      (J .EQ. NXTION(I)) ) THEN
	       CC = COLNEU(IC)
	       IF( CC .LT. CMIN ) THEN
	          IMIN = I
	          JMIN = J
	          CMIN = CC
	       END IF
	       IF( CC .GT. CMAX ) THEN
	          IMAX = I
	          JMAX = J
	          CMAX = CC
	       END IF
            END IF
         END DO
      END DO

C***********************************************************************
C---OUTPUT ELECTRON CROSS-SECTIONS--------------------------------------
C***********************************************************************

      WRITE( 6, 2001 ) SCALE, FACCEM
      WRITE( 6, 2002 )
      WRITE( 6, 2003 ) (I, NXTION(I), COLL(NK*(I - 1) - (I*(I - 1))/2 +
     >	NXTION(I) - I), I = 1, NL)
      WRITE( 6, 2004 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = 1, N1)
      WRITE( 6, 2005 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = N1 + 1, NCC)
      WRITE( 6, 2006 ) IMIN, JMIN, CMIN, IMAX, JMAX, CMAX

C***********************************************************************
C---H I	 - COLLISION CROSS-SECTIONS-------------------------------------
C***********************************************************************

C---READ OVERALL SCALING FACTOR FOR H I

      READ( 1, * ) FACCHI

C***********************************************************************
C---READIN OF HI-COLLISION CROSS-SECTIONS-------------------------------
C***********************************************************************

C*************************************************
C----4. MAXIMUM 'REDUCED' CROSS-SECTIONS (LOW AND UP LEVEL, MAX CROSS-SECT
C----	I .GT. 0, J .GT. 0
C*************************************************
      READ( 1, * ) NCC
      IF( NCC .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), CCS(IC), IC =
     >	1, NCC)

C---INSERT CROSS-SECTIONS IN COLNEU ARRAY-----------

      DO IC = 1, NCC
	 I = ICLOW(IC)
	 J = ICUP(IC)
	 ICC = NK*(I - 1) - (I*(I - 1))/2 + J - I
	 COLNEU(ICC) = CCS(IC)
      END DO

C***********************************************************************
C---OUTPUT HI COLLISION CROSS-SECTIONS----------------------------------
C***********************************************************************

      WRITE( 6, 2007 ) FACCHI
      WRITE( 6, 2008 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = 1, NCC)


C ***
C *** Addition for Mats Carlsson's et al. package
C ***
      READ( 1, *, IOSTAT = IERR ) J
C ***             ^^^^^^ check if keywords are present; could be only numerical
C ***                    for some opacity scaling (addon by W. Steenbock)
      IF( IERR .GT. 0 ) THEN
C ***
C ***    MULTI keywords detected (hopefully)
C ***
         BACKSPACE 1
         CALL GENCOL
      ELSE IF( IERR .EQ. 0 ) THEN
C ***
C ***    these addons are present, so:
C ***
         BACKSPACE 1
      END IF
       

C***********************************************************************

 2001 FORMAT( //, ' ', 45('*'), 'ELECTRON COLLISION CROSS SECTIONS', 
     >	45('*'), ///, ' SCALING FACTOR FOR DEFAULT EXCITATION ',
     >	'CROSS-SECTIONS =', 1P, E10.2, /, ' OVERALL SCALING FACTOR =',
     >	1P, E10.2 )
 2002 FORMAT( /, 31X, 15('*'), 'IONISATION CROSS-SECTIONS', 15('*') )
 2003 FORMAT(:, /, 6('   I   J  COLL(I,J)  '), /, ( 6(2I4, 1P, G11.4E1,
     >	2X) ) )
 2004 FORMAT(//, 31X, 15('*'), 'EXCITATION CROSS SECTIONS', 15('*'), //,
     1  ' OPTICALLY ALLOWED WITH DEFAULT VALUES')
 2005 FORMAT( //, ' ALL OTHER CROSS-SECTIONS = DEFAULT, EXCEPT FOR ',
     >  'THE FOLLOWING:' )
 2006 FORMAT( /, ' MINIMUM CROSS-SECT.:  IMIN = ', I4, '  JMIN = ', I4,
     >	'  COLL = ', 1P, G11.4E1, /, ' MAXIMUM CROSS-SECT.:  IMAX = ',
     >	I4, '  JMAX = ', I4, '  COLL = ', 1P, G11.4E1 )
 2007 FORMAT( //, 1X, 45('*'), 'HI COLLISION CROSS SECTIONS', 45('*'),
     >	///, ' OVERALL SCALING FACTOR = ', 1P, E10.2 )
 2008 FORMAT( //, ' ALL CROSS SECTIONS = COLL. EL., EXCEPT FOR THE ',
     >	'FOLLOWING:' )
      END

