	function gam4def( z, ionpot, chik, dchik )
C ***
C ******************************************************************************
C ***
C ***	Compute approximation of gamma_4 (1e8/per electron) according to Griem
C ***	and Cowley.
C ***
C ***	z	> i	Ion charge + 1, i.e., z=1 for neutrals
C ***	ionpot	> r	Ionization potential [eV]
C ***	chik	> r	Excitation energy of level [eV]
C ***	dchik	> r	Excitation energy of parent term [eV]
C ***
C ***	gam4def < r	log C4	Stark broadening parameter
C ***
C ***
C ***	Modifications:
C ***
C ***	12-MAY-1995 15:55:56.00		ML / IoA.  Created.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	real ionpot, chik, dchik
	integer z
	real gam4def

C ***
C ***	Local variables
C ***
	real neff2	! n_eff^2

C ***
C ***	Code
C ***
	neff2 = z*z*13.598/( ionpot + dchik - chik )

	if( z .eq. 1 ) then
	   gam4def = 2.259e-7*4
	else
	   gam4def = 5.421e-7/z/z*4
	end if

	gam4def = gam4def*neff2*neff2*1.e-8


	end
                                 
