      SUBROUTINE GENCLRT( IID, C, ICOL )
C ***
C ******************************************************************************
C ***
C *** Computes collision frequencies. General version.
C ***
C ***
C *** Michael Lemke
C ***
C *** 31-MAY-1991 13:07:39.67
C ***
C *** Modifications:
C ***
C ***	20-JAN-1993 11:02:07.99		ML / UT Austin.  Removed silly IF block.
C ***	20-JAN-1993 15:15:07.99		ML / UT Austin.  Added provision for
C ***		                      multiple ionization rates from same level.
C ***	13-MAY-1995 17:35:17.00		ML / IoA.  Include files, zetah
C ***	29-APR-1996 16:50:02.73		ML / Bamberg.  New ion stuff.
C ***                                   Not sure how much of that is valid for
C ***                                   multiple ions.  Drawin's a mess.
C ***	 3-MAY-1996 20:49:55.42		ML / Bamberg.  Read lower label only
C ***                                   for neutrals, just like upper level.
C ***	17-MAY-1996 14:02:52.90		ML / Bamberg.  ICOL
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer iid, icol
	real c(icol,icol)

C ***
C ***	Global variables
C ***
        INCLUDE 'MOD_SIZ.INC'
        INCLUDE 'MC_MULTI.INC'
        include 'NNS.INC'
        include 'MODEL.INC'
        INCLUDE 'POP.INC'
        include 'LEVEL.INC'
        include 'LINE.INC'
        INCLUDE 'COLIS.INC'

C ***
C ***	Local variables
C ***
      	INTEGER I, IC, ID, IS, J, JS, stage_i, stage_j

      	REAL ACON, BCON, BE, DUM, F2, F3, NE, NHI0, T, TEV, W, X
        REAL UIJ(26), PSI2(26), PSI3(26)
        CHARACTER INTERN*4

        REAL FACCEM, FACCHI
        EQUIVALENCE (FH1BB,FACCEM), (FH1BF,FACCHI)

C ***
C ***	Functions
C ***
	real xp, f00, f0, f1, psi
	integer stage, map1
	real expint

	external map1, expint, stage


      DATA UIJ/ 1.E-5, 1.E-4, 1.E-3, 1.E-2, 1.E-1, 4.E-1, 1., 1.5, 2.,
     >  3., 4., 6., 8., 10., 12., 14., 16., 18., 20., 22., 24., 26.,
     >  28., 30., 40., 50. /
      DATA PSI2/ 1.116E+1, 8.852, 6.532, 4.164, 1.790, 7.285E-1,
     >  2.210E-1, 1.098E-1, 6.130E-2, 2.068E-2, 6.914E-3, 8.733E-4,
     >  1.139E-4, 1.508E-5, 2.010E-6, 2.691E-7, 3.611E-8, 4.855E-9,
     >  6.534E-10, 8.807E-11, 1.188E-11, 1.602E-12, 2.163E-13,
     >  2.920E-14, 1.315E-18, 5.941E-23 /
      DATA PSI3/ 6.6655E-6, 6.6575E-5, 6.5983E-4, 6.2129E-3, 4.348E-3,
     >  8.0092E-2, 6.2433E-2, 4.064E-2, 2.5022E-2, 8.9304E-3, 3.0993E-3,
     >  3.6726E-4, 4.372E-5, 5.2614E-6, 6.3984E-7, 7.8568E-8, 9.7264E-9,
     >  1.2126E-9, 1.521E-10, 1.9177E-11, 2.4288E-12, 3.0888E-13,
     >  3.9424E-14, 5.049E-15, 6.76E-24, 6.11E-41 /

      XP( X ) = EXP( X*2.30258E0 )
      F00( W ) = 4.*W*EXP( -W )
      F0( W ) = 4.*W*( EXP( -W ) - W*EXPINT( W ) )
      F1( W, BE ) = 4.*W/(1. + W)*EXP( -W )*( 1./(20. + W) + 
     >              ALOG( (1. + W)/W*1.25*BE) )
      PSI( W ) = (1. + 2./W)/( 1. + (1.08808E-3/W)**2 )*EXP( -W )

      ID = IID
      T = TEMP(ID)
      TEV = 8.6171E-5*T
      NE = XP( PELG(ID) )/T/1.38062E-16
      NHI0 = 2.*RHO(ID)/AVEMAS*ZETAH(ID)
      ACON = NE*5.8875E-9*SQRT( TEV )*FACCEM
      BCON = NHI0*5.93095E-13*SQRT( TEV*WEI*(WEI + 1.008) )*FACCHI
      DO J = 1, NK
         DO I = 1, NK
            C(I,J) = 0.
         END DO
      END DO
      IC = 0
      DO I = 1, NL
         STAGE_I = stage(i)
         if( stage_i .eq. 0 ) then
            WRITE( INTERN, '( A4 )' ) LABEL(I)
            READ( INTERN, '( 2X, I1 )' ) IS
         end if
         DO J = I + 1, NK
            IC = IC + 1
            stage_j = stage(j)
            IF( stage_i .eq. 0 ) THEN ! I = neutral level
C ***       
C ***          Neutrals
C ***       
               IF( stage_j .eq. 0 ) THEN ! J = neutral level
C ***       
C ***             B-B transitions
C ***       
                  W = (EPS(J) - EPS(I))/TEV
                  IF( COLL(IC) .GE. 0. ) THEN
C ***       
C ***                OPTICALLY ALLOWED
C ***       
                     C(I,J) = BCON*COLNEU(IC)*PSI( W ) +
     >                 ACON*COLL(IC)*F1( W, 1. )*0.8278
                  ELSE
C ***       
C ***                OPTICALLY DISALLOWED
C ***       
                     WRITE( INTERN, '( A4 )' ) LABEL(J)
                     READ( INTERN, '( 2X, I1 )' ) JS
                     IF( IS .EQ. JS ) THEN
C ***       
C ***                   DELTA S = 0
C ***       
                        C(I,J) = BCON*COLNEU(IC)*PSI( W ) +
     >                    ACON*ABS( COLL(IC) )*F0( W )
                     ELSE IF( I .NE. 1 ) THEN
C ***       
C ***                   DELTA S .NE. 0, excited states
C ***       
                        C(I,J) = BCON*COLNEU(IC)*PSI( W ) +
     >                    ACON*ABS( COLL(IC) )*F00( W )
                     ELSE
C ***       
C ***                   DELTA S .NE. 0, FROM GROUND STATE
C ***       
                        DUM = MAP1( UIJ, PSI3, 26, W, F3, 1 )
                        C(I,J) = BCON*COLNEU(IC)*PSI( W ) +
     >                    ACON*ABS( COLL(IC) )*W*F3*5.38
                     END IF
                  END IF
               ELSE IF( stage_j .eq. 1 .and. J .GE. NXTION(I) ) THEN
                                                ! J = parent term of I or
                                                !     higher term
C ***       
C ***             B-F transitions
C ***       
                  W = (CHI_J(0) - EPS(I) + EPS(J) + DI(I))/TEV
                  C(I,J) = BCON*COLNEU(IC)*PSI( W ) + 
     >              ACON*ABS( COLL(IC) )*F1( W, 1. )
               END IF
            ELSE 
C ***       
C ***          Ions
C ***
	       IF( stage_j .eq. stage_i .and. J .LT. NXTION(I) ) THEN
                                                 ! J != parent term of I
C ***
C ***             B-B transitions
C ***             
                  W = (EPS(J) - EPS(I))/TEV
                  IF( COLL(IC) .GE. 0. ) THEN
                     DUM = MAP1( UIJ, PSI2, 26, W, F2, 1 )
                     C(I,J) = BCON*COLNEU(IC)*PSI( W ) + 
     >                 ACON*ABS( COLL(IC) )*3.31*W*F2
                  ELSE
C ***             
C ***                CROSS-SECTIONS (OMEGA) FROM NUSSBAUMER
C ***                ASSUME OMEGA = CONST
C ***             
                     C(I,J) = BCON*COLNEU(IC)*PSI( W ) + 
     >                 ACON*ABS( COLL(IC) )*F00( W )
                  END IF
               ELSE if( stage_j .eq. stage_i + 1 .and.
     >                  j .ge. nxtion(i) ) then
                                          ! J = parent term of I or higher term
C ***       
C ***             B-F transitions
C ***             
                  W = (CHI_J(stage_i) - EPS(I) + DI(I))/TEV
                  C(I,J) = BCON*COLNEU(IC)*PSI( W ) + 
     >              ACON*ABS( COLL(IC) )*F1( W, 1.25 )
	       END IF
            END IF
            C(J,I) = C(I,J)*N(ID,I)/N(ID,J)
         END DO
      END DO
      DO J = 1, NK
         DO I = 1, NK
            C(I,J) = C(I,J) + MC_C(I,J,ID)
         END DO
      END DO
      END
