      SUBROUTINE GENCSC
C ***
C ******************************************************************************
C ***
C *** General collision routine, default and readin section.
C ***
C *** Pinched from Fe routine and corresponding section from MULTI
C *** (M. Carlsson).
C ***
C *** M. Lemke, 15-MAY-1991 14:09:08.03
C ***
C *** Modifications:
C ***
C *** 20-JAN-1993 15:00:13.99   ML / UT Austin.  Add provision for multiple
C ***                           ionization collision rates from same level.
C *** 13-FEB-1993 22:00:15.95   ML / UT Austin.  Changed lower limit of
C ***                           x-section computed from f-value to 0.1 instead
C ***                           of silly `default' value
C *** 15-FEB-1993 01:32:40.13   ML / UT Austin.  Removed limit completely.
C *** 13-MAY-1995 17:36:57.00	ML / IoA.  Include files, zetah
C ***  7-MAR-1996 17:25:31.56	ML / Bamberg.  Moved initialization of MC_C
C ***                           to here.  Got done only when Uppsala x-sect.s
C ***                           included but MC_C is added to C anyway.
C *** 29-APR-1996 19:31:57.22	ML / Bamberg.  New ion stuff.
C *** 17-MAY-1996 13:44:44.80	ML / Bamberg.  Removed common V et al.
C *** 29-NOV-1999 16:28:12.00	ML / McA.  Format x -> 1x
C ***
C ******************************************************************************
C ***
      implicit none

      INCLUDE 'MOD_SIZ.INC'
      include 'MC_MULTI.INC'
      include 'NNS.INC'
      include 'LEVEL.INC'
      INCLUDE 'COLIS.INC'

C ***
C *** Local variables
C ***

      INTEGER ICLOW(M_COLL), ICUP(M_COLL)
      REAL CCS(M_COLL)

      INTEGER I, IC, ICC, ID, IERR, IMAX, IMIN, J, JMAX, JMIN, N1, N2,
     >        NC, NCC

      REAL CC, CMAX, CMIN, EI, EIH, SCALE, X, XXI, faccem, facchi

C ***
C *** Functions
C ***
      real xp
      integer stage
      external stage


      EQUIVALENCE (FH1BB,FACCEM), (FH1BF,FACCHI)

      XP(X) = EXP( X*2.30258509 )


C ***
C *** Code
C ***

C *** initialize COLL arrays

      NC = (NL*NK)/2
      DO IC = 1, NC
         COLL(IC) = 0.
      END DO
      do id = 1, ndepth
         DO J = 1, NK
            DO I = 1, NK
               MC_C(I,J,ID) = 0.
            END DO
         END DO
      end do
      IC = 0
      EIH = 13.605

C ***
C ******************************************************************************
C ***
C *** Default electron collision cross-sections:
C ***
C *** set everything equal to pi*a0^2 * SCALE, assuming optically forbidden 
C *** transitions
C ***
C ******************************************************************************
C ***
      READ( 1, * ) SCALE
      DO I = 1, NL
         DO J = I + 1, NK
            IC = IC + 1
            IF( stage(i) .eq. 0 ) THEN
C ***
C ***          Neutrals
C ***
               IF( stage(J) .eq. 0 ) THEN
C ***
C ***             calculate excitation cross sections
C ***
                  COLL(IC) = -1.*SCALE
               ELSE IF( J .EQ. NXTION(I) ) THEN
C ***
C ***             calculate ionization cross-sections
C ***
                  EI = CHI_J(0) - EPS(I) + DI(I) + EPS(J)
                  IF( I .EQ. 1 ) THEN
C ***
C ***                Fe I ground state: two equivalent electrons
C ***
                     XXI = 2.
                  ELSE
                     XXI = 1.
                  END IF
                  COLL(IC) = (EIH/EI)**2*0.665*XXI
               END IF
            ELSE
C ***
C ***          Ions
C ***
               IF( stage(J) .eq. stage(i) ) THEN
C ***
C ***             calculate excitation cross sections
C ***
                  COLL(IC) = +1.*SCALE
               ELSE if( j .eq. nxtion(i) ) then
C ***
C ***             calculate ionization cross sections
C ***
                  EI = CHI_J(stage(i)) - EPS(I) + DI(I)
                  XXI = 1.
C ***                   ^: Fe II ground state: one equivalent electron
C ***                      don't know about other elements.
                  COLL(IC) = (EIH/EI)**2*0.665*XXI
               END IF
            END IF
         END DO
      END DO
C ***
C ******************************************************************************
C ***
C *** Readin cross sections:
C ***
C ***   a. optically allowed (LS) transitions for which default values are to 
C ***      be used. All other transitions (i.e. those not specified later)
C ***      default to forbidden transitions.
C ***
C *** Read I and J.
C ***
C ******************************************************************************
C ***
      READ( 1, * ) N1
      NCC = N1
      IF( NCC .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), IC = 1, NCC)
      DO IC = 1, NCC
         I = ICLOW(IC)
         J = ICUP(IC)
         ICC = NK*(I - 1) - (I*(I-1))/2 + J - I
         COLL(ICC) = ABS( COLL(ICC) )
         CCS(IC) = COLL(ICC)
      END DO

C *** FACCEM: global scling factor for all electron collision cross sections

      READ( 1, * ) FACCEM
C ***
C ******************************************************************************
C ***
C *** b. individual b-b cross sections:
C ***
C ***      i) optically allowed transitions with given maximum X section
C ***         (data: I > 0, J > 0, X section)
C ***
C ***     ii) same but using dipole approximation
C ***         (data: I > 0, J < 0, log gf)
C ***
C ***    iii) optically disallowed transitions with given maximum X section
C ***         (data: I < 0, J > 0, X section)
C ***
C *** c. b-f cross sections
C ***         (data: I > 0, J > 0, X section)
C ***
C ******************************************************************************

      READ( 1, * ) N2
      NCC = NCC + N2

      IF( N2 .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), CCS(IC), IC =
     >  N1 + 1, NCC)
      DO IC = N1 + 1, NCC
         I = ABS( ICLOW(IC) )
         J = ABS( ICUP(IC) )
         ICC = NK*(I - 1) - (I*(I - 1))/2 + J - I
         IF( ICUP(IC) .GT. 0 ) THEN
C ***
C ***       Max. X section given (ICLOW negative if forbidden)
C ***
            COLL(ICC) = CCS(IC)*ISIGN( 1, ICLOW(IC) )
         ELSE
C ***
C ***       Dipole-approximation (log gf value given)
C ***
C ***       If resulting X section < 0.1 set to 0.1
C ***
            CC = ( EIH/(EPS(J) - EPS(I)) )**2*XP( CCS(IC) )/G(I)
**            IF( CC .LT. 0.1 ) THEN
*               CC = ABS( COLL(ICC) )
**               CC =  0.1
**               ICUP(IC) = J
**            END IF
            COLL(ICC) = CC
            CCS(IC) = CC
         END IF
      END DO
C ***
C ******************************************************************************
C ***
C *** Output electron cross sections
C ***
C *** First, find minimum and maximum cross section
C ***
C ***
C ******************************************************************************
C ***
      CMIN = 1.E99
      CMAX = -CMIN
      IC = 0
      DO I = 1, NL
         DO J = I + 1, NK
            IC = IC + 1
            IF( stage(i) .eq. stage(j) .or.
     >          j .eq. nxtion(i) ) then
               CC = ABS( COLL(IC) )
               IF( CC .GT. 0. ) THEN
                  IF( CC .LT. CMIN ) THEN
                     IMIN = I
                     JMIN = J
                     CMIN = CC
                  END IF
                  IF( CC .GT. CMAX ) THEN
                     IMAX = I
                     JMAX = J
                     CMAX = CC
                  END IF
               END IF
            END IF
         END DO
      END DO
C ***
C *** Print the stuff
C ***
      WRITE( 6, 2001 ) SCALE, FACCEM
      WRITE( 6, 2002 )
      WRITE( 6, 2003 ) (I, NXTION(I), COLL(NK*(I - 1) - (I*(I - 1))/2 +
     >  NXTION(I) - I), I = 1, NL)
      WRITE( 6, 2004 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = 1, N1)
      WRITE( 6, 2005 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = N1 + 1, NCC)
      WRITE( 6, 2006 ) IMIN, JMIN, CMIN, IMAX, JMAX, CMAX

C *** FACCHI: Global scaling factor for H I collision X sections

      READ( 1, * ) FACCHI
C ***
C *** H I collision cross sections:
C *** 
C ***    Default them to e- X sections first
C *** 
      DO IC = 1, NC
         COLNEU(IC) = ABS( COLL(IC) )
      END DO
C *** 
C *** Try to read some (if there are any, good luck!)
C *** 
C *** data: (I > 0, J > 0, X section)
C *** 
      READ( 1, * ) NCC
      IF( NCC .GT. 0 ) READ( 1, * ) (ICLOW(IC), ICUP(IC), CCS(IC),
     >  IC = 1, NCC)
      
      DO IC = 1, NCC
         I = ICLOW(IC)
         J = ICUP(IC)
         ICC = NK*(I - 1) - (I*(I - 1))/2 + J - I
         COLNEU(ICC) = CCS(IC)
      END DO
C *** 
C *** Print the H I X sections
C *** 
      WRITE( 6, 2007 ) FACCHI
      WRITE( 6, 2008 )
      WRITE( 6, 2003 ) (ICLOW(IC), ICUP(IC), CCS(IC), IC = 1, NCC)

C ***
C *** Addition for Mats Carlsson's et al. package
C ***
      READ( 1, *, IOSTAT = IERR ) J
C ***             ^^^^^^ check if keywords are present; could be only numerical
C ***                    for some opacity scaling (addon by W. Steenbock)
      IF( IERR .GT. 0 ) THEN
C ***
C ***    MULTI keywords detected (hopefully)
C ***
         BACKSPACE 1
         CALL GENCOL
      ELSE IF( IERR .EQ. 0 ) THEN
C ***
C ***    these addons are present, so:
C ***
         BACKSPACE 1
      END IF
       

 2001 FORMAT( //, ' ', 45('*'), 'ELECTRON COLLISION CROSS SECTIONS', 
     >  45('*'), ///, ' SCALING FACTOR FOR DEFAULT EXCITATION ',
     >  'CROSS-SECTIONS =', 1P, E10.2, /, ' OVERALL SCALING FACTOR =',
     >  1P, E10.2 )
 2002 FORMAT( /, 31X, 15('*'), 'IONIZATION CROSS-SECTIONS', 15('*') )
 2003 FORMAT(:, /, 6('   I   J  COLL(I,J)  '), /, ( 6(2I4, 1P, G11.4E1,
     >  2X) ) )
 2004 FORMAT(//, 31X, 15('*'), 'EXCITATION CROSS SECTIONS', 15('*'), //,
     1  ' OPTICALLY ALLOWED WITH DEFAULT VALUES')
 2005 FORMAT( //, ' ALL OTHER CROSS-SECTIONS = DEFAULT, EXCEPT FOR ',
     >  'THE FOLLOWING:' )
 2006 FORMAT( /, ' MINIMUM CROSS-SECT.:  IMIN = ', I4, '  JMIN = ', I4,
     >  '  COLL = ', 1P, G11.4E1, /, ' MAXIMUM CROSS-SECT.:  IMAX = ',
     >  I4, '  JMAX = ', I4, '  COLL = ', 1P, G11.4E1 )
 2007 FORMAT( //, 1X, 45('*'), 'HI COLLISION CROSS SECTIONS', 45('*'),
     >  ///, ' OVERALL SCALING FACTOR = ', 1P, E10.2 )
 2008 FORMAT( //, ' ALL CROSS SECTIONS = COLL. EL., EXCEPT FOR THE ',
     >  'FOLLOWING:' )
      END
      SUBROUTINE GENCOL
C
C  GENERAL ROUTINE FOR COMPUTING COLLISIONAL RATES (VERSION MAR '89)
C  FORMAT IS EITHER GIVEN IN TERMS OF A COLLISION STRENGTH OMEGA
C         OR                          A CE(TE) RATE
C
C  OMEGAS ARE USED FOR CHARGED IONS IN GENERAL, 
C  SINCE THEY REMAIN ROUGHLY CONSTANT WITH TEMPERATURE.  
C  RATE IS PROP. TO 1/SQRT(TE) * EXP(DELTA-E)
C
C  CE(TE) VALUES ARE USED LARGELY FOR NEUTRALS FOR SAME REASON.
C  RATE IS PROP. TO SQRT(TE) * EXP(DELTA-E)
C
C  CI(TE) VALUES ARE USED FOR IONIZATION RATES: THESE HAVE A
C         SQRT(TE) * EXP(DELTA-E) DEPENDENCE AS FOR CE(TE)
C
C  ORIGINALLY CODED BY P.G. JUDGE,  APRIL 2ND, 1987
C  MODIFIED FOR FORTRAN-77 STANDARD BY P.G. JUDGE, MAY 21ST 1987
C:
C: GENCOL 87-05-21  NEW ROUTINE: (PHILIP JUDGE)
C:        GENERAL COLLISONAL ROUTINE READING TABLES
C:
C:        89-03-24  MODFICATIONS: (PHILIP JUDGE)
C:        NEW INPUT 'CP' AND 'CH' ADDED FOR INELASTIC COLLISIONS
C:        WITH NEUTRAL AND IONIZED HYDROGEN.
C:        'SEMI ' CHANGED SO THAT THE ABSORPTION OSCILLATOR STRENGTH 
C:        IS REQUIRED FOR INPUT.
C:
C:        89-06-07  MODIFICATIONS: (MATS CARLSSON)
C:        DO LOOP VARIABLE IT CHANGED TO JT IN ORDER NOT TO CONFLICT
C:        WITH THE MAIN ITERATION VARIABLE IT IN ITER
C ***
C *** 14-MAY-1991 14:14:39.58: Modified to fit into Kiel package
C ***                          Michael Lemke
C ***
C *** 30-MAY-1991 15:24:54.99: Keywords CEH, OHMH, SEMIH, and CIH added to
C ***                          account for Hydrogen collisions. Derives H I rate
C ***                          from e- rate assuming Drawin formulae #29, 33,
C ***                          and 42 for e- cross section (i.e. opt. allowed
C ***                          transition) and Steenbock formula for H I
C ***                          X-sect. Max. H I X-sect. assumed equal to max.
C ***                          e- X-sect:
C ***
C ***                          C(H I) = C(e-)*BCON*PSI(W)/[ACON*F1(W,1)*0.8278]
C ***
C ***                          with ACON, BCON, PSI and F1 defined as in
C ***                          routine GENCLRT.
C ***
C ***                          Also some CHARACTER nonsense with variable KEY
C ***                          corrected.
C ***
C ***                          Michael Lemke
C ***
C *** 20-FEB-1993 00:59:40.36  ML / UT Austin.  Added Auer&Mihalas formulae #9
C ***                     & 13 as keywords AUMI9 and AUMI13 (ApJ 184,167, 1973)
C ***
C *** 23-FEB-1993 05:13:37.14  ML / UT Austin.  Added Mihals formula 5-79
C ***                     2nd ed. for collisional ionization.  Keyword MI5-79.
C ***  3-FEB-1995 19:08:07.00  ML / IoA.  Added GAM for Iron Project data.
C ***			  Is exactly like OHM except for Boltzmann factor.
C *** 16-FEB-1995 13:57:59.00  ML / IoA.  GAM is not exactly like OHM.  It's
C ***		          the upward rate!
C *** 29-APR-1996 20:14:56.17  ML / Bamberg.  New ion stuff.
C ***
C *** 30-APR-1996 13:08:45.80  ML / Bamberg.  More on new ion stuff.
C ***
C
      INCLUDE 'MOD_SIZ.INC'
      INCLUDE 'MC_MULTI.INC'
      PARAMETER( LOUT = 6, LJOBLO = LOUT, LDUMS = 1 )

      REAL NH, NHII, NE

      include 'NNS.INC'
      include 'LEVEL.INC'
      include 'MODEL.INC'
      INCLUDE 'COLIS.INC'
      EQUIVALENCE (FH1BB,FACCEM), (FH1BF,FACCHI)
      INCLUDE 'POP.INC'
C
      PARAMETER (MTGRD=20)
C
C  LOCAL VARIABLES: UP TO MTGRD POINTS IN TEMPERATURE GRID ALLOWED
C  THIS CAN BE INCREASED
C
      DIMENSION TGRID(MTGRD),CGRID(MTGRD)
      CHARACTER *20 KEY

      integer stage
C
      XP( X ) = EXP( 2.30258*X )
C ***
C *** some constants
C ***
      BK = 1.38062E-16
      CC = 2.99792458E10
      EE = 1.602189E-12
      HH = 6.6261276E-27
      EK = EE/BK
C
C  OUTPUT FOR IWATOM .GT. 1
C
C ***
C *** IWATOM set to 2. Maybe it gets changed when a better concept for
C *** output of collision data has been found.
C ***
      IWATOM = 2
      IF(IWATOM .GT. 1)WRITE(LOUT,800)
  800   FORMAT('1COLLISION RATES FROM SUBROUTINE  G E N C O L'//
     *  1X,'PARAMETERS ARE AS FOLLOWS:'//
     *  1X,'TEMP  -  TEMPERATURE GRID GIVEN IN FILE ATOM'/
     *  1X,'         (SPLINE INTEPOLATIONS ARE MADE)'//
     *  1X,'CE    -  NEUTRAL B-B COLLISION RATE WHERE'/
     *  1X,'         R(U-L) = CE * NE * G(L) * SQRT(TEMP) / G(U)'//
     *  1X,'CEH   -  SAME AS CE BUT H I RATE ADDED GUESSED FROM CE'/
     *  1X,'         R(U-L) = R(U-L)_CE + R(U-L)_CE * ',
     >               'BCON*PSI(W)/[ACON*F1(W,BETA)]'//
     *  1X,'GAM   -  ION B-B COLLISION RATE WHERE'/
     *  1X,'         R(L-U) = GAM * NE * 8.63E-6 / SQRT(TEMP) / G(L)'/
     *  1X,'         * EXP(-DELTAE/KT)'//
     *  1X,'OHM   -  ION B-B COLLISION RATE WHERE'/
     *  1X,'         R(U-L) = OHM * NE * 8.63E-6 / SQRT(TEMP) / G(U)'//
     *  1X,'OHMH  -  SAME AS OHM BUT H I RATE ADDED GUESSED FROM OHM'/
     *  1X,'         R(U-L) = R(U-L)_OHM + R(U-L)_OHM * ',
     >               'BCON*PSI(W)/[ACON*F1(W,BETA)]'//
     *  1X,'SEMI  -  B-B COLLISIONAL RATE USING THE SEMI-EMPIRICAL'/
     *  1X,'         FORMULAE OF VAN REGEMORTER AND SHEVELKO'//
     *  1X,'SEMIH -  SAME AS SEMI BUT H I RATE ADDED GUESSED FROM SEMI'/
     *  1X,'         R(U-L) = R(U-L)_SEMI + R(U-L)_SEMI * ',
     >               'BCON*PSI(W)/[ACON*F1(W,BETA)]'//
     *  1X,'AUMI9 -  B-B COLLISIONAL RATE USING THE SEMI-EMPIRICAL'/
     *  1X,'         FORMULAE OF VAN REGEMORTER AS DEFINED IN'/
     >  1X,'         AUER & MIHALAS FORMULA #9, ApJ 184,167,1973',/
     >  1X,'         R(L-U) = 5.465E-11 * NE * SQRT(T) * 14.5 * Fij *',
     >               '(13.6/EIJ)^2 * W * EXP(-W) * GAMMA(W)',//
     *  1X,'AUMI13-  B-B COLLISIONAL RATE FOR OPTICALLY FORBIDDEN'/
     *  1X,'         TRANSITIONS ASSUMING OMEGA = CONST. AS IN'/
     >  1X,'         AUER & MIHALAS FORMULA #13, ApJ 184,167,1973',/
     >  1X,'         R(L-U) = 5.465E-11 * NE * SQRT(T) * OMEGA/G_L *',
     >               '(13.6/EIJ) * W * ( EXP(-W) - W*E1(W) )',//
     *  1X,'CP    -  B-B COLLISIONAL RATE WITH PROTONS WHERE'/
     *  1X,'         R(U-L) = CP * NP '//
     *  1X,'CH    -  B-B COLLISIONAL RATE WITH NEUTRAL HYDROGEN WHERE'/
     *  1X,'         R(U-L) = CH * N(H I) '//
     *  1X,'CI    -  B-F IONIZATION RATE WHERE'/
     *  1X,'         R(L-U) = CI * NE * EXP(-DELTAE/KT) * SQRT(T)'//
     *  1X,'CIH   -  SAME AS CI BU H I RATE ADDED GUESSED FROM CI'/
     *  1X,'         R(L-U) = R(L-U)_CI + R(L-U)_CI * ',
     >               'BCON*PSI(W)/[ACON*F1(W,BETA)]'//
     *  1X,'MI5-79-  B-F IONIZATION RATE USING FORMULA #5-79 OF'/
     >  1X,'         MIHALAS 2ND ED., P. 134',/
     >  1x,'         R(L-U) = 1.55e-13 * NE * SQRT(T) * GAUNT * ',
     >               'SIGMA_0 * EXP(-W)/W'//
     *  1X,'***********************************************'/
     *  1X,'*** ALL THE ABOVE HAVE REVERSE RATES IN LTE ***'/
     *  1X,'*** THE FOLLOWING HAVE ZERO REVERSE RATES   ***'/
     *  1X,'*** UNLESS SPECIFIED EXPLICITLY             ***'/
     *  1X,'***********************************************'//
     *  1X,'CH0   -  CHARGE TRANSFER RATE WITH NEUTRAL HYDROGEN'/
     *  1X,'         R(I-J) = CH0 * NH(LEVEL1)'//
     *  1X,'CH+   -  CHARGE TRANSFER RATE WITH IONIZED HYDROGEN'/
     *  1X,'         R(I-J) = CH+ * N(H II)'///
     *  1X,'KEY ','FROM  TO  PARAMETERS... '/)
C
C  ASSUME INITIALLY THAT DATA ARE GIVEN INDEPENDENT OF TEMPERATURE:
C  I.E. THAT NTEMP IS ORIGINALLY SET = 1
C ***
C *** How is TGRID defined in that case? M.L.
C
      NTEMP=1

      DO K = 1, NDEPTH
         DO J = 1, NK
            DO I = 1, NK
               MC_C( I, J, K ) = 0.
            END DO
         END DO
      END DO
C
C  READ THE KEYWORD 'KEY' AND ASSOCIATED PARAMETERS
C
      DO 1 I=1,100000
        READ(LDUMS,100,END=301)KEY
  100   FORMAT(A)
        CALL LJUST(KEY)
        IF (KEY .EQ. 'END')THEN
          GOTO 300
        ELSE IF (KEY .EQ. 'TEMP')THEN
          READ(LDUMS,*)NTEMP,(TGRID(JT),JT=1,MIN(NTEMP,MTGRD))
          IF(NTEMP .GT. MTGRD)THEN
            WRITE( 6, * ) ' GENCOL: WORK ARRAYS (TGRID) TOO SMALL'
            CALL EXIT( 1 )
          ENDIF
          IF (IWATOM .GT. 1)THEN
            WRITE(LOUT,803) 'TEMP',(TGRID(JT),JT=1,NTEMP)
  803       FORMAT(1X,A,8X,10(1P,1X,E10.3))
          ENDIF
          GOTO 1
        ELSE IF( KEY .EQ. 'SEMI' .OR. 
     >           KEY .EQ. 'SEMIH' .OR.
     >           KEY .EQ. 'AUMI9' .OR.           ! read log gf
     >           KEY .EQ. 'AUMI13' .OR.          ! read Omega
     >           KEY .EQ. 'MI5-79' ) THEN        ! read a(nu_0) in 1e-18 cm^2
          READ(LDUMS, *) IL,IH, FAB
          IF( IWATOM .GT. 1 ) WRITE( LOUT, 8004 ) KEY(:IDX(KEY)), IL,
     >        IH, FAB
 8004     FORMAT(1X,A,2(2X,I2),1x, 1p, e10.3)
        ELSE IF(
     >           KEY .EQ. 'GAM'
     >  .OR.     KEY .EQ. 'OHM'
     *  .OR.     KEY .EQ. 'OHMH'
     *  .OR.     KEY .EQ. 'CE'
     *  .OR.     KEY .EQ. 'CEH'
     *  .OR.     KEY .EQ. 'CP'
     *  .OR.     KEY .EQ. 'CH'
     *  .OR.     KEY .EQ. 'CH0'
     *  .OR.     KEY .EQ. 'CH+'
     *  .OR.     KEY .EQ. 'CI'
     *  .OR.     KEY .EQ. 'CIH'
     >  )THEN
          READ(LDUMS,*)IL,IH,(CGRID(JT),JT=1,NTEMP)
          IF(IWATOM .GT. 1)THEN
            WRITE( LOUT, 802 ) KEY(:IDX(KEY)), IL, IH, (CGRID(JT), JT =
     >      1, NTEMP)
  802       FORMAT(1X,A,2(2X,I2),10(1P,1X,E10.3))
          ENDIF
        ELSE
          WRITE(LJOBLO,191) KEY(:IDX(KEY))
  191     FORMAT('  GENCOL: KEYWORD IS  (',A,')')
          WRITE( 6, * ) ' GENCOL:  UNKNOWN KEYWORD IN ATOMIC DATA'
          CALL EXIT( 1 )
        ENDIF
C
C  IDENTIFY THE UPPER AND LOWER LEVELS:
C
        ILO=MIN( IL, IH )
        IHI=MAX( IL, IH )
C ***
C ***   Get the level energies as this stuff expects them relative to the ground
C ***   state of the *model* atom
C ***
C ***	I don't believe this.  An absolute energy is only used by SEMIC's
C ***   Shevelko approximation.
C ***   Everything else uses E_HI-E_LO one way or another.  I can't see how
C ***   the ionization energy of lower stages can enter.
C ***
C ***	29-APR-1996 20:33:20.38		ML / Bamberg.  Disable this for now.
C ***
C ***   Well, this breaks for the ionization levels.  E.g., ground state of
C ***	neutral has 0 eV, ground state of first ion stage has 0 eV in our
C ***	scheme => E_HI-E_LO = 0.  Need to add ionization energy to upper level.
C ***	This is only a problem for ionization.  In fact, the old system was
C ***	probably wrong for the Shevelko approx. as it assigned strange energies
C ***	as eps(I) + sum of all lower ion energies.  Try again:
C ***

        e_lo = eps(ilo)
	e_hi = eps(ihi)
	if( stage(ihi) .eq. stage(ilo) + 1 ) then
C ***
C ***     Ionization
C ***
          e_hi = e_hi + chi_j( stage(ilo) )
        end if

	go to 1111	! skip old junk


        IF( ILO .LT. NXTION(1) ) THEN
C ***
C ***      First ionization stage
C ***
           E_LO = EPS(ILO)
        ELSE IF( ILO .LT. NK ) THEN
C ***
C ***      Second ionization stage
C ***
           E_LO = EPS(ILO) + CHI_J(0)
        ELSE
C ***
C ***      Continuum
C ***
           IF( NK .EQ. NXTION(1) ) THEN
C ***
C ***         Only one ionization stage present
C ***
              E_LO = CHI_J(0)
           ELSE
C ***
C ***         Two stages there
C ***
              E_LO = CHI_J(0) + CHI_J(1)
           END IF
        END IF
C ***
C ***   Same for higher level
C ***
        IF( IHI .LT. NXTION(1) ) THEN
           E_HI = EPS(IHI)
        ELSE IF( IHI .LT. NK ) THEN
           E_HI = EPS(IHI) + CHI_J(0)
        ELSE
           IF( NK .EQ. NXTION(1) ) THEN
              E_HI = CHI_J(0)
           ELSE
              E_HI = CHI_J(0) + CHI_J(1)
           END IF
        END IF

1111	continue		! skip for the stuff above.
        EIJ = E_HI - E_LO
C
C  CHECK TO SEE THAT THE LEVELS ARE BETWEEN 1 AND NK
C
        IF(ILO .LT. 1 .OR. IHI .GT. NK) THEN
           WRITE( 6, * ) ' GENCOL:  LEVEL INDEX OUTSIDE RANGE (1,NK)'
           CALL EXIT( 1 )
        END IF
C
C  CHECK TO SEE IF THERE ARE LEVELS WITH ENERGIES NOT INCREASING WITH
C  THE LEVEL LABEL (THIS AFFECTS CI ETC.)
C
        IF(E_HI .LT. E_LO )THEN
          WRITE(LJOBLO,1001) KEY(1:4),IHI,ILO
 1001     FORMAT('  GENCOL:  WARNING- KEYWORD ',A,2(2X,I2),
     *           ' ENERGY OF UPPER LEVEL IS .LT. LOWER LEVEL'/
     *           '           WARNING- CHECK RESULTS CAREFULLY')
        ENDIF
C
C  DEPTH DO-LOOP:
C
        DO 3 K = 1, NDEPTH
          NE = XP( PELG(K) )/TEMP(K)/BK
          NH = 2.*RHO(K)/AVEMAS*ZETAH(K)
          NHII = RHO(K)/AVEMAS*ZETAHII(K)
          IF (KEY  .EQ.  'AUMI9')THEN
C
C           Auer & Mihalas Formula #9 (van Regemorter)
C     
            FEM = XP( FAB )/G( ILO )
            IF( stage(ILO) .eq. 0 ) THEN
               ION = 1
            ELSE
               WRITE( *, * ) 'Formula AUMI9 not implemented for ions.'
               CALL EXIT(1)
            END IF
            U0 = EIJ/( 8.6171E-5*TEMP(K) )
            IF( U0 .LE. 14. ) THEN
               GAMMA = 0.276*EXP( U0 )*EXPINT( U0 )
            ELSE
               GAMMA = 0.066/SQRT( U0 )*( 1 + 1.5/U0 )
            END IF
            CT = 5.465E-11*SQRT( TEMP(K) )*
     >           14.5*FEM*(13.59/EIJ)**2*U0*EXP(-U0)*GAMMA
            CUP = CT*NE*FACCEM
            CDN = CUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
          ELSE IF (KEY  .EQ.  'AUMI13')THEN
C
C           Auer & Mihalas Formula #13 (optically forbidden, Allen)
C     
            OMEGA = FAB
            IF( stage(ILO) .eq. 0 ) THEN
               ION = 1
            ELSE
               WRITE( *, * ) 'Formula AUMI13 not implemented for ions.'
               CALL EXIT(1)
            END IF
            U0 = EIJ/( 8.6171E-5*TEMP(K) )
            CT = 5.465E-11*SQRT( TEMP(K) )*OMEGA/G(ILO)*
     >           (13.59/EIJ)*U0*( EXP(-U0) - U0*EXPINT(U0) )
            CUP = CT*NE*FACCEM
            CDN = CUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
          ELSE IF (KEY  .EQ.  'MI5-79')THEN
C
C           Mihalas Formula #5-79 (ionization rate, based on Jefferies)
C     
            A0 = FAB*1E-18                  ! FAB in 1e-18 cm^2
            gbar = 0.1*( stage(ilo) + 1 )
            U0 = EIJ/( 8.6171E-5*TEMP(K) )
            CT = 1.55E13/SQRT( TEMP(K) )*GBAR*A0*EXP(-U0)/U0
            CUP = CT*NE*FACCEM
            CDN = CUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
          ELSE IF (KEY  .EQ.  'SEMI')THEN
C
C           SEMI-EMPIRICAL COLLISIONS
C     
            EUPCM=E_HI*EE/CC/HH
            ELOCM=E_LO*EE/CC/HH
            FEM = FAB*G(ILO)/G(IHI)
            ion = stage(ilo) + 1
            CT=SEMIC(ION,EUPCM,ELOCM,FEM,TEMP(K),III)
            CDN=CT*NE*FACCEM
            CUP = CDN*N(K,IHI)/N(K,ILO)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
          ELSE IF( KEY .EQ. 'SEMIH' ) THEN
C
C           SEMI-EMPIRICAL COLLISIONS + HYDROGEN COLLISIONS
C     
            EUPCM=E_HI*EE/CC/HH
            ELOCM=E_LO*EE/CC/HH
            FEM = FAB*G(ILO)/G(IHI)
            ion = stage(ilo) + 1
            CT = SEMIC(ION,EUPCM,ELOCM,FEM,TEMP(K),III)
            CDN = CT*NE*FACCEM
            CUP = CDN*N(K,IHI)/N(K,ILO)
            CHUP = FACCHI*C_HI( CUP/FACCEM, EIJ, TEMP(K), NE, NH )
            CHDN = CHUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + CHDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + CHUP + MC_C( ILO, IHI, K )
          ELSE
            CT=SPLIN(TEMP(K),TGRID,CGRID,NTEMP,NTEMP)
          ENDIF
C
C  Effective OMEGAS ARE GIVEN (+VE IONS)
C
          IF(KEY .EQ. 'GAM') THEN
            DEKT=  (E_HI-E_LO) * EK / TEMP(K)
            CUP = FACCEM * 8.63E-06 * CT * NE * EXP(-DEKT) /
     >            ( G(ILO)*SQRT(TEMP(K)) )
            CDN = CUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C
C  OMEGAS ARE GIVEN (+VE IONS)
C
          ELSE IF(KEY .EQ. 'OHM') THEN
            CDN = FACCEM * 8.63E-06 * CT * NE / ( G(IHI)*SQRT(TEMP(K)) )
            CUP = CDN*N(K,IHI)/N(K,ILO)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C
C  OMEGAS ARE GIVEN (+VE IONS) + HYDROGEN COLLISIONS
C
          ELSE IF(KEY .EQ. 'OHMH') THEN
            CDN = FACCEM * 8.63E-06 * CT * NE / ( G(IHI)*SQRT(TEMP(K)) )
            CUP = CDN*N(K,IHI)/N(K,ILO)
            CHUP = FACCHI*C_HI( CUP/FACCEM, EIJ, TEMP(K), NE, NH )
            CHDN = CHUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + CHDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + CHUP + MC_C( ILO, IHI, K )
C
C  CE VALUES ARE GIVEN (NEUTRALS)
C
          ELSE IF (KEY .EQ. 'CE')THEN
            CDN = FACCEM * NE * CT * G(ILO) * SQRT(TEMP(K)) / G(IHI)
            CUP = CDN*N(K,IHI)/N(K,ILO)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C
C  CEH VALUES ARE GIVEN (NEUTRALS) + HYDROGEN COLLISIONS
C
          ELSE IF (KEY .EQ. 'CEH')THEN
            CDN = FACCEM * NE * CT * G(ILO) * SQRT(TEMP(K)) / G(IHI)
            CUP = CDN*N(K,IHI)/N(K,ILO)
            CHUP = FACCHI*C_HI( CUP/FACCEM, EIJ, TEMP(K), NE, NH )
            CHDN = CHUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + CHDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + CHUP + MC_C( ILO, IHI, K )
C
C  CP VALUES ARE GIVEN (B-B COLLISIONS WITH PROTONS)
C
          ELSE IF (KEY .EQ. 'CP')THEN
            CDN = NHII * CT 
            CUP = CDN*N(K,IHI)/N(K,ILO)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C
C  CH VALUES ARE GIVEN (COLLISIONS WITH NEUTRAL HYDROGEN)
C
          ELSE IF (KEY .EQ. 'CH')THEN
            CDN = NH * CT 
            CUP = CDN*N(K,IHI)/N(K,ILO)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C       
C  CI VALUES ARE GIVEN
C
          ELSE IF (KEY .EQ. 'CI')THEN
            DEKT=  (E_HI-E_LO) * EK / TEMP(K)
            CUP = FACCEM * NE * CT * EXP(-DEKT) * SQRT(TEMP(K))
            CDN = CUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + MC_C( ILO, IHI, K )
C       
C  CIH VALUES ARE GIVEN + HYDROGEN COLLISIONS
C
          ELSE IF (KEY .EQ. 'CIH') THEN
            DEKT=  (E_HI-E_LO) * EK / TEMP(K)
            CUP = FACCEM * NE * CT * EXP(-DEKT) * SQRT(TEMP(K))
            CDN = CUP*N(K,ILO)/N(K,IHI)
            CHUP = FACCHI*C_HI( CUP/FACCEM, EIJ, TEMP(K), NE, NH )
            CHDN = CHUP*N(K,ILO)/N(K,IHI)
            MC_C( IHI, ILO, K ) = CDN + CHDN + MC_C( IHI, ILO, K )
            MC_C( ILO, IHI, K ) = CUP + CHUP + MC_C( ILO, IHI, K )
C
C  CH OR CH+ VALUES GIVEN:  CHARGE TRANSFER COLLISIONS WITH 
C            HYDROGEN ATOMS/IONS  NB:  ORDERING IMPORTANT HERE
C  EXAMPLE INPUT:
C
C  CH0
C  1  2  1.E-9 1.E-9 1.E-9
C
C  THIS ASSUMES THAT THE RATE 1.E-9*NH WILL BE ADDED TO THE
C  COLLISION RATE from level 1 to level 2
C
          ELSE IF (KEY .EQ. 'CH0')  THEN
            MC_C( IL, IH, K ) = NH * CT + MC_C( IL, IH, K )
          ELSE IF (KEY .EQ. 'CH+') THEN
            MC_C( IL, IH, K ) = NHII * CT + MC_C( IL, IH, K )
C
C  MORE KEYWORDS CAN BE ADDED HERE
C
          ENDIF
    3   CONTINUE
    1 CONTINUE
C
 301  WRITE( 6, * ) ' GENCOL: KEYWORD ''END'' NOT FOUND IN ',
     >  'COLLISION FILE'
      CALL EXIT( 1 )
 300  CONTINUE
      END
C
C ****************************************************************
C
      FUNCTION C_HI( CUP, EIJ, TEMP, NE, NH )
C ***
C ******************************************************************************
C ***
C *** Returns C_ij with H I derived from C_ij for e- collisions.
C *** Assumes optically allowed e- transition.
C ***
C *** CUP:     e- rate
C *** EIJ:     E_hi - E_lo
C *** TEMP:    Temperature
C *** NE:      e- number density
C *** NH:      H I number density
C ***
C *** 30-MAY-1991 17:08:58.83
C ***
C ******************************************************************************
C ***
C *** Parameters
C ***
      REAL C_HI, CUP, EIJ, TEMP, NE, NH
C ***
C *** General dimensions
C ***
      INCLUDE 'MOD_SIZ.INC'
C ***
C *** COMMON blocks
C ***
      include 'LINE.INC'
C ***
C *** Local variables
C ***
      REAL F1, PSI, W, BETA, ACON, BCON, TEV
C ***
C *** Statement functions
C ***
      F1( W, BETA ) = 4.*W/(1. + W)*EXP( -W )*( 1./(20. + W) + ALOG( (1.
     >                + W)/W*1.25*BETA) )
      PSI( W ) = (1. + 2./W)/( 1. + (1.08808E-3/W)**2 )*EXP( -W )
C ***
C *** Executables
C ***
      TEV = 8.6171E-5*TEMP
      ACON = NE*5.8875E-9*SQRT( TEV )
      BCON = NH*5.93095E-13*SQRT( TEV*WEI*( WEI + 1.008 ) )
      W = EIJ/TEV
      C_HI = CUP*BCON*PSI(W)/(ACON*F1( W, 1. )*0.8278)

      END
C
C ****************************************************************
C
        FUNCTION SPLIN(T, X, Y, N, NNN)
C
C  SPLINE INTERPOLATION OF T IN A GIVEN TABLE OF POINTS(X(I),Y(I)).
C
C Y(I) IS A FUNCTION OF X(I) - X(N)>X(N-1)>....X(1) -OR- X(1)> X(2)
C > ....> X(N) - WHEN X IS NORMALISED X(MIN) = 0, XMAX=1,
C NORM(ABS(X(I+1)-X(I))) < 1.0E-8,(I=1,N-1). THE ARRAY X IS DIVIDED
C INTO OVERLAPPING INTERVALS. EACH INTERVAL CONSISTS OF NNN POINTS,
C THE OVERLAP IS INT(LOG(NNN)+3) POINTS. ( 3<=NNN<=100,N).
C EACH TIME AFTER A CALL WHEN T CHANGES OF INTERVAL OR AFTER A CALL
C WITH ANOTHER SET OF POINTS, A NEW CUBIC SPLINE OF NNN POINTS HAS
C TO BE COMPUTED, THEREFORE THE COMPUTING TIME DEPENDS STRONGLY OF
C THE SUCCESSION OF CALLS.
C FOR EXAMPLE: SUCCESSIVE CALLS, WHERE T IS RANDOMLY CHOSEN WILL COST
C A LOT OF COMPUTING TIME - (T OFTEN CHANGES OF INTERVAL) --->
C TAKE NNN THEN AS LARGE AS POSSIBLE. SUCCESSIVE CALLS WHERE T CHANGES
C SLIGHTLY COST LESS COMPUTING TIME, SO IT WILL BE CLEAR THAT YOUR
C CHOICE OF NNN DEPENDS HEAVILY UPON ITS WAY OF USEAGE.
C
C COMPUTING TIME:   FOR THE COMPUTATION OF SPLINE COEFFICIENTS
C        THIS ROUTINE NEEDS ABOUT 21*N MULTIPLICATIONS OR DIVISIONS
C        FOR THE INTERPOLATION BY MEANS OF THE SPLINE COEFFICIENTS
C        THE ROUTINE NEEDS 9 MULTIPLICATIONS OR DIVISIONS.
C
C EXTRAPOLATION: IF T IS NOT IN [X(1),X(N)] T IS LINEAR EXTRAPOLATED
C
C A PART OF THE ALGORITM FOR THE COMPUTATION OF A NATURAL SPLINE
C IS TAKEN FROM T.N.E. GREVILLE, THE LINEAR SYSTEM IS SOLVED BY
C THE METHOD OF SUCCESSIVE OVERRELAXATION. (ERROR = 1.0E-6)
C
C REF: T.N.E. GREVILLE, MATHEMATICS RESEARCH CENTER, U.S. ARMY,
C        UNIVERSITY OF WISCONSIN. MATHEMATICAL METHODS.
C
C THIS ROUTINE WAS IMPLEMENTED BY E.B.J. VAN DER ZALM, STERRENWACHT
C UTRECHT (MAY 7TH 1981). ADAPTED FOR VAX\11 BY PAUL KUIN AT OXFORD.
C WRITTEN IN REAL*4 BY P.G. JUDGE, OXFORD
C MODIFIED SO THAT IF THE ARGUMENT IS OUTSIDE THE RANGE OF X-VALUES
C THEN THE FIRST (OR LAST) Y-VALUE IS TAKEN
C
C INPUT:         T        ARGUMENT   (REAL)
C                X        ARRAY OF ARGUMENTS        (REAL)
C                Y        ARRAY OF FUNCTION VALUES (REAL)
C                N        LENGTH OF THE ARRAYS X,Y
C                        IF N = 2 ---> LINEAR INTERPOLATION
C                NNN        NUMBER OF POINTS OF THE CUBIC SPLINE
C                        IF NNN > N   ---> NNN = N
C                        IF NNN > 100 ---> NNN = 100
C                        IF NNN = 2 LINEAR INTERPOLATION ADOPTED.
C
C                        IF NNN IS ZERO NNN=7  (ALTERED BY P JUDGE)
C                       (USED TO BE IF NNN OMITTED )
C OUTPUT:        SPLIN INTERPOLATED VALUE AT T
C
C      INCLUDE 'PREC'
        DIMENSION X(N), Y(N), S2(100), S3(100), DELY(100), H(100),
     * H2(100), DELSQY(100), C(100), B(100), XX(100), YY(100)
        LOGICAL SEARCH, FOUND
        character*4 init
	data init /'nook'/
C
C  NN = NUMBER OF POINTS OF THE SPLINE INTERVAL
C
        NN = MIN(7,N)
        IF (NNN .EQ. 0) GO TO 10
C PGJ        IF (%LOC(NNN).EQ.0) GO TO 10
        NN = NNN
        NN = MIN(N,NNN,100)
C
C           IF T IN [X(K)-ERROR,X(K)+ERROR] T = X(K)
C
 10        ERROR = 1.0E-12
        IF (T.NE.0) ERROR = ABS(1.0E-12*T)
        FOUND = SEARCH(X,T,1,N,K,ERROR)
        IF (.NOT.FOUND) GOTO 20
        SPLIN = Y(K)
        RETURN
 20        IF (N.NE.2.AND.NN.NE.2) GOTO 30
C
C           IF N=2 OR NNN=2 LINEAR INTERPOLATION IS ADOPTED
C
        SPLIN = (Y(K+1)-Y(K))*(T-X(K))/(X(K+1)-X(K)) + Y(K)
        RETURN
 30        IF (N.NE.1) GOTO 40
        SPLIN = Y(1)
        RETURN
C
C           INTERVAL OF THE SPLINE IS COMPUTED
C
 40        IOVER = LOG10(FLOAT(NN))*3
        INT = (K-IOVER)/(NN-2*IOVER) + 1
        I1 = (INT-1)*(NN-2*IOVER)
        I2 = I1 + NN -1
        IF (I1.GE.1) GOTO 50
        I2 = NN
        I1 = 1
 50        IF (I2.LE.N) GOTO 60
        I2 = N
        I1 = N - NN + 1
 60         ISHIFT = I1 - 1
        N1 = NN - 1
C
C           INITIALIZATION CHECK
C
        IF (INIT.NE.'OKAY') GOTO 80
C
C           INTERVAL CHECK
C
        DO 70 I=1,NN
        IF (XX(I).NE.X(I+ISHIFT)) GOTO 90
        IF (YY(I).NE.Y(I+ISHIFT)) GOTO 90
70        CONTINUE
        GOTO 200
80        INIT = 'OKAY'
C
C           START OF THE COMPUTATION OF SPLINE COEFFICIENTS
C
 90        RMIN = 1.0E37
        RMAX = -1.0E37
        DO 100 I=1,NN
        IF (Y(I+ISHIFT).LT.RMIN)  RMIN = Y(I+ISHIFT)
        IF (Y(I+ISHIFT).GT.RMAX)  RMAX = Y(I+ISHIFT)
 100        CONTINUE
C
C           COMPUTATION OF THE NORM FACTORS XNORM AND YNORM
C
        XNORM = 1/(X(NN+ISHIFT)-X(1+ISHIFT))
        YNORM = RMAX - RMIN
        IF (YNORM.EQ.0) YNORM = RMAX
        IF (YNORM.EQ.0) YNORM = 1
        YNORM = 1 / YNORM
        DO 110 I=1,N1
           H(I) = (X(I+1+ISHIFT)-X(I+ISHIFT))*XNORM
           DELY(I) = ((Y(I+1+ISHIFT)-Y(I+ISHIFT))/H(I))*YNORM
 110        CONTINUE
        DO 120 I=2,N1
           H2(I) = H(I-1) + H(I)
           B(I) = 0.5*H(I-1)/H2(I)
           DELSQY(I) = (DELY(I)-DELY(I-1))/H2(I)
           S2(I) = 2.*DELSQY(I)
           C(I) = 3.*DELSQY(I)
 120        CONTINUE
        S2(1) = 0
        S2(N1+1) = 0
C
C        SOLUTION OF THE LINEAR SYSTEM OF THE SPLINE COEFFICIENTS
C        BY SUCCESSIVE OVERRELAXATION.
C        CONSTANTS: EPS = ERROR CRITERION IN THE ITERATIVE SOLUTION.
C                   OMEGA = RELAXATION COEFFICIENT.
C
        EPSLN = 1.0E-8
        OMEGA = 1.0717968
 130        ETA = 0.
        DO 160 I=2,N1
           W = (C(I)-B(I)*S2(I-1)-(0.5-B(I))*S2(I+1)-S2(I))*OMEGA
           IF (ABS(W)-ETA) 150, 150, 140
 140        ETA = ABS(W)
 150        S2(I) = S2(I) + W
 160        CONTINUE
        IF (ETA-EPSLN) 170, 130, 130
 170        DO 180 I=1,N1
           S3(I) = (S2(I+1)-S2(I))/H(I)
 180        CONTINUE
C
C        X AND Y STORED IN XX AND YY FOR INTERVAL CHECK
C
        DO 190 I=1,NN
           XX(I) = X(I+ISHIFT)
           YY(I) = Y(I+ISHIFT)
 190        CONTINUE
 200        I = K - I1 + 1
        HT1 = (T-X(I+ISHIFT))*XNORM
        HT2 = (T-X(I+ISHIFT+1) )*XNORM
        IF ((T-X(1))*XNORM.GT.0) GOTO 210
C
C        EXTRAPOLATION   T < X(1)
C
        SPLIN = Y(1)
        RETURN
210        IF ((T-X(N))*XNORM.GT.0) GOTO 220
C
C        INTERPOLATION BY MEANS OF THE SPLINE COEFFICIENTS
C
        SS2 = S2(I) + HT1*S3(I)
        PROD = HT1*HT2
        DELSQS = (S2(I)+S2(I+1)+SS2)/6.
        SPLIN = Y(I+ISHIFT) + (HT1*DELY(I)+PROD*DELSQS)/YNORM
        RETURN
C
C        EXTRAPOLATION   T > X(N)
C
 220        SPLIN =  Y(N)
        RETURN
        END
C
C **********************************************************************
C
      LOGICAL FUNCTION SEARCH(ARR,X,IA,IB,K,ERR)
C
C * SEARCHES THE POINT X WITHIN AN ERROR BOUND -ERR- IN ARRAY ARR      *
C * IF THAT POINT IS NOT FOUND, IT GIVES THE                           *
C * INTERVAL IN THE ARRAY, WHERE THAT POINT FITS.                      *
C * THE INTERVAL K IF X IN [ARR(K),ARR(K+1)), INTERVAL 1 IF X IN       *
C * (-INF,ARR(2)), INTERVAL N-1 IF X IN [ARR(N-1),INF)-(ARR INCREASING)*
C * INTERVAL 1 IF X IN (INF,ARR(2)),INTERVAL N-1 IF X IN [ARR(N-1),    *
C * -INF)-(ARR DECREASING)                                             *
C * IN THE WORST CASE A SEARCH COSTS 2*LOG2(N) CYCLES.                 *
C * THIS ROUTINE IS VERY ECONOMIC, WHEN IN A SEQUENCE OF SEARCHES,     *
C * THE DIFFERENCES BETWEEN THE POINTS WHICH ARE TO BE SEARCHED ARE    *
C * SMALL.                                                             *
C * FOR EXAMPLE: A SEQUENCE OF SEARCHES WHICH GOES ALONG THE ARRAY,    *
C * OR A SEQUENCE OF SEARCHES WHICH REMAINS IN A SMALL AREA OF         *
C * THE ARRAY.                                                         *
C *                                                                    *
C * INPUT:     - ARR         ARRAY WITH A NON DECREASING OR A NON      *
C *                          INCREASING FUCNTION.                      *
C *            - IA,IB       LOWER AND UPPER LIMIT OF THE ARRAY WITHIN *
C *                          IS SEARCHED.                              *
C *            - X           VALUE WHICH YOU WANT TO SEARCH            *
C *            - K           LAST INDEX WHICH WAS FOUND                *
C *                          SEARCH                                    *
C *            - ERR         ERROR BOUND                               *
C *                                                                    *
C * OUTPUT:    - K           INDEX OF THE POINT OR INTERVAL OF THE     *
C *                          ARRAY WHICH IS FOUND.                     *
C *            - SEARCH      TRUE - X IS FOUND WITHIN THE ERRORBOUND   *
C *                                  -ERR-                             *
C *                          FALSE - X IS NOT FOUND.                   *
C *                                                                    *
C * SEND IN BY E.V.D.ZALM, UTRECHT, STERREWACHT, MARCH 24TH 1981       *
C **********************************************************************
C      INCLUDE 'PREC'
      DIMENSION ARR(IB)
C
      ONE=1.0
      SEARCH = .TRUE.
      IF (ARR(IB)-ARR(IA).LT.0.0) GOTO 100
5     IF (X.LT.ARR(IA).OR.X.GT.ARR(IB)) GOTO 25
      IF (K.LT.IA.OR.K.GT.IB-1) GOTO 25
      L = K
      IP = SIGN(ONE,X-ARR(K))
      IF (IP.LT.0.) GOTO 20
10    L = MIN(K+IP,IB)
      IF (ARR(L).GE.X) GOTO 30
      IP = IP * 2
      K = L            
      GOTO 10
20    K = MAX(L+IP,IA)
      IF (ARR(K).LE.X) GOTO 30
      IP = IP * 2
      L = K
      GOTO 20
25    K = IA
      L = IB
30    IF ((L-K).LE.1) GOTO 50
      I = INT((K+L)/2.)
      IF (ARR(I).GE.X) GOTO 40
      K = I
      GOTO 30
40    L = I
      GOTO 30
100   IF (X.LT.ARR(IB).OR.X.GT.ARR(IA)) GOTO 250
      IF (K.LT.IA.OR.K.GT.IB-1) GOTO 250
      L = K
      IP = SIGN(ONE,ARR(K)-X)
      IF (IP.LT.0.) GOTO 200
110   L = MIN(K+IP,IB)
      IF (ARR(L).LE.X) GOTO 300
      IP = IP * 2
      K = L
      GOTO 110
200   K = MAX(L+IP,IA)
      IF (ARR(K).GE.X) GOTO 300
      IP = IP * 2
      L = K
      GOTO 200
250   K = IA
      L = IB
300   IF ((L-K).LE.1) GOTO 50
      I = INT((K+L)/2.)
      IF (ARR(I).LE.X) GOTO 400
      K = I
      GOTO 300
400   L = I
      GOTO 300
50    IF (ABS(X-ARR(K)).LE.ERR) RETURN
      IF (ABS(X-ARR(L)).GT.ERR) GOTO 60
      K = L
      RETURN
60    SEARCH = .FALSE.
      RETURN
      END
C
C***********************************************************************
C
      FUNCTION SEMIC(Z,EUP,ELO,FEM,TEMP,IFLAG)
C
C  RETURNS COLLISION RATE DOWNWARDS USING SEMI-EMPIRICAL GF'S
C
C  P.G. JUDGE, NOVEMBER 1987.
C
C  INPUT:
C         Z      - CHARGE OF ION+1 (E.G. FOR C II Z=2)
C         EUP    - ENERGY OF UPPER LEVEL IN CM-1
C         ELO    - ENERGY OF LOWER LEVEL IN CM-1
C         FEM    - EMISSION OSCILLATOR STRENGTH ( = GF/G(UPPER))
C         TEMP   - TEMPERATURE IN DEGREES KELVIN
C
C  OUTPUT:
C         IFLAG  -  = 1 FOR VAN REGEMORTER APPROXIMATION
C         IFLAG  -  = 2 FOR SHEVELKO       APPROXIMATION
C         SEMIC-  DOWNWARD COLLISION RATE IN CM3 /S
C
C      INCLUDE 'PREC'
      INTEGER Z
      DATA C1,RY /1.43882,109737.312/
C
C initialize
C
      DELTE=EUP-ELO
      BETA=DELTE*C1/TEMP
      SEMIC=0.0
C
C  IFLAG=1 => Van Regemorter approxn.
C
      IF(BETA .GE. 0.01)THEN
        IFLAG=1
        SEMIC = 3.2E-7*FEM*(RY/DELTE)**1.5*SQRT(BETA)*P(Z,BETA)
C
C  IFLAG=2 => Shevelko approx
C
      ELSE IF(BETA .LT. 0.01)THEN
        IFLAG=2
        SEMIC=1.74E-07*FEM*EXP(BETA)/SQRT(BETA*DELTE/RY)
     +  *LOG( ELO / DELTE * SQRT(RY / C1 / TEMP))
      ENDIF
C
      RETURN
      END
C
C **********************************************************************
C
      FUNCTION P(Z,B)
C
C  THERMAL P-FUNCTION FOR SEMIC EMPIRICAL COLLISION RATES
C  REFERENCE: SOBEL'MAN - 'ATOMIC PHYSICS II.  '
C
C      INCLUDE 'PREC'
      PARAMETER (N1=10)
      DIMENSION  BETREF(N1),PNREF(N1),PCREF(N1)
      INTEGER Z
      DATA BETREF/-2.0,-1.699,-1.398,-1.0,-0.699,-0.398,0.0
     +  ,0.301,0.602,1.0/
      DATA PNREF/1.160,0.956,0.758,0.493,0.331,0.209
     +  ,0.100,0.063,0.040,0.023/
      DATA PCREF/1.160,0.977,0.788,0.554,0.403
     +  ,0.290,0.214,0.201,0.200,0.200/
C
C  Limit of high temperature: kT >> E
C
      p = 0.	! confuses compiler if not set
      IF (B .LT. 0.01)THEN
        P = 0.27566 * (0.577 + LOG(B))
      RETURN
C
C  Intermediate temps (MOST IMPORTANT FOR EQM PLASMAS)
C  SPLINE interpolation onto logB grid:
C
      ELSE IF(B .GT. 0.01 .AND. B .LT. 10.0)THEN
        BB=LOG10(B)
        IF (Z .EQ. 1)P = SPLIN(BB,BETREF,PNREF,N1,N1)
        IF (Z .GT. 1)P = SPLIN(BB,BETREF,PCREF,N1,N1)
      RETURN
C
C  Limit of low temperature: kT << E
C
      ELSE  IF(B .GT. 10.0)THEN
        IF(Z .EQ. 1)P = 0.066 / SQRT(B)
        IF(Z .GT. 1)P = 0.200
        RETURN
      ENDIF
      END
C
C **********************************************************************
C
      SUBROUTINE LJUST(TEXT)
C
C  LEFT JUSTIFIES THE STRING TEXT
C
C ***
C *** completely modified
C ***
C *** M. Lemke 14-MAY-1991 14:59:14.18
C ***
      CHARACTER*(*) TEXT
C
      DO J = 1, LEN( TEXT )
        IF( TEXT(J:J) .GT. ' ' ) GO TO 200
      END DO
  200 CONTINUE
      TEXT = TEXT(J:)
      END

