        function integral( x, f, work, n )
        implicit none
C ***
C ******************************************************************************
C ***
C ***   Similar function than Kurucz' subroutine INTEG for integrating a
C ***   discrete data set.  Uses weighted forward/backward parabolas to
C ***   approximate integrand. 
C ***
C ***   Arguments:
C ***
C ***   X(N)            REAL    input           independent variable
C ***   F(N)            REAL    input           dependent variable
C ***   WORK(3*N)       REAL    input           work space
C ***   N               INTEGER input           number of points to integrate
C ***
C ***   INTEGRAL        REAL    output          the integral
C ***
C ***   It is possible to reformulate the problem without requiring the
C ***   work space but this should run faster on a vector machine.
C ***
C ***   23-APR-1993 21:15:37.03
C ***
C ******************************************************************************
C ***

C ***
C ***   Arguments
C ***
        integer n
        real x(n), f(n), work(3*n), integral

C ***
C ***   Local variables
C ***

        integer i, a, b, c

        a = 0
        b = n
        c = 2*n

        call parcoe( f, x, work(a+1), work(b+1), work(c+1), n )
        integral = 0.
        do i = 1, n - 1
           integral = integral + 
     >                ( work(a+i) + work(b+i)/2.*(x(i + 1) + x(i)) +
     >                work(c+i)/3.*( (x(i + 1) + x(i))*x(i+1) +
     >                x(i)*x(i)) )*( x(i + 1) - x(i) )
        end do
        end
