        SUBROUTINE JOSH
C ***
C ******************************************************************************
C ***
C ***   Subroutine wurde in Anlehnung an die gleichnamige Routine aus ATLAS6
C ***   von Kurucz geschrieben.
C ***
C ***   Zweck:  Berechnung von Tau Nue, S Nue, J Nue, H Nue Surf
C ***
C ***   Input:  NDEPTH          COMMON /MODEL/  Zahl der Tiefenpunkte der Atm.
C ***           TEMP(I)                         Temperatur
C ***           PELG(I)                         Elektronendruck
C ***           ROPREF(I)                       Kappa ref + Sigma ref
C ***           TAUREF(I)                       Tau ref
C ***
C ***           BNU(I)          COMMON /RAD/    B nue
C ***
C ***           RKACON(I)       COMMON /OPAC/   Kappa Nue cont.
C ***           RSICON(I)                       Sigma Nue cont.
C ***           RLINE(I)                        Kappa Nue line (Lineblanketing)
C ***
C ***
C ***   Output: TAUNU(I)        COMMON /RAD/    Tau Nue
C ***           SNU(I)                          S Nue
C ***           JNU(I)                          J Nue
C ***           HSURF                           H Surface
C ***
C ***
C ***
C ***                    01-DEC-87 11:32:07
C ***
C ***   Aenderungen:
C ***
C ***   15-JUN-1990: NXTAU in Iterationsschleifen ersetzt auf
C ***                hoeheren Wert. Metallarme Fluesse konvergieren nicht bei 
C ***                niedrigem Teff (4500) und log g (1.0)
C ***	20-DEC-1994: ML / IoA.  Serious bug in computation of S_nu.  Error
C ***		     check worked only on last depth point.
C ***	21-DEC-1994 19:57:47.00		ML / IoA.  Uses now matrices from
C ***					ATLAS9 with 51 points instead of 43.
C ***                                   Had numerical problems with old one
C ***                                   leading to negative fluxes. - Noticed
C ***	                                by J. Drake, thanks.
C ***	13-JAN-1995 19:32:32.00		ML / IoA.  Spurious statement deleted.
C ***					Debug leftover?
C ***
C ***
C ******************************************************************************
C ***
	implicit none

	REAL coefj, COEFH, XTAU
	INTEGER NXTAU
        common /matx/coefj( 51, 51 ), coefh( 51 ), xtau( 51 ), nxtau
        include 'MOD_SIZ.INC'
        include 'MODEL_RA.INC'
        include 'RAD_RATE.INC'
        include 'OPAC_RAT.INC'


        REAL XS(51), XBNU(51), XBETA(51), DIAG(51)
        REAL HNU(51), JMINS(51), DUM(51)
        REAL BETA(51), XJS(51)
!        EQUIVALENCE (XBNU(1), HNU(1), DUM(1), XJS(1))
!        EQUIVALENCE (XBETA(1),SNU(1)), (DIAG(1),JMINS(1))

	integer id, j, k, l, m, n, mdummy, maxj, maxj1, nm1, nmj
	real delxs, error, snew

	integer map1

        DO 10 ID = 1, NDEPTH
           DUM(ID) = (RKACON(ID) + RLINE(ID) + RSICON(ID))/ROPREF(ID)
           BETA(ID) = RSICON(ID)/(DUM(ID)*ROPREF(ID))
10      CONTINUE
C ***
C ***   Neue Tauskala TAUNU
C ***
        CALL INTEG(TAUREF, DUM, TAUNU, NDEPTH, TAUREF(1)*DUM(1))
        TAUNU(1) = 0.
C ***
C ***   Interpolation von B Nue und Beta Nue auf Kurucz-Tauskala
C ***
        MAXJ = MAP1(TAUNU, BNU, NDEPTH, XTAU, XBNU, NXTAU)
        MAXJ = MAP1(TAUNU, BETA, NDEPTH, XTAU, XBETA, NXTAU)
	
C ***
C ***
C ***
C ***   Berechnung von S Nue (XS) auf Kurucz-Tauscala fuer Tau < 20 mit
C ***   tabellierten Matrizen.
C ***
        DO 31 L = 1, NXTAU
           IF( XBETA(L) .LE. 0. ) XBETA(L) = 0.
           XS(L) = XBNU(L)
           DIAG(L) = 1. - XBETA(L)*COEFJ(L,L)
           XBNU(L) = (1. - XBETA(L))*XBNU(L)
31      CONTINUE
        DO 34 L = 1, 1000
C ***
C ***      Iterationsschleife, oberer Index ist beliebig,
C ***      kann erhoeht werden.
C ***
	   ERROR = 0.
           DO 33 K = NXTAU, 1, -1
              delxs = 0.
              do m = 1, nxtau
                 delxs = delxs + coefj( k, m )*xs(m)
	      end do
              DELXS = (DELXS*XBETA(K) + XBNU(K) - XS(K))/DIAG(K)
              ERROR = MAX( ERROR, ABS(DELXS/XS(K)) )
              XS(K) = XS(K) + DELXS
33         CONTINUE
C ***
C ***      Iterationsende:
C ***
           IF( ERROR .LT. .00001 ) GO TO 35
C ***
34      CONTINUE

        WRITE(*, 1001)ERROR
1001    FORMAT(41H QUELLFUNKTION KONVERGIERT NICHT!  ERROR=F9.5)

C ***
C ***   Rueckinterpolation von S Nue auf vorgegebene Tauskala:
C ***
35      continue
	MDUMMY = MAP1(XTAU, XS, NXTAU, TAUNU, SNU, MAXJ)
        IF( MAXJ .NE. NDEPTH ) THEN
C ***
C ***
C ***      Berechnung von S Nue fuer Tau > 20 mit
C ***      genaeherten Lambda- und Phi-Operatoren:
C ***
           DO 40 N = MAXJ, NDEPTH
              SNU(N) = BNU(N)
40         CONTINUE
           M = MAX(MAXJ - 2, 1)
           MAXJ1 = MAX(MAXJ - 1, 1)
           NM1 = NDEPTH - M + 1
           NMJ = NDEPTH - MAXJ1 + 1
           DO 45 L = 1, 1000
C ***
C ***         Iterationsschleife, oberer Index ist beliebig,
C ***         kann erhoeht werden.
C ***
              ERROR = 0.
              CALL DERIV(TAUNU(M), SNU(M), HNU(M), NM1)
              DO 41 N = M, NDEPTH
                 HNU(N) = HNU(N)/3.
41            CONTINUE
              CALL DERIV(TAUNU(MAXJ1), HNU(MAXJ1), JMINS(MAXJ1), NMJ)
              DO 43 N = MAXJ, NDEPTH
                 JNU(N) = JMINS(N) + SNU(N)
                 SNEW = (1. - BETA(N))*BNU(N) + BETA(N)*JNU(N)
                 ERROR = ABS(SNEW - SNU(N))/SNEW + ERROR
                 SNU(N) = SNEW
43            CONTINUE
C ***
C ***         Iterationsende:
C ***
              IF( ERROR .LT. .00001 ) GO TO 46

45         CONTINUE

           WRITE(*, 1002)ERROR
1002       FORMAT( ' QUELLFUNKTION FUER GROSSE TAU KONV. NICHT, ERROR= '
     >       , F9.5 )
        END IF
C ***
C ***
C ***   Berechnung von J Nue:
C ***
46      CONTINUE
        DO 51 K = 1, NXTAU
           xjs( k ) = -xs( k )
           do m = 1, nxtau
              XJS(K) = XJs(k) + coefj( k, m )*XS(m)
           end do
51      CONTINUE
        IF( MAXJ .NE. NDEPTH ) MAXJ = MAXJ - 1
        IF( MAXJ .GT. 0 ) THEN
           MDUMMY = MAP1(XTAU, XJS, NXTAU, TAUNU, JMINS, MAXJ)
           DO 52 J = 1, MAXJ
              JNU(J) = JMINS(J) + SNU(J)
52         CONTINUE
        END IF
C ***
C ***
C ***   Berechnung von H Nue Surf:
C ***
        HSURF = 0.
        DO 61 M = 1, NXTAU
           HSURF = HSURF + COEFH(M)*XS(M)
61      CONTINUE
        HSURF = 4.*HSURF
        RETURN
        END
