        SUBROUTINE JOSH
C ***
C ******************************************************************************
C ***
C ***   Subroutine wurde in Anlehnung an die gleichnamige Routine aus ATLAS6
C ***   von Kurucz geschrieben.
C ***
C ***   Zweck:  Berechnung von Tau Nue, S Nue, J Nue, H Nue Surf
C ***
C ***   Input:  NDEPTH          COMMON /MODEL/  Zahl der Tiefenpunkte der Atm.
C ***           TEMP(I)                         Temperatur
C ***           PELG(I)                         Elektronendruck
C ***           ROPREF(I)                       Kappa ref + Sigma ref
C ***           TAUREF(I)                       Tau ref
C ***
C ***           BNU(I)          COMMON /RAD/    B nue
C ***
C ***           RKACON(I)       COMMON /OPAC/   Kappa Nue cont.
C ***           RSICON(I)                       Sigma Nue cont.
C ***           RLINE(I)                        Kappa Nue line (Lineblanketing)
C ***
C ***
C ***   Output: TAUNU(I)        COMMON /RAD/    Tau Nue
C ***           SNU(I)                          S Nue
C ***           JNU(I)                          J Nue
C ***           HSURF                           H Surface
C ***
C ***
C ***
C ***                    01-DEC-87 11:32:07
C ***
C ***   Aenderungen:
C ***
C ***   15-JUN-1990: NXTAU in Iterationsschleifen ersetzt auf
C ***                hoeheren Wert. Metallarme Fluesse konvergieren nicht bei 
C ***                niedrigem Teff (4500) und log g (1.0)
C ***	20-DEC-1994: ML / IoA.  Serious bug in computation of S_nu.  Error
C ***		     check worked only on last depth point.
C ***
C ***
C ******************************************************************************
C ***
	implicit none

	REAL C1,  C2,  C3,  C4,  C5,  C6,  C7,  C8,  C9,  C10, C11, C12,
     >       C13, C14, C15, C16, C17, C18, C19, C20, C21, C22, C23, C24,
     >       C25, C26, C27, C28, C29, C30, C31, C32, C33, C34, C35, C36,
     >       C37, C38, C39, C40, C41, C42, C43
	REAL COEFH, XTAU
	INTEGER NXTAU
        COMMON /MATX/C1(43), C2(43),  C3(43),  C4(43),  C5(43),  C6(43),
     >     C7(43),  C8(43),  C9(43), C10(43), C11(43), C12(43), C13(43),
     >    C14(43), C15(43), C16(43), C17(43), C18(43), C19(43), C20(43),
     >    C21(43), C22(43), C23(43), C24(43), C25(43), C26(43), C27(43),
     >    C28(43), C29(43), C30(43), C31(43), C32(43), C33(43), C34(43),
     >    C35(43), C36(43), C37(43), C38(43), C39(43), C40(43), C41(43),
     >    C42(43), C43(43), COEFH(43), XTAU(43), NXTAU
        include 'MOD_SIZ.INC'
        include 'MODEL_RA.INC'
        include 'RAD_RATE.INC'
        include 'OPAC_RAT.INC'


        REAL COEFJ(43,43)
        EQUIVALENCE (COEFJ(1,1), C1(1))
        REAL XS(43), XBNU(50), XBETA(50), DIAG(50)
        REAL HNU(50), JMINS(50), DUM(50)
        REAL BETA(50), XJS(50)
        EQUIVALENCE (XBNU(1), HNU(1), DUM(1), XJS(1))
        EQUIVALENCE (XBETA(1),SNU(1)), (DIAG(1),JMINS(1))

	integer id, j, k, l, m, n, mdummy, maxj, maxj1, nm1, nmj
	real delxs, error, xj, snew

	integer map1

        DO 10 ID = 1, NDEPTH
           DUM(ID) = (RKACON(ID) + RLINE(ID) + RSICON(ID))/ROPREF(ID)
           BETA(ID) = RSICON(ID)/(DUM(ID)*ROPREF(ID))
10      CONTINUE
C ***
C ***   Neue Tauskala TAUNU
C ***
        CALL INTEG(TAUREF, DUM, TAUNU, NDEPTH, TAUREF(1)*DUM(1))
        TAUNU(1) = 0.
C ***
C ***   Interpolation von B Nue und Beta Nue auf Kurucz-Tauskala
C ***
        MAXJ = MAP1(TAUNU, BNU, NDEPTH, XTAU, XBNU, NXTAU)
        MAXJ = MAP1(TAUNU, BETA, NDEPTH, XTAU, XBETA, NXTAU)
	
C ***
C ***
C ***
C ***   Berechnung von S Nue (XS) auf Kurucz-Tauscala fuer Tau < 20 mit
C ***   tabellierten Matrizen.
C ***
        DO 31 L = 1, NXTAU
           IF( XBETA(L) .LE. 0. ) XBETA(L) = 0.
	xbeta(k) = 0.
           XS(L) = XBNU(L)
           DIAG(L) = 1. - XBETA(L)*COEFJ(L,L)
           XBNU(L) = (1. - XBETA(L))*XBNU(L)
31      CONTINUE
        DO 34 L = 1, 1000
C ***
C ***      Iterationsschleife, oberer Index ist beliebig,
C ***      kann erhoeht werden.
C ***
	   ERROR = 0.
           DO 33 K = NXTAU, 1, -1
              DELXS = C1(K)*XS(1) + C2(K)*XS(2) + C3(K)*XS(3) +
     >            C4(K)*XS(4) +   C5(K)*XS(5) +   C6(K)*XS(6) +
     >            C7(K)*XS(7) +   C8(K)*XS(8) +   C9(K)*XS(9) +
     >          C10(K)*XS(10) + C11(K)*XS(11) + C12(K)*XS(12) +
     >          C13(K)*XS(13) + C14(K)*XS(14) + C15(K)*XS(15) +
     >          C16(K)*XS(16) + C17(K)*XS(17) + C18(K)*XS(18) +
     >          C19(K)*XS(19) + C20(K)*XS(20) + C21(K)*XS(21) +
     >          C22(K)*XS(22) + C23(K)*XS(23) + C24(K)*XS(24) +
     >          C25(K)*XS(25) + C26(K)*XS(26) + C27(K)*XS(27) +
     >          C28(K)*XS(28) + C29(K)*XS(29) + C30(K)*XS(30) +
     >          C31(K)*XS(31) + C32(K)*XS(32) + C33(K)*XS(33) +
     >          C34(K)*XS(34) + C35(K)*XS(35) + C36(K)*XS(36) +
     >          C37(K)*XS(37) + C38(K)*XS(38) + C39(K)*XS(39) +
     >          C40(K)*XS(40) + C41(K)*XS(41) + C42(K)*XS(42) +
     >          C43(K)*XS(43)
              DELXS = (DELXS*XBETA(K) + XBNU(K) - XS(K))/DIAG(K)
              ERROR = MAX( ERROR, ABS(DELXS/XS(K)) )
              XS(K) = XS(K) + DELXS
33         CONTINUE
C ***
C ***      Iterationsende:
C ***
           IF( ERROR .LT. .00001 ) GO TO 35
C ***
34      CONTINUE

        WRITE(*, 1001)ERROR
1001    FORMAT(41H QUELLFUNKTION KONVERGIERT NICHT!  ERROR=F9.5)

C ***
C ***   Rueckinterpolation von S Nue auf vorgegebene Tauskala:
C ***
35      continue
	MDUMMY = MAP1(XTAU, XS, NXTAU, TAUNU, SNU, MAXJ)
        IF( MAXJ .NE. NDEPTH ) THEN
C ***
C ***
C ***      Berechnung von S Nue fuer Tau > 20 mit
C ***      genaeherten Lambda- und Phi-Operatoren:
C ***
           DO 40 N = MAXJ, NDEPTH
              SNU(N) = BNU(N)
40         CONTINUE
           M = MAX0(MAXJ - 2, 1)
           MAXJ1 = MAX0(MAXJ - 1, 1)
           NM1 = NDEPTH - M + 1
           NMJ = NDEPTH - MAXJ1 + 1
           DO 45 L = 1, 1000
C ***
C ***         Iterationsschleife, oberer Index ist beliebig,
C ***         kann erhoeht werden.
C ***
              ERROR = 0.
              CALL DERIV(TAUNU(M), SNU(M), HNU(M), NM1)
              DO 41 N = M, NDEPTH
                 HNU(N) = HNU(N)/3.
41            CONTINUE
              CALL DERIV(TAUNU(MAXJ1), HNU(MAXJ1), JMINS(MAXJ1), NMJ)
              DO 43 N = MAXJ, NDEPTH
                 JNU(N) = JMINS(N) + SNU(N)
                 SNEW = (1. - BETA(N))*BNU(N) + BETA(N)*JNU(N)
                 ERROR = ABS(SNEW - SNU(N))/SNEW + ERROR
                 SNU(N) = SNEW
43            CONTINUE
C ***
C ***         Iterationsende:
C ***
              IF( ERROR .LT. .00001 ) GO TO 46

45         CONTINUE

           WRITE(*, 1002)ERROR
1002       FORMAT( ' QUELLFUNKTION FUER GROSSE TAU KONV. NICHT, ERROR= '
     >       , F9.5 )
        END IF
C ***
C ***
C ***   Berechnung von J Nue:
C ***
46      CONTINUE
        DO 51 K = 1, NXTAU
           XJ =   C1(K)*XS(1) +   C2(K)*XS(2) +   C3(K)*XS(3) +
     >            C4(K)*XS(4) +   C5(K)*XS(5) +   C6(K)*XS(6) +
     >            C7(K)*XS(7) +   C8(K)*XS(8) +   C9(K)*XS(9) +
     >          C10(K)*XS(10) + C11(K)*XS(11) + C12(K)*XS(12) +
     >          C13(K)*XS(13) + C14(K)*XS(14) + C15(K)*XS(15) +
     >          C16(K)*XS(16) + C17(K)*XS(17) + C18(K)*XS(18) +
     >          C19(K)*XS(19) + C20(K)*XS(20) + C21(K)*XS(21) +
     >          C22(K)*XS(22) + C23(K)*XS(23) + C24(K)*XS(24) +
     >          C25(K)*XS(25) + C26(K)*XS(26) + C27(K)*XS(27) +
     >          C28(K)*XS(28) + C29(K)*XS(29) + C30(K)*XS(30) +
     >          C31(K)*XS(31) + C32(K)*XS(32) + C33(K)*XS(33) +
     >          C34(K)*XS(34) + C35(K)*XS(35) + C36(K)*XS(36) +
     >          C37(K)*XS(37) + C38(K)*XS(38) + C39(K)*XS(39) +
     >          C40(K)*XS(40) + C41(K)*XS(41) + C42(K)*XS(42) +
     >          C43(K)*XS(43)
           XJS(K) = XJ - XS(K)
51      CONTINUE
        IF( MAXJ .NE. NDEPTH ) MAXJ = MAXJ - 1
        IF( MAXJ .GT. 0 ) THEN
           MDUMMY = MAP1(XTAU, XJS, NXTAU, TAUNU, JMINS, MAXJ)
           DO 52 J = 1, MAXJ
              JNU(J) = JMINS(J) + SNU(J)
52         CONTINUE
        END IF
C ***
C ***
C ***   Berechnung von H Nue Surf:
C ***
        HSURF = 0.
        DO 61 M = 1, NXTAU
           HSURF = HSURF + COEFH(M)*XS(M)
61      CONTINUE
        HSURF = 4.*HSURF
        RETURN
        END
