	function profpol( dlam, freq, prof, m_depth, n1 )
C ***
C ******************************************************************************
C ***
C ***	Interpolate profile to given delta lambda/delta lambda_D.
C ***
C ***	dlam	> r	Distance from line center in units of Doppler width
C ***	freq	> r	Frequencies where profile defined, starting at 0
C ***	prof	> r	Profile to interpolate
C ***	m_depth	> i	first dimension of prof (as in its dimension statement
C ***	n1	> i	number of frequency points in prof
C ***
C ***	profpol < r	interpolated profile
C ***
C ***	Modifications:
C ***
C ***	23-MAY-1995 16:03:49.00		ML / IoA.  Created.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer m_depth, n1
	real dlam, freq(n1), prof(m_depth,n1)

	real profpol

C ***
C ***	Local variables
C ***
	integer i, i0


C ***
C ***	Code
C ***

	do i = 1, n1
	   if( abs(dlam) .lt. freq(i) ) go to 1
	end do
1	continue

	i0 = i - 1
	if( i0 .le. 0 ) then
	  i0 = 1
	  i = 2
        else if( i .gt. n1 ) then
	  i0 = n1 - 1
          i = n1
	end if


	if( abs(dlam) .eq. freq(i0) ) then
	   profpol = prof(1,i0)
	else
	   profpol = prof(1,i0) + (prof(1,i) - prof(1,i0))/
     >                            (freq(i) - freq(i0))*
     >                            (abs(dlam) - freq(i0))
	end if

	end
