        PROGRAM RATES
C ***
C ***  Version vom 10-JAN-1995 19:32:58.00 ***
C ***
C ******************************************************************************
C ***
C ***   ZWECK:  Berechnung von:
C ***           1. Photoionisationsraten
C ***           2. Strahlungsfeld fuer Linien, die nicht im NLTE gehalten werden
C ***
C ***   INPUT:  File CROSS  mit Wellenlaengenskala fuer Integration der Raten,
C ***                       Linienuebergaengen (wahlweise), Photionisations-
C ***                       querschnitten fuer die Kanten des Modellatoms
C ***
C ***           File DF     mit Opacitydistributionfunction (Kurucz)
C ***
C ***           File JB     with J/B as deliverd by the swedes
C ***
C ***           File ATM    mit Modellatmosphaere (von ATMOS)
C ***
C ***           File $IN    mit Steuerparameter IFLPT fuer Ausdruck,
C ***                       ODFTYPE for choice of ODF:
C ***                            0 or 6 or 60: ATLAS6,
C ***                                      90: ATLAS9 fine spacing (LIT ODFs)
C ***                                      91: ATLAS9 coarse spacing (BIG ODFs)
C ***                       SCLMIN, SCLMAX, SCDF: lambda min, lambda max,
C ***                                             Skalierungsfaktor
C ***                                             fuer Skalierung der ODF
C ***                                             zwischen lambda min und
C ***                                             lambda max
C ***                       IWOUT, LMIN, LMAX:    Output der Gesamtopazitaeten:
C ***                                             fuer jede IWOUTte Tiefenstufe
C ***                                             zwischen Wellenlaengen LMIN
C ***                                             und LMAX werden Kontinuums-
C ***                                             und Linienopazitaeten gedruckt
C ***
C ***
C ***   OUTPUT: File RADIAT mit Strahlungsfeld fuer feste Linien
C ***                       (wenn gewuenscht)
C ***
C ***           File RIJ    mit Photoionisationsraten
C ***
C ***           File JNUE   mit ionisierendem Strahlungsfeld J nue fuer alle
C ***                       Wellenlaengen der ODF und fuer alle Tiefenpunkte
C ***
C ***           Ausdruck    der kontinuierlichen Fluesse fuer jeden IFLPT-en
C ***                       Wert
C ***
C ***
C ***
C ***   Grobe Programmstruktur:
C ***
C ***   1. Einlesen der Modellatmosphaere
C ***   2. Berechnen der Teilchenkonzentrationen mit IONDIS und OPALAM
C ***   3. Berechnen des Strahlungsfeldes J nue fuer feste Linien in Anlehnung
C ***      an ATLAS6 von Kurucz mit Beruecksichtigung der Distributionfunction
C ***      und Output auf File RADIAT
C ***   4. Berechnen der Photoionisationsraten:
C ***        i. Strahlungsfeld mit verschiedenen Optionen:
C ***           a: J nue = B nue
C ***           b: J nue nur aus kontinuierlichen Opazitaeten
C ***           c: J nue mit kontinuierlichen und Linienopazitaten,
C ***              = Fall b + Distributionfunction ODF
C ***           d: J nue with continous and line opacities but using J/B
C ***              computed by different porgram (i.e. MARCS) (IFRAD=3)
C ***       ii. Eigentliche Berechnung der Raten aufwaerts und abwaerts mit
C ***           diesem, gerade berechneten Strahlungsfeld
C ***      iii. Output auf File RIJ
C ***
C ***
C ***   Anmerkung: Der Steuerindex ITRAD ueberschreibt die Wirkung des Index
C ***              IFRAD.
C ***              Mit Steuerindex IFRAD laesst sich die Berechnung des
C ***              Strahlungsfeldes (Fall 4.i.) abschalten. IFRAD = 0 erzeugt
C ***              ueberhaupt kein Feld.
C ***              ITRAD = 0 nimmt immer das durch IFRAD bestimmte
C ***              Strahlungsfeld.
C ***              Mit ITRAD darf also kein Feld angefordert werden, das vorher
C ***              nicht berechnet wurde. Die fatale Fehlerbedingung ist also
C ***              gegeben, wenn ITRAD = 0 und IFRAD = 0.
C ***
C ***
C ***   Modifications:
C ***
C ***   earlier                    ML / UT Austin.  Added option for using
C ***                              J/B from file JB to get radiation field
C ***                              (IFRAD=3)
C ***
C ***   24-MAR-1993 01:02:21.00    ML / UT Austin.  Upgrade to use ATLAS9 odf.
C ***
C ***   12-APR-1993 14:33:23.94    ML / UT Austin.  Changed rates integration
C ***
C ***	10-JAN-1995 19:33:26.00	   ML / IoA.  BLOCKJ -> BLOCKHJ to use ATLAS9
C ***				   integration matrices 
C ***	 2-MAY-1996 18:25:17.82	   ML / Bamberg.  More x-sections.
C ***
C ******************************************************************************
C ***
C ***
C ***   Dimensions in COMMON blocks not set to PARAMETER since not local
C ***   to this program unit. Should be done in COMDECK
C ***
        implicit none
        include 'MOD_SIZ.INC'

C ***
C ***   Global variables
C ***
        include 'OPALION.INC'

        real ze, fr, densnc
        COMMON /ZFRA/ ZE(max_ion,M_DEPTH), FR(max_ion,M_DEPTH), ! RFIELD/OPALAM
     >         DENSNC(M_DEPTH)

        include 'WAVELE_R.INC'

        include 'RE.INC'                                      ! Teff/log g etc.

        real aln10, boltzk, hplanc, clight
        COMMON /CONST/ ALN10, BOLTZK, HPLANC, CLIGHT

        include 'MODEL_RA.INC'

        include 'OPAC_RAT.INC'                              ! LINOPxx

        include 'RAD_RATE.INC'                               ! JOSH

C ***
C ***   Local variables
C ***
        INTEGER N_LIT6, N_LIT9, N_BIG9  ! number of points in each ODF
        PARAMETER( N_LIT6 = 419, N_LIT9 = 1221, N_BIG9 = 337)

        INTEGER LIT6, LIT9, BIG9        ! starting location of each ODF in WAVE
        PARAMETER( LIT6 = 1, LIT9 = LIT6 + N_LIT6,
     >             BIG9 = LIT9 + N_LIT9 )

        integer M_ALL_ODF, M_WAVE
        parameter( M_ALL_ODF = N_LIT6 + N_LIT9 + N_BIG9 )
        parameter( M_WAVE = N_LIT9 )
C ***                       ^^^^^^
C ***     MAX( N_LIT6, N_LIT9, N_BIG9 ); FORTRAN77 does not allow functions here

        integer m_x_sect, m_x_prof
        parameter( m_x_sect = 4000, m_x_prof = m_x_sect + 50 )
C ***                                                     ^^
C ***                                               arbitrary,
C ***            worst case would be M_WAVE but that is silly: 
C ***            all x-section points in a single ODF interval  :-()
        integer e, ee, emax, i, id, j, k, lo, n, nu, numax, status,
     >          ifrad, itrad
        integer ignore

        real wave( m_all_odf ), jnui( m_wave, m_depth )
        real wave_fix( 4 ), jnui_fix( m_depth )
        integer left

        real XWA(M_X_SECT), AWA(M_X_SECT)
        real nue( m_x_prof ), x_sect( m_x_prof )
        real a( m_x_prof ), b( m_x_prof ), c( m_x_prof ), d( m_x_prof )
        real fup( m_x_prof ), fdown( m_x_prof )
        real work( 3*m_x_prof ), jnue
        real RIJ(M_DEPTH,m_levels), RJI(M_DEPTH,m_levels)
        real TR(M_DEPTH)

        real a1, as

        real pele, pgas, dnc

        LOGICAL PRINT
        PARAMETER( PRINT = .TRUE. )

C ***
C ***   Files
C ***
        INTEGER F_ATM, F_CROSS, F_RADIAT, F_RIJ, F_JNUE
        PARAMETER( F_ATM = 1, F_CROSS = 2,
     >             F_RIJ = 21, F_RADIAT = 22, F_JNUE = 23) 


        logical hydro, odf
        integer nulo, nuhi, nudel, nn, ielem, nrij, imax, nbeg, nx
        real dum, endtra, endlam, trad, delt, aa, bb


C ***
C ***   Functions
C ***
        real rint, integral
        external rint, integral

C ***
C ***   Block Data
C ***
        external blockhj

C ***
C ***   ODF wavelengths
C ***
        include 'WAVE_DAT.INC'

C ***
C ***   Code
C ***
        BOLTZK = 1.38062E-16
        HPLANC = 6.62620E-27
        ALN10 = LOG( 10. )
        CLIGHT = 2.997925E+10
C ***
        IFLPT = 10
        ODFTYPE = 6
        SCLMIN = 0.
        SCLMAX = 0.
        SCDF = 1.
        IWOUT = 0
        WMIN = 0.
        WMAX = 0.
        READ( 5, *, IOSTAT = STATUS ) IFLPT, ODFTYPE, SCLMIN, SCLMAX,
     >                                SCDF, IWOUT, WMIN, WMAX
        IF( STATUS .NE. 0 ) THEN
           SCLMIN = 0.
           SCLMAX = 0.
           SCDF = 1.
           IWOUT = 0
           WMIN = 0.
           WMAX = 0.
        END IF
        IF( ODFTYPE .NE. 0 .AND. ODFTYPE .NE. 6 .AND. ODFTYPE .NE. 60
     >      .AND. ODFTYPE .NE. 90 .AND. ODFTYPE .NE. 91 )
     >  THEN
           WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >       ''error: Don''''t know ODF type: '', I4)' ) ODFTYPE
           CALL EXIT(1)
        END IF
C ***
C ***
C ******************************************************************************
C ***
C ***   Modellatmosphaere einlesen
C ***
        OPEN( UNIT = F_ATM, FILE = 'ATM' )
        REWIND F_ATM
        READ( F_ATM, '( I7 )' ) MAI
        AS = 0.
        DO 1 I = 1, MAI
           READ( F_ATM, '( I7, F9.4, F7.3 )' ) NAMI(I), WTI(I), A1
           A1 = EXP( ALN10*A1 )
           FRACI(I) = A1
           AS = AS + A1
1       CONTINUE
        DO 3 I = 1, MAI
           FRACI(I) = FRACI(I)/AS
3       CONTINUE
        AVEMAT = 0.
        DO 4 I = 1, MAI
           AVEMAT = AVEMAT + WTI(I)*FRACI(I)
4       CONTINUE
        AVEMAT = AVEMAT*1.660531E-24
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, I4/1X, F8.1, F6.2, F10.2, F7.2// )' )
     >    NDEPTH, TEFF, GLOG, REFLAM, RSU
        READ( F_ATM, '( 1X, 5E14.6 )' ) ( TAUREF(ID), ID = 1, NDEPTH )
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, 5F14.6 )' ) ( TEMP(ID), ID = 1, NDEPTH )
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, 5E14.6 )' ) ( PELG(ID), ID = 1, NDEPTH )
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, 5E14.6 )' ) ( PGLG(ID), ID = 1, NDEPTH)
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, 5E14.6 )' ) ( RKACON(ID ), ID = 1, NDEPTH )
        READ( F_ATM, '(//)' )
        READ( F_ATM, '( 1X, 5E14.6 )' ) ( RSICON(ID), ID = 1, NDEPTH )
        CLOSE( F_ATM )
        DO 5 ID = 1, NDEPTH
           TEMP( ID) = 5040./TEMP(ID)
           ROPREF(ID) = EXP( ALN10*RKACON(ID) ) + 
     >                  EXP( ALN10*RSICON(ID) )
5       CONTINUE
C ***
C ******************************************************************************
C ***
C ***
        OPEN( UNIT = F_CROSS, FILE = 'CROSS' )
        REWIND F_CROSS
        DO 10 ID = 1, NDEPTH
C ***
C ***      Berechnen fuer jede Tiefenstufe:
C ***
C ***           Fuer alle Elemente von IONDIS Mode 1:
C ***
C ***              ZETAJ = Bruchteil pro Quantenzelle der Atome einer
C ***                      Ionisationsstufe, die im Grundzustand sind, bezogen
C ***                      auf die Kerndichte.
C ***
C ***              FRACJ = Bruchteil aller Atome einer Ionisationsstufe,
C ***                      bezogen auf die Kerndichte.
C ***                      = Summe ueber alle Zustaende von ZETAJ.
C ***
           PELE = EXP( ALN10*PELG(ID) )
           CALL IONDIS( TEMP(ID), PELE, 1, 1, PGAS, DNC )
C ***                     ^         ^   ^  ^    v    v
           DENSNC(ID) = DNC
           DO 101 I = 1, JMOL2
              ZE(I,ID) = ZETAJ(I) ! aus IONDIS, COMMON/I/
              FR(I,ID) = FRACJ(I) ! aus IONDIS, COMMON/I/
101        CONTINUE
10      CONTINUE
C ***
C ***
C ******************************************************************************
C ***
        READ( F_CROSS, '(I4)' ) NUMAX
        IF( NUMAX .GT. M_WAVE ) THEN
           WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >       ''error: There can''''t be more fixed lines than '', I3)' )
     >       M_WAVE
           call exit( 1 )
        END IF
        IF( NUMAX .GT. 0 ) THEN
C ***
C ***      Fuer Linien mit festen Raten (W.Steenbock: Nicht zu empfehlen.):
C ***        
C ***      Berechnung des Strahlungsfeldes J nue fuer jede Linie in jeder
C ***      Tiefenstufe:
C ***        
           k = 0
           do i = 1, numax, 4
              left = min( numax - i + 1, 4 )
              read( f_cross, '(4f10.3)' ) ( wave_fix(nu),
     >                                      nu = 1, left )
              do j = 1, left
                 call rfield( wave_fix(j), 1, 2, .not.print,
C ***                                         ^
C ***                   UV-Strahlungsfeld = J nue mit kont. + lin. Opaz. (ODF)
C ***
     >                        jnui_fix, 1 )
                 do id = 1, ndepth
                    jnui( k + j, id ) = jnui_fix( id )
                 end do
              end do
              k = k + 4
           end do
C ***
C ***      Output des Ergebnisses auf File RADIAT:
C ***
           OPEN( UNIT = F_RADIAT, FILE = 'RADIAT',
     >           FORM = 'UNFORMATTED' )
           REWIND F_RADIAT
           DO ID = 1, NDEPTH
              DO NU = 1, NUMAX
                 WRITE( F_RADIAT ) JNUI(NU,ID)
              END DO
           END DO
           REWIND F_RADIAT
           CLOSE( F_RADIAT )
        END IF
         
C ***
C ******************************************************************************
C ***
C ***   Berechnung von J nue kont. fuer jede Wellenlaenge der ODF.
C ***
C ***   Wird spaeter ausschliesslich zur Bestimmung der Photoionisationsraten
C ***   nur von RATES benoetigt.
C ***
C ***   Output auf File JNUE.
C ***
        READ( F_CROSS, '(I3)' ) IFRAD
C ***
C ***   Steuert, ob J nue berechnet werden soll (mit oder ohne ODF).
C ***
C ***   IFRAD = 0:      UV-Strahlungsfeld = B nue(T)  ACHTUNG: ITRAD darf nie   
C ***                                                 gleichzeitig null sein!   
C ***   IFRAD = 1:      UV-Strahlungsfeld = J nue (nur kont. Opazitaeten)       
C ***   IFRAD = 2:      UV-Strahlungsfeld = J nue mit kont. + lin. Opaz. (ODF)  
C ***   IFRAD = 3:      UV-Strahlungsfeld = J nue mit kont. + lin. Opaz. (JB)   
C ***    

        IF( ODFTYPE .EQ. 90 ) THEN
           NULO = LIT9
           NUHI = LIT9 + N_LIT9 - 1
           NUDEL = 1
        ELSE IF( ODFTYPE .EQ. 91 ) THEN
           NULO = BIG9
           NUHI = BIG9 + N_BIG9 - 1
           NUDEL = 1
        ELSE
           NULO = LIT6
           NUHI = LIT6 + N_LIT6 - 1
           NUDEL = 1
        END IF

        nn = 0
        DO NU = NULO, NUHI, NUDEL
C ***
C ***      WAVE umsortieren, so dass WAVE(1) = WAVE(NULO) und in Angstroem.
C ***
           NN = NN + 1
           WAVE(NN) = 10.*WAVE(NU)
        end do

        read( f_cross, '(4i4)' ) numax, nulo, nuhi, nudel
        if( numax .gt. 0 ) then
C ***
C ***      Ignore wavelengths (backward compatability)
C ***
           READ( F_CROSS, '(10F8.1)' ) (DUM, NU = 1, NUMAX)
        ELSE if( numax .lt. 0 ) then
C ***
C ***      Add or substitute wave length points
C ***
           numax = abs( numax )
           IF( NULO .EQ. 0  ) THEN
              WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error: NULO must not be 0!'')' ) 
              call exit( 1 )
           END IF
           if( nudel .le. 0 ) then
              write( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error: NUDEL must not be 0 or negative!'')' )
              call exit( 1 )
           end if
           if( nulo .lt. 0 ) then
C ***
C ***         Take everything from file, i.e, ignore ODF points known
C ***         to program
C ***
              i = 0
           else if( nulo .gt. 0 ) then
C ***
C ***         Add points
C ***
              i = nn
           end if
           nulo = abs( nulo )
           if( numax .gt. m_wave - i ) then
              write( *, '(''0'', 132(''-''), /, ''0 RATES>> Fatal '',
     >          ''error: Sorry, we can''''t use more than '', I3,
     >          ''wavelength points'')' ) M_WAVE - i
              call exit( 1 )
           end if
           if( nuhi .le. nulo .or. nuhi .gt. numax ) then
              write( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error: NUHI must not be larger NULO or NUMAX!'')' )
              call exit( 1 )
           end if
           read( f_cross, '(10f8.1)' ) (wave(nu), nu = i + 1, i + numax)
           do nu = i + nulo, i + nuhi, nudel
C ***
C ***         Multiply by 10 and thorw some points away (nulo, nuhi, nudel)
C ***
              nn = nn + 1
              wave( nn ) = 10.*wave( nu )
           end do

           call qqsort( wave, nn, .true. )
C ***                              ^^^^ ascending sort
        END IF

        CALL RFIELD( WAVE, NN, IFRAD, PRINT, JNUI, M_WAVE )

C ***
C ***   Ergebnis: JNUI(NU,ID), und HSURFI
C ***
        IF( IFRAD .NE. 0 ) THEN
           OPEN( F_JNUE, FILE = 'JNUE' )
           REWIND F_JNUE
           WRITE( F_JNUE, * ) NDEPTH, NN
           DO NU = 1, NN
              WRITE( F_JNUE, * ) WAVE(NU)
              WRITE( F_JNUE, '(X, 1P, 6E12.5)' )
     >                                (JNUI(NU,ID), ID = 1, NDEPTH)
           END DO
           REWIND F_JNUE
           CLOSE( F_JNUE )

        END IF
C ***
C ***
C ******************************************************************************
C ***
C ***   Berechnung der Photoionisationsraten RIJ, RJI
C ***

C ***
C ***   Make pseudo-wavenumbers [inverse Aangstroems]
C ***
        do i = 1, nn
           wave( i ) = 1./wave( i )
        end do

        READ( F_CROSS, '(I5)' ) IELEM
        READ( F_CROSS, * ) NRIJ
        DO 100 N = 1, NRIJ
C ***
C ***      Einlesen der Photoionisationsquerschnitte:
C ***
C ***      IMAX         :       1: Querschnittsverlauf ~ nue^-3,
C ***                              XWA(1) = Lambda Kante
C ***                              AWA(1) = Querschnitt Kante
C ***                          >1: Anzahl Punkte
C ***
C ***      XWA(1..IMAX) :       lambda [A]
C ***
C ***      AWA(1..IMAX) :       Querschnitt( lambda ) [10^-18 cm^2]
C ***
C ***      ITRAD        :       0: UV-Feld = J nue
C ***                           1: UV-Feld = B nue(T rad), T rad = T e
C ***                           2: UV-Feld = B nue(T rad), T rad = T e Photosph.
C ***                           3: UV-Feld = B nue(T rad), T rad = T e Chromosph
C ***
C ***      ENDTRA       :       ITRAD = 0:    Ende der Integration
C ***                           ITRAD .ne. 0: T rad
C ***
           READ( F_CROSS, *, IOSTAT = STATUS ) IMAX,
     >                          (XWA(J), J = MIN(IMAX,M_X_SECT), 1, -1),
     >                          (AWA(J), J = MIN(IMAX,M_X_SECT), 1, -1),
     >                          ITRAD, ENDTRA
           IF( STATUS .NE. 0 ) THEN
              WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error when reading cross section #'', I3, '' ('',
     >          f11.3, ''A)'')' ) N, xwa(min(imax,m_x_sect))
              WRITE( *, *, IOSTAT = IGNORE ) '        STATUS: ', 
     >          status, ' IMAX: ', imax, ' ITRAD: ', itrad, ' ENDTRA: ',
     >          endtra
              CALL EXIT(1)
           ELSE IF( IMAX .GT. M_X_SECT ) THEN
              WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error: Not more than '', I4, '' points in cross '',
     >          ''sections! Offending cross section is #'', I3, '' ('',
     >          f11.3, ''A); wants '' I4, '' points.'')' ) M_X_SECT, N, 
     >          xwa(m_x_sect), IMAX
              CALL EXIT(1)
           END IF
           IF( IFRAD .EQ. 0 .AND. ITRAD .EQ. 0 ) THEN
              WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >           ''error: If IFRAD = 0, ITRAD must not be zero. '',
     >           ''Change crossection #'', I3, '' ('', f11.3, ''A)'')' )
     >           N, xwa(imax)
              CALL EXIT(1)
           END IF

           hydro = imax .eq. 1
           odf = itrad .eq. 0

C ***
C ***      Make pseudo-wavenumbers (frequencies) [inverse Aangstroems]
C ***      
           xwa( 1 ) = 1./xwa( 1 )
           do i = 2, imax 
              if( xwa(i) .le. xwa(i-1) ) then
                 WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >             ''error: wavelength for x-section #'', I3, '' not'',
     >             ''monotonic around '', f11.3, ''A and '', f11.3,
     >             ''A'')' ) N, xwa(i-1), xwa(i)
                 CALL EXIT(1)
              end if
              xwa( i ) = 1./xwa( i )
           end do

           IF( .not.odf ) THEN
C ***
C ***         UV-Feld = B nue(T rad)
C ***
              TRAD = ENDTRA
              DO 33 ID = 1, NDEPTH
C ***
C ***            Bestimmung von T rad {TR}
C ***
                 IF( ID .NE. NDEPTH ) DELT = TEMP(ID) - TEMP(ID + 1)
                 IF( ITRAD .EQ. 1 ) THEN
C ***
C ***               T rad = T e
C ***
                    TR(ID) = TEMP(ID)
                 ELSE IF( ITRAD .EQ. 2 ) THEN
C ***
C ***               T rad = T e Photosphaere
C ***
                    TR(ID) = TEMP(ID)
                    IF( DELT .GE. 0.0  ) THEN
                       TR(ID) = TRAD
                    ELSE
                       IF( TRAD .GT. TEMP(ID) ) TR(ID) = TRAD
                    END IF
                 ELSE IF( ITRAD .EQ. 3 ) THEN
C ***
C ***               T rad = T e Chromosphaere
C ***
                    TR(ID) = TEMP(ID)
                    IF( TRAD .LT. TEMP(ID) .AND. DELT .GT. 0.0 ) 
     >                TR(ID) = TRAD
                 END IF
33            CONTINUE
              IF( hydro ) THEN
C ***
C ***            Bestimmung der Photoionisationsraten RIJ und RJI wenn:
C ***
C ***            UV-Feld = B nue(T rad) und Querschnitt wasserstoffaehnlich.
C ***
                 DO 37 ID = 1, NDEPTH
                    AA = 7.534607E17*AWA(1)*XWA(1)**3     ! 8*pi*c * a/lambda**3
                    BB = RINT( 1.438836E8/TR(ID)*XWA(1) ) !  -hc/kT / lambda
                    RIJ(ID,N) = AA*BB
                    IF( TR(ID) .NE. TEMP(ID) ) BB =
     >                RINT( 1.438836E8/TEMP(ID )*XWA(1) ) !  -hc/kT / lambda
                    RJI(ID,N) = AA*BB
37               CONTINUE
C ***
C ***            UV-Feld = B nue(T rad) und Querschnitt wasserstoffaehnlich:
C ***            FERTIG! Naechster Uebergang.
C ***
                 GO TO 100
C ***
C ***
C ***
              ELSE
                 READ( F_CROSS, * ) ENDTRA
              END IF
           END IF
C ***
C ***      Bestimmung der Photoion'raten RIJ und RJI, ausser wenn gleichzeitig
C ***      UV-Feld = B nue(T rad) und wasserstoffaehnlicher Querschnitt.
C ***
           endlam = 1./endtra
           if( endlam .gt. wave(1) ) then
              write( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >           ''error:  ENDLAM too small for x-section #'', I3,
     >           '' ('', f11.3, ''A)'')' ) N, 1./xwa(imax)
              call exit(1)
           end if
           nbeg = nn
           do while( wave(nbeg) .lt. xwa(imax) .and. nbeg .ge. 1 )
C ***         
C ***         Search for photoionization edge in wavenumber array of ODF
C ***         so point is in interval [nbeg,nbeg+1).     ( [k,k+1) )
C ***         
              nbeg = nbeg - 1
           end do
           if( nbeg .le. 1 ) then
              write( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >           ''error:  ODF wavelength range too small in UV for '',
     >           ''x-section #'', I3, '' ('', f11.3, ''A)'')' ) N,
     >           1./xwa(imax)
              call exit(1)
           end if

           if( hydro ) then
C ***
C ***         Generate x-section for Hydrogenic approximation
C ***
              k = nbeg
              if( wave(k) .ge. endlam ) then
                 write( *, '(''0'', 132(''-''), /, ''0RATES>> '',
     >             ''Fatal error:  ENDLAM too large '',
     >             ''for x-section #'', i3, '' ('', f11.3, ''A)'')' ) N,
     >             1./xwa(imax)
                 call exit(1)
              end if
              nx = 1
              nue( nx ) = xwa( imax )
              x_sect( nx ) = awa( imax )
              if( nue(nx) .eq. wave(k) ) k = k - 1 ! Hit ODF point in tabulated
                                                   ! x-sect. by coincidence
              do while( wave(k) .lt. endlam )
                 nx = nx + 1
                 nue( nx ) = wave(k)
                 x_sect( nx ) = awa( imax )*( xwa(imax)/nue(nx) )**3
                 k = k - 1
Can't happen:    IF( NX .gt. m_x_prof ) or ODF wouldn't fit in memory
              end do
           else
C ***
C ***         X-section given explicitly
C ***
C ***         Generate x-section with at least one point per ODF interval
C ***         producing NX many
C ***
              i = imax
              k = nbeg
              nx = 1
              nue( nx ) = xwa( imax )
              x_sect( nx ) = awa( imax )
              if( nue(nx) .eq. wave(k) ) k = k - 1 ! Hit ODF point in tabulated
                                                   ! x-sect. by coincidence
              do while( k .ge. 1 .and. wave(k) .le. endlam )
              
C ***            
C ***            no point in x-section for this interval ==> 
C ***            interpolate one for interval center
C ***            
                 nx = nx + 1
                 nue( nx ) = wave( k )
                 call mintep( nue(nx), x_sect(nx), xwa,
     >                        awa, imax, status )
                 k = k - 1     ! next ODF interval
              end do !while
              if( k .ge. nbeg - 2 ) then
                 write( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >             ''error:  ENDLAM too large for '',
     >             ''x-section #'', i3, '' ('', f11.3, ''A)'')' ) N,
     >             1./xwa(imax)
                 call exit(1)
              end if
           end if !( hydro )

           do nu = 1, nx
              a( nu ) = 1.8964671e9/nue( nu )*x_sect( nu ) ! 4*pi/h * a * lambda
              b( nu ) = 7.534607e17*nue( nu )*nue( nu )*x_sect( nu )
                                                         ! 8*pi*c * a / lambda^2
              c( nu ) = -1.438836e8*nue( nu )              !  -hc/k / lambda
              d( nu ) = 3.97297e8*nue( nu )**3             !
           end do

           DO 26 ID = 1, NDEPTH
C ***
C ***         Bestimmen des ionisierenden Strahlungsfeldes BNUJ und
C ***         des Integranden aufwaerts F1 und abwaerts F2.
C ***
C ***         F1 = 4*pi/h nue * a * BNUJ
C ***         F2 = 4*pi/h nue * a *(2hnue**3/c**2 + BNUJ)*exp(-h nue/kT) ( ODF )
C ***            = 4*pi/h nue * a * B nue                             ( no ODF )
C ***
              if( odf ) then
C ***
C ***            Interpolate flux for first x-sect. point
C ***
                 lo = max( 1, nbeg - 1 )
                 call mintep( xwa(imax), jnue, wave(lo), jnui(lo,id),
     >                        min(4, m_wave-lo+1), status )
                 nu = 1
                 fup( nu ) = a( nu )*jnue
                 fdown( nu ) = ( fup(nu) + b(nu) )*
     >                         exp( c(nu)/temp(id) )
                 do nu = 2, nx
C ***
C ***               Take other flux points at ODF interval centers
C ***
                    fup( nu ) = a( nu )*jnui( nbeg - nu + 2, id )
                    fdown( nu ) = ( fup(nu) + b(nu) )*
     >                            exp( c(nu)/temp(id) )
                 end do
                 rij( id, n ) = integral( nue, fup, work, nx )
                 rji( id, n ) = integral( nue, fdown, work, nx )
              else
                 do nu = 1, nx
                    jnue = d( nu )/
     >                     ( exp( -c(nu)/tr(id) ) + 1. )
                    fup( nu ) = a( nu )*jnue
                 end do
                 rij( id, n ) = integral( nue, fup, work, nx )
                 if( tr(id) .ne. temp(id) ) then
                    do nu = 1, nx
                       fdown( nu ) = a( nu )*d( nu )/
     >                               ( exp( -c(nu)/temp(id) ) + 1. )
                    end do
                    rji( id, n ) = integral( nue, fdown, work, nx )
                 else
                    rji( id, n ) = rij( id, n )
                 end if
              end if
26         CONTINUE
100     CONTINUE
        REWIND F_CROSS
        CLOSE( F_CROSS )
C ***
C ***   Herausschreiben des Ergebnisses (RIJ, RJI) auf File RIJ.
C ***
        OPEN( UNIT = F_RIJ, FILE = 'RIJ', FORM = 'UNFORMATTED' )
        REWIND F_RIJ
        DO ID = 1, NDEPTH
           DO I = 1, NRIJ
              WRITE( F_RIJ ) RIJ(ID,I), RJI(ID,I)
           END DO
        END DO
        REWIND F_RIJ
        CLOSE( F_RIJ )

        END
