        SUBROUTINE RFIELD( WAVE, NWAVE, FIELD_TYPE, PRINT, JNUI, NJNU )
C ***
C ******************************************************************************
C ***
C ***   Computes radiation field at all wavelengths WAVE with 
C ***
C ***   FIELD_TYPE = 0:   field = B nue(T)
C ***   FIELD_TYPE = 1:   field = J nue (only cont. opacities)       
C ***   FIELD_TYPE = 2:   field = J nue with cont. + line opac. (ODF)
C ***   FIELD_TYPE = 3:   field = J nue with cont. + line opac. (JB)   
C ***
C ***   PRINT:  print fluxes if .TRUE.
C ***   NWAVE:  number of wavelength points
C ***
C ***   Output in COMMON /....
C ***
C ***   Modifications:
C ***
C ***   26-MAR-1993 00:37:26.00         ML / UT Austin.  First version taken
C ***                                   out of RATES.
C ***
C ***    7-APR-1993 13:59:45.58         ML / UT Austin.  Swapped index in JNUI.
C ***
C ***   15-APR-1993 17:10:08.03         ML / UT Austin.  Changed to use mean
C ***                                   wavelength between two points in WAVE.
C ***                                   Needed for correct use of ODF.
C ***
C ***   23-APR-1993 21:40:27.88         ML / UT Austin.  Swapped index in JNUI
C ***                                   (again) and RESID.  Changed JNUI to
C ***                                   always return J without ODF.  Multiply
C ***                                   by RESID to get it with ODF included.
C ***
C ***   25-APR-1993 00:56:38.80         ML / UT Austin.  Reversed modification
C ***                                   from 15-APR-1993 17:10:08.03.  Not
C ***                                   needed, was wrong.
C ***
C ***   16-MAY-1993 16:00:32.29         ML / UT Austin.  Removed RESID.
C *** 
C ***    8-JUL-1993 19:15:01.26         ML / IoA Cambridge.  Assigned value 
C ***                                   to HSURFC if IFRAD .eq. 1
C ***
C ***	 7-MAR-1996 19:12:15.87		ML / Bamberg.  Removed rewind on ODF
C ***                                   close to allow read from named pipe.
C ***
C ***	29-APR-1996 14:02:22.53		ML / Bamberg.  Accomodate 5 ion stages.
C ***
C ***	29-NOV-1999 16:25:23.00		ML / McA.  Format x -> 1x
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***   Arguments
C ***
        integer nwave, njnu
        real wave(nwave), jnui(njnu,*)
        integer field_type
        logical print

C ***
C ***   Global variables
C ***
        include 'MOD_SIZ.INC'

	include 'OPALION.INC'

        real ze, fr, densnc
        COMMON /ZFRA/ ZE(max_ion,M_DEPTH), FR(max_ion,M_DEPTH), ! RATES/OPALAM
     >         DENSNC(M_DEPTH)

        include 'WAVELE_R.INC'

        include 'RE.INC'                                    ! Teff/log g etc.

        real aln10, boltzk, hplanc, clight
        COMMON /CONST/ ALN10, BOLTZK, HPLANC, CLIGHT

        include 'MODEL_RA.INC'

        include 'OPAC_RAT.INC'                            ! LINOPxx

        include 'RAD_RATE.INC'                             ! JOSH

C ***
C ***   Local variables
C ***
        INTEGER F_DF, F_JB
        PARAMETER( F_DF = 9, F_JB = 10 )

        integer page_len
        parameter( page_len = 58 )

        REAL RINIT
        PARAMETER( RINIT = -1.E37 )

        INTEGER JB_DEPTH
        REAL JB_TAU( M_DEPTH ), JB_WAVE( 4 ), JB( 4, M_DEPTH ),
     >       JB_HELP( M_DEPTH ), JB_NEW
        DATA JB_WAVE /4*RINIT/

        INTEGER I, J, ID, NU
        REAL PELE, EX, OKAPPA, OSIGMA

        LOGICAL DONE
        integer n, nsteps
        real stepwt, sumwt, hsurfc, hsurfi, jnuc(m_depth),
     >       taunuc(m_depth)

        integer ier, ntone
        integer page, ip
        real cl

        data page /1/, ip /page_len/

C ***
C ***   Code
C ***


        IF( FIELD_TYPE .NE. 0 ) THEN
C ***
C ***      Open ODF or J/B file
C ***
           IF( FIELD_TYPE .EQ. 2 ) THEN
              OPEN( UNIT = F_DF, FILE = 'DF', FORM = 'UNFORMATTED' )
           ELSE IF( FIELD_TYPE .EQ. 3 ) THEN
              OPEN( F_JB, FILE = 'JB', STATUS = 'OLD' )
              READ( F_JB, * ) JB_DEPTH, ( JB_TAU(ID), ID = 1, MIN(
     >          JB_DEPTH, M_DEPTH ) )
              IF( JB_DEPTH .GT. M_DEPTH ) THEN
                 WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >             ''error: Not more than '', I2, '' depth points in '', 
     >             ''file JB!'')' ) M_DEPTH
                 STOP
              END IF
           ELSE IF( FIELD_TYPE .LT. 0 .OR. FIELD_TYPE .GE. 4 ) THEN
              WRITE( *, '(''0'', 132(''-''), /, ''0RATES>> Fatal '',
     >          ''error: Unknown option for rad. field: '', I3)' )
     >          FIELD_TYPE
              STOP
           END IF
        END IF
C ***    
C ***   J nue mit oder ohne ODF.
C ***   
        DO 11 NU = 1, NWAVE
C ***   
C ***      Fuer jede Wellenlaenge der ODF:
C ***   
           LAMBDA = WAVE(NU)
           DO ID = 1, NDEPTH
C ***       
C ***         Fuer alle Tiefenstufen:
C ***       
C ***         Berechnung von Kappa und Sigma [cm^2 pro Kern].
C ***       
              PELE = EXP( ALN10*PELG(ID) )
              EX = EXP( -1.438836E8/LAMBDA/TEMP(ID) )
              BNU(ID) = 3.97297E8/LAMBDA**3*EX/(1. - EX)
              DO I = 1, JMOL2
                 ZETAJ(I) = ZE(I,ID) ! Input fuer OPALAM, COMMON/I/
                 FRACJ(I) = FR(I,ID) ! Input fuer OPALAM, COMMON/I/
              END DO
              CALL OPALAM( LAMBDA, TEMP(ID), PELE, DENSNC(ID), 1,
C                             ^          ^     ^      ^        ^
     >          OKAPPA, OSIGMA )
C                  v      v
              RKACON(ID) = OKAPPA
              RSICON(ID) = OSIGMA
              RLINE(ID) = 0.
           END DO
C ***   
C ***      Berechnung von J nue, Tau nue und S nue OHNE ODF
C ***                
           CALL JOSH 
           DO ID = 1, NDEPTH
              JNUI(NU,ID) = 0.
              JNUC(ID) = JNU(ID)
              TAUNUC(ID) = TAUNU(ID)
           END DO
           IF( FIELD_TYPE .EQ. 1 ) THEN
C ***                
C ***         UV-Feld ohne ODF,
C ***                
C ***         J nue und H nue surf fuer weitere Verwendung aufbewahren.
C ***                
              DO ID = 1, NDEPTH
                 JNUI(NU,ID) = JNUC(ID)
              END DO
              HSURFI = HSURF
              HSURFC = HSURF
           ELSE IF( FIELD_TYPE .EQ. 2 ) THEN
C ***                
C ***         UV-Feld mit ODF,
C ***                
C ***         Berechnung von J nue, Tau nue und S nue MIT ODF,
C ***         Summation ueber alle Teilintervalle.
C ***                
              HSURFI = 0.
              HSURFC = HSURF
              SUMWT = 0.
              N = 0
              NSTEPS = 1
              DONE = .FALSE.
                     
              DO WHILE( .NOT.DONE .AND. N .NE. NSTEPS )
                 N = N + 1
C ***                   
C ***            Linienopazitaet RLINE common/OPAC/ bereitstellen:
C ***            Aus ODF als Funktion von LAMBDA        common/WAVELE/,
C ***                                     TEMP(i)       common/MODEL/,
C ***                                     PELG(i)       common/MODEL/
C ***            fuer Teilintervall N.
C ***                   
                 IF( ODFTYPE .EQ. 0 .OR. ODFTYPE .EQ. 6 .OR.
     >               ODFTYPE .EQ. 60 )
     >           THEN
                    CALL LINOP( N, NSTEPS, STEPWT, AVEMAT )
                 ELSE IF( ODFTYPE .EQ. 90 .OR. ODFTYPE .EQ. 91 ) THEN
                    CALL LINOPV9( N, NSTEPS, STEPWT, AVEMAT )
                 ELSE   
                    WRITE( *, * ) 'RATES/RFIELD>> Don''t know ODF type',
     >                            ODFTYPE
                    CALL EXIT(1)
                 END IF 
C ***                   
C ***            Output der Opazitaeten
C ***                   
                 IF( LAMBDA .GE. WMIN .AND. LAMBDA .LE. WMAX )
     >           THEN
                    CALL WROP( 1, NDEPTH, IWOUT, LAMBDA, TEFF, 
     >                         GLOG )
                 END IF 
C ***                   
C ***            Neues Strahlungsfeld mit Linienopazitaet.
C ***                   
                 CALL JOSH
C ***                   
C ***            Summieren
C ***                   
                 SUMWT = SUMWT + STEPWT
                 DONE = .TRUE.
                 DO ID = 1, NDEPTH
                    JNUI(NU,ID) = JNUI(NU,ID) + JNU(ID)*STEPWT
	if( jnu(id).le.0. .or. jnui(nu,id) .le. 0. ) then
	   write( 6, * ) 'Negative J!!!, ID=', id, ' jnu(id)=', jnu(id),
     >        ' stepwt=', stepwt, 'jnui=', jnui(nu,id)
	end if
                    IF( JNU(ID)/JNUC(ID) .LT. .998 ) DONE = .FALSE.
                 END DO
                 HSURFI = HSURFI + HSURF*STEPWT

                 IF( DONE ) THEN
C ***                
C ***               Opazitaet weiterer Steps tragen nicht bei, dann ohne ODF
C ***               weiter summieren.
C ***                   
                    STEPWT = 1. - SUMWT
                    DO ID = 1, NDEPTH
                       JNUI(NU,ID) = JNUI(NU,ID) + JNUC(ID)*STEPWT
	if( jnuc(id) .le. 0. .or. jnui(nu,id) .le. 0. ) then
	   write( 6, * ) 'Negative J!!!, ID=', id, ' jnuc(id)=',
     >        jnuc(id), ' stepwt=', stepwt
	end if
                    END DO
                    HSURFI = HSURFI + HSURFC*STEPWT
                 
                 END IF
              END DO

           ELSE IF( FIELD_TYPE .EQ. 3 ) THEN
C ***                
C ***         UV field using J/B from file
C ***                
C ***                
C ***         Find wavelength in J/B
C ***                
              DO I = 1, 4
                 IF( JB_WAVE(I) .GT. LAMBDA ) GO TO 181
              END DO 
C ***                
C ***         Not there, read some so that LAMBDA is between JB_WAVE(3) and 
C ***         JB_WAVE(4)
C ***                
              I = 4  
              J = 5 
              DO WHILE( JB_WAVE(I) .LE. LAMBDA )
                 JB_WAVE(3) = JB_WAVE(I)
                 DO ID = 1, NDEPTH
                    JB(3,ID) = JB(I,ID)
                 END DO
                 READ( F_JB, *, END = 182 ) JB_WAVE(I), 
     >             ( JB(I,ID), ID = 1, JB_DEPTH )
                 J = J - 1
              END DO 
              DO J = 4, MAX( J, 3 ), -1
C ***                
C ***            Interpolate the *new* data in tau
C ***                
                 DO ID =  1, JB_DEPTH
                    JB(J,ID) = EXP( ALN10*JB(J,ID) )
                    JB_HELP( ID ) = JB( J, ID )
                 END DO
                 CALL INTEP( TAUREF(1), JB(J,1), JB_TAU(1),
     >             JB_HELP(1), JB_DEPTH, IER )
                 DO ID = 2, NDEPTH
                    CALL EINTEP( TAUREF(ID), JB(J,ID), JB_TAU(1),
     >                JB_HELP(1), JB_DEPTH, IER )
                 END DO
              END DO 
181           CONTINUE
              IF( JB_WAVE(3) .EQ. RINIT ) THEN
C ***                
C ***            Wavelength requested too small, not in JB:
C ***            J/B set = 1
C ***                
                 GO TO 182
              END IF 
C ***                
C ***         Is there, so shift and read J/B for new, higher wavelengths
C ***                
              DO J = 1, 4 - I + 2
                 JB_WAVE(J) = JB_WAVE(I - 1 + J - 1)
                 DO ID = 1, NDEPTH
                    JB(J,ID) = JB( I - 1 + J - 1, ID )
                 END DO
              END DO 
              DO J = 4 - I + 3, 4
                 READ( F_JB, *, END = 182 ) JB_WAVE(J), 
     >             (JB(J,ID), ID = 1, JB_DEPTH)
                 DO ID =  1, JB_DEPTH
                    JB(J,ID) = EXP( ALN10*JB(J,ID) )
                    JB_HELP( ID ) = JB( J, ID )
                 END DO
C ***                
C ***            Interpolate the *new* data in tau
C ***                
                 CALL INTEP( TAUREF(1), JB(J,1), JB_TAU(1),
     >             JB_HELP(1), JB_DEPTH, IER )
                 DO ID = 2, NDEPTH
                    CALL EINTEP( TAUREF(ID), JB(J,ID), JB_TAU(1),
     >                JB_HELP(1), JB_DEPTH, IER )
                 END DO
              END DO 
C ***                
C ***         Interpolation in lambda
C ***                
              DO ID = 1, NDEPTH
                 CALL INTEP( LAMBDA, JB_NEW, JB_WAVE(1), JB(1,ID),  
     >             4, IER )
                 JNUI(NU,ID) = JB_NEW*BNU(ID)
              END DO 
              GO TO 183
182              CONTINUE
                 DO ID = 1, NDEPTH
                    JNUI(NU,ID) = BNU(ID)
                 END DO
                 WRITE( *, '('' RATES>> Warning: J/B = 1 assumed '',
     >             ''for '', I6, '' A'')' ) INT( LAMBDA )
183           CONTINUE
              HSURFI = HSURF         ! some fudge
              HSURFC = HSURFI        ! more fudge
           END IF
           IF( PRINT ) THEN
C ***                
C ***         Lineprinter Output
C ***                   
              NTONE = NDEPTH + 1
              DO 41 ID = NDEPTH, 1, -1
                 IF( TAUNUC(ID) .GE. 1. ) NTONE = ID
41            CONTINUE  
              IF( NU .EQ. 1 .OR. NU .EQ. NWAVE .OR. MOD( NU, IFLPT )
     >            .EQ. 0 ) THEN  
C ***                   
C ***            Nur manchmal Output, Papier sparen.
C ***                   
                 CL = 2.997925E18/LAMBDA/LAMBDA ! c/lambda**2 [(A/s)/A**2]
                 IF( IP .EQ. PAGE_LEN ) THEN
                    WRITE( *, 2002 ) PAGE
                    IF( SCDF .NE. 1. ) THEN
                       WRITE( *, 2003 ) SCDF, SCLMIN, SCLMAX
                    END IF
                    WRITE( *, 2004 ) INT( TEFF ), GLOG
                    IP = 8
                    PAGE = PAGE + 1
                 END IF 
                 WRITE( *, 2006 ) NU, LAMBDA, N, HSURFC, HSURFI,
     >                            HSURFI/HSURFC, HSURFI*CL,
     >                            HSURFI*CL*3.1415926535897932,
     >                            NTONE, LOG10( TAUNUC(NDEPTH) )
                 IP = IP + 1
2002             FORMAT( 1H1, /, 1X, ' F L U X E S    (F = ',
     >             'Astrophysical Flux)   as used for ',
     >             'photoionisation in program RATES', T127, 'Page ',
     >             I2)  
2003             FORMAT( 1X, T45, 'ODF scaled with factor', F5.2,
     >             ' from', F8.2, 'A to', F8.2, 'A.' )
2004             FORMAT( 1X, T45, 'Teff = ', I5, '   log g = ', F4.2,
     >             //, 1X,
     >             '       lambda   Steps      F nu cont           ',
     >             'F nu         Resid.       F lambda       pi*F ',
     >             'lambda     N tau=1  log tau(last point)', /, 1X,
     >             '         [A]           [erg/s/cm**2/Hz]  ',
     >             '[erg/s/cm**2/Hz]            [erg/s/cm**2/A]  ',
     >             '[erg/s/cm**2/A]', / )
2006             FORMAT( 1X, I4, 2X, F7.0, 5X, I2, 6X, 1P, E10.3, 8X,
     >             E10.3, 5X, 0P, F7.4, 5X, 1P, E10.3, 7X, E10.3, 9X,
     >             0P, I2, 12X, F6.3 )
              END IF
           END IF
11      CONTINUE

        IF( FIELD_TYPE .EQ. 2 ) THEN
*           REWIND F_DF
           CLOSE( F_DF )
        ELSE IF( FIELD_TYPE .EQ. 3 ) THEN
           REWIND F_JB
           CLOSE( F_JB )
        END IF
         
        END

