        subroutine sscal( n, sa, sx, incx )
C ***
C ******************************************************************************
C ***
C ***   Substitue for Cray routine.
C ***
C ***
C ***SCAL(3SCI)
C ***
C ***
C ***
C ***     NAME
C ***          SSCAL, CSSCAL, CSCAL - Scales a real or complex vector
C ***
C ***     SYNOPSIS
C ***          CALL SSCAL(n,sa,sx,incx)
C ***
C ***          CALL CSSCAL(n,sa,cx,incx)
C ***
C ***          CALL CSCAL(n,ca,cx,incx)
C ***
C ***     DESCRIPTION
C ***          n         Number of elements in the vector. If n <= 0,
C ***                    SSCAL, CSSCAL, and CSCAL return without any
C ***                    computation.  (input)
C ***
C ***          sa        Real scaling factor  (input)
C ***
C ***          ca        Complex scaling factor  (input)
C ***
C ***          sx        Real vector to be scaled  (input/output)
C ***
C ***          cx        Complex vector to be scaled  (input/output)
C ***
C ***          incx      Increment between elements of sx or cx  (input)
C ***
C ***          These subroutines scale a vector.
C ***
C ***          SSCAL computes
C ***
C ***               X = aX
C ***
C ***          where a is a real number and X is a real vector.
C ***
C ***          CSSCAL computes
C ***
C ***               X = aX
C ***
C ***          where a is a real number and X is a complex vector.
C ***
C ***          CSCAL computes
C ***
C ***               Y = aY
C ***
C ***          where a is a complex number and Y is a complex vector.
C ***
C ***     CAUTIONS
C ***          Do not specify incx as zero; doing so yields unpredictable
C ***          results.
C ***
C ***
C ***
C ***
C ***
C ***
C ***
C ***     SR-2081 5.1                     1             Cray Research, Inc.
C ***
C ***
C ***
C ***   19-DEC-1990 20:31:49.22
C ***
C ******************************************************************************
C ***
        real sx(n*incx)

        if( n .gt. 0 ) then
           j = 1
           do i = 1, n
              sx(j) = sx(j)*sa
              j = j + incx
           end do
        end if
        end
