        function stark( n, m, freq, t, pele )
C ***
C ******************************************************************************
C ***
C ***	Compute Stark parameters for Hydrogen.  Stolen from ATLAS9.
C ***
C ***	n:	in	lower quantum number
C ***	m:	in	upper quantum number
C ***	freq:	in	frequency [Hz]
C ***	t:	in	temperature [K]
C ***	pele:	in	electron density [cm^-3]
C ***
C ***	14-FEB-1994 17:56:38.00		ML / IoA
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***	Arguments
C ***
	integer n, m
	real freq, t, pele, stark

C ***
C ***	Local variables

	integer mminn
        real knmtab( 5, 4 ), fstark( 10, 4 )
        real beta, del, dbeta, dioi, exy2, f0, fnm, freqnm, hkt
	real impact, knm, kt, ne, nn, mm, oldt, prof, qstat
	real ratio, x, xm, xn, xx, y1, y2

C       IF YOUR RYDBERG IS DIFFERENT YOU MAY GET LINES IN STRANGE PLACES
	real ryd
        parameter( ryd = 3.28805E15 )

C ***
C ***	Functions
C ***
	real exint

	save oldt, kt, ne, hkt, f0, knmtab, fstark

        data knmtab/.000356, .000523, .00109, .00149, .00225, .0125,
     >       .0177, .028, .0348, .0493, .124, .171, .223, .261, .342,
     >       .683, .866, 1.02, 1.19, 1.46/
        data fstark/.1387, .07910, .02126, .01394, .006462, .004814,
     >       .002779, .002216, .001443, .001201, .3921, .1193, .03766,
     >       .02209, .01139, .008036, .005007, .003850, .002658,
     >       .002151, .6103, .1506, .04931, .02768, .01485, .01023,
     >       .006588, .004996, .003542, .002838, .8163, .1788, .05985,
     >       .03189, .01762, .01196, .007825, .005882, .004233, .003375/
	data oldt/-5./


        exint( x ) = -log( x ) - .57516 +
     >               ( .97996 - (.21654 - (.033572 - (.0029222 -
     >               1.05439E-4*x)*x)*x)*x )*x


	if( t .ne. oldt ) then
	   kt = 1.38063e-16*t
	   ne = pele/kt
	   hkt = 6.6262e-27/kt
           f0 = 1.25E-9*ne**.6666667

	   oldt = t
	end if

        xn = n
        xm = m
        x = xn/xm
        xx = x**2
        nn = n*n
        mm = m*m
        mminn = m - n
        if( mminn .gt. 5 ) then
           knm = 5.5E-5*( nn*mm )**2/( mm - nn )
        else
           knm = knmtab( mminn, n )
        end if
        if( mminn .gt. 10 ) then
           fnm = fstark( 10, n )*( (20.*xn + 100.)/(xn + 10.)/xm/
     >           (1. - xx) )**3
        else
           fnm = fstark( mminn, n )
        end if
        freqnm = ryd*( 1./nn - 1./mm )
        del = abs( freq - freqnm )
        dbeta = 2.997925E18/freqnm**2/f0/knm
        beta = dbeta*del
        y1 = mm*del*hkt/2.
        y2 = ( 3.14159*3.14159/2./.0265384/2.997925E10 )*del**2/ne
        qstat = 1.5 + .5*( y1**2 - 1.384 )/( y1**2 + 1.384 )
        impact = 0.
        if( y1 .le. 8. .and. y1 .lt. y2 ) then
           exy2 = 0.
           if( y2 .le. 8. ) exy2 = exint( y2 )
           impact = 1.438*sqrt( y1*(1. - xx) )*
     >              ( .4*exp(-y1) + exint(y1) - .5*exy2 )
        end if
        if( beta .gt. 20. ) then
           prof = 1.5/beta/beta/sqrt( beta )
           dioi = 6.28*1.48E-25*( 2.*mm*ryd/del )*ne*
     >            ( sqrt(2.*mm*ryd/del)*(1.3*qstat + .30*impact) -
     >            3.9*ryd*hkt )
           ratio = qstat*min( 1. + dioi, 1.25e0 ) + impact
        else
           prof = 8./( 80. + beta**3 )
           ratio = qstat + impact
        end if
        stark = .0265384*fnm*prof*dbeta*ratio
        end
