        subroutine statistics( x, y, x_last, y_last )
C ***
C ******************************************************************************
C ***
C ***   $Id: statistics.F,v 1.3 2004/11/27 20:53:26 michael Exp $
C ***
C ***   Calculate and print some statistics on data within rectangle
C ***   (x,y), (x_last,y_last).
C ***
C ***   See also note in re_paint.F about data structures.
C ***
C ***
C ***    8-FEB-2004 21:40:54.00         ML / Stegaurach.  Made.
C ***    9-FEB-2004 08:20:01.00         ML / Stegaurach.  min/max points added.
C ***    9-FEB-2004 21:40:46.00         ML / Stegaurach.  Computation of stddev
C ***                                   fixed.
C ***   27-NOV-2004 21:47:50.00         ML / Stegaurach.  Call of PGPLOT routine
C ***                                   replaced with newpen.  Requires newpen
C ***                                   to accept ipen = 0.   This is for the
C ***                                   previously (2/9/2004) added plotting of
C ***                                   statistics rectangle.
C ***
C ******************************************************************************
C ***
        implicit none
        real x, y, x_last, y_last

        include 'RESIZE.INC'
        include 'RECT_SIZE.INC'
        include 'ERASURE.INC'

C ***
C ***   Local variables
C ***
        real xavg, yavg, xstd, ystd
        integer n
        double precision xsum, ysum, xsum2, ysum2
        real dxmin, dxmax, dymin, dymax
        integer i
        real pxmin(2), pxmax(2), pymin(2), pymax(2)
        real rectx(5), recty(5), markx(1), marky(1)
        integer icall /0/
        save icall, rectx, recty, markx, marky

C ***
C ***	Data structure of a curve:
C ***
	integer ndata		! no. of points in curve
#if defined( CRAY_PTR ) 
	real xdata(*), ydata(*)
	pointer( p_xdata, xdata )
	pointer( p_ydata, ydata )
#elif defined( REF_PTR )
	integer p_xdata
	integer p_ydata
#endif

C ***
C ***	Functions
C ***
	integer ideref
	external ideref
	external resize_init	! block data


C ***
C ***	Code
C ***
        icall = icall + 1

        dxmin = min( x, x_last )
        dxmax = max( x, x_last )
        dymin = min( y, y_last )
        dymax = max( y, y_last )

        ! limit to plot area:
        dxmin = max( dxmin, xmin )
        dxmax = min( dxmax, xmax )
        dymin = max( dymin, ymin )
        dymax = min( dymax, ymax )

        xsum = 0.d0
        ysum = 0.d0
        xsum2 = 0.d0
        ysum2 = 0.d0
        n = 0
        do i = 1, 2
           pxmin(i) = 1e38
           pymin(i) = 1e38
           pxmax(i) = -1e38
           pymax(i) = -1e38
        end do

        if( icall .gt. 1 ) then
           call gplane( 1, noErase, Both_planes )
           if( icall .gt. 2 ) then
              call newpen(0)
              call over( rectx, recty, 5, 0.1 )
              call over( markx, marky, 1, -32. )
           end if
           rectx(1) = dxmin
           rectx(2) = dxmax
           rectx(3) = dxmax
           rectx(4) = dxmin
           rectx(5) = dxmin

           recty(1) = dymin
           recty(2) = dymin
           recty(3) = dymax
           recty(4) = dymax
           recty(5) = dymin

           markx(1) = x
           marky(1) = y

           call newpen(1)
           call over( rectx, recty, 5, 0.1 )
           call over( markx, marky, 1, -32. )
           call tplane( noErase, Both_planes )
        end if

	do i = 1, ncurv
#if defined( CRAY_PTR )
	   p_xdata = xc(i)
           p_ydata = yc(i)
	   ndata = nc(i)
#elif defined( REF_PTR )
	   p_xdata = ideref( %val(pp_xdata), i )
           p_ydata = ideref( %val(pp_ydata), i )
	   ndata = ideref( %val(p_ndata), i )
#else
	   ndata = nc(i)
#endif
#if defined( CRAY_PTR )
           call do_stats( dxmin, dxmax, dymin, dymax,
     >                    xdata, ydata, ndata,
     >                    xsum, ysum, xsum2, ysum2, n,
     >                    pxmin, pxmax, pymin, pymax )
#elif defined( REF_PTR )
           call do_stats( dxmin, dxmax, dymin, dymax,
     >                    %val(p_xdata), %val(p_ydata), ndata,
     >                    xsum, ysum, xsum2, ysum2, n,
     >                    pxmin, pxmax, pymin, pymax )
#else
           call do_stats( dxmin, dxmax, dymin, dymax,
     >                    xc(1,i), yc(1,i), ndata,
     >                    xsum, ysum, xsum2, ysum2, n,
     >                    pxmin, pxmax, pymin, pymax )
#endif
	end do
        xavg = 1.e35
        yavg = 1.e35
        xstd = 1.e35
        ystd = 1.e35
        if( n .gt. 0 ) then
           xavg = xsum/n
           yavg = ysum/n
        end if
        if( n .gt. 1 ) then
           xstd = sqrt( (n*xsum2 - xsum*xsum)/(n-1)/n )
           ystd = sqrt( (n*ysum2 - ysum*ysum)/(n-1)/n )
        end if
        if( icall .eq. 1 ) then
           write( stdout, * ) 'Hit C again: '
           return
        end if
        write( stdout, '()' )
        write( stdout, * ) 'Statistics between ', dxmin, ' <= x <= ',
     >                     dxmax
        write( stdout, * ) '               and ', dymin, ' <= y <= ',
     >                     dymax
        write( stdout, * ) 'number of points: ', n
	if( n .gt. 0 ) then
           write( stdout, * ) '       x average: ', xavg, ' +/- ', xstd
           write( stdout, * ) '       y average: ', yavg, ' +/- ', ystd
           write( stdout, * ) '           x min: ', pxmin(1),
     >                        ' (y= ', pxmin(2), ')'
           write( stdout, * ) '           x max: ', pxmax(1),
     >                        ' (y= ', pxmax(2), ')'
           write( stdout, * ) '           y min: ', pymin(2),
     >                        ' (x= ', pymin(1), ')'
           write( stdout, * ) '           y max: ', pymax(2),
     >                        ' (x= ', pymax(1), ')'
        end if

        end
        subroutine do_stats( dxmin, dxmax, dymin, dymax,
     >                       xdata, ydata, ndata,
     >                       xsum, ysum, xsum2, ysum2, n,
     >                       pxmin, pxmax, pymin, pymax )
        implicit none
C ***
C ***   Arguments
C ***
        real dxmin, dxmax, dymin, dymax               ! intend in
        integer ndata                                 ! intend in
        real xdata(ndata), ydata(ndata)               ! intend in
        double precision xsum, ysum, xsum2, ysum2     ! intent in/out
        integer n                                     ! intent in/out
        real pxmin(2), pxmax(2), pymin(2), pymax(2)   ! intent out
C ***
C ***   Local variables   
C ***
        real x, y
        integer i

        do i = 1, ndata
           x = xdata(i)
           y = ydata(i)
           if( x .ge. dxmin .and. x .le. dxmax .and.
     >         y .ge. dymin .and. y .le. dymax ) then
              n = n + 1
              xsum = xsum + x
              ysum = ysum + y
              xsum2 = xsum2 + x*x
              ysum2 = ysum2 + y*y
              if( x .lt. pxmin(1) ) then
                 pxmin(1) = x
                 pxmin(2) = y
              end if
              if( x .gt. pxmax(1) ) then
                 pxmax(1) = x
                 pxmax(2) = y
              end if
              if( y .lt. pymin(2) ) then
                 pymin(1) = x
                 pymin(2) = y
              end if
              if( y .gt. pymax(2) ) then
                 pymax(1) = x
                 pymax(2) = y
              end if
           end if
        end do
        
        end
