	subroutine resize_add_text( x, y, text, size, loc, icol, icoor )
C ***
C ******************************************************************************
C ***
C ***	Routine to add text strings for resize.
C ***
C ***	This adds just one text string to the curves to be plotted and 
C ***	increments ntext (in common).
C ***
C ***	x		r >	x position for start of text
C ***	y		r >	y position for start of text
C ***	text		c >	text to add
C ***	size		r >	size for text (y user coord or normalized coord)
C ***	loc		i >	relative location of text:
C ***				 7 8 9
C ***	                         4 5 6
C ***                            1 2 3
C ***	icol		i >	color index for text
C ***	icoor		i >	coordinate unit flag: 101 x: norm, y: user,
C ***                           size: norm
C ***
C ***
C ***	19-JUN-1996 14:04:50.73		ML / Bamberg.  Created.
C ***	21-JUN-1996 21:01:49.36		ML / Bamberg.  icoord added.
C ***	 5-JUL-1996 19:30:23.20		ML / Bamberg.  Fixed copying
C ***                                   text_addr, text_len after allocating
C ***                                   new memory
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***   Arguments
C ***
        character text*(*)
	real x, y, size
	integer icol, loc, icoor

C ***
C ***	Gobal variables
C ***
	include 'RESIZE.INC'

C ***
C ***	Local variables
C ***
	integer chunk
	parameter( chunk = 20 )	! chunks in which to allocate memory

	logical first /.true./
	integer status, i

	real nxt(*), nyt(*), nsizet(*)
	integer nloct(*), ncolt(*), ncoort(*)

	integer*4 ntext_addr(*)
	integer*2 ntext_len(*)

	pointer( np_xt, nxt )
	pointer( np_yt, nyt )
	pointer( np_sizet, nsizet )
	pointer( np_loct, nloct )
	pointer( np_colt, ncolt )
	pointer( np_coort, ncoort )

	pointer( np_text_a, ntext_addr )
	pointer( np_text_l, ntext_len )


C ***
C ***	Functions
C ***
	integer lib$get_vm, lib$free_vm
	external lib$get_vm, lib$free_vm
	

	save first


C ***
C ***	Code
C ***
	if( first ) then
           first = .false.
	   ntext = 0
           ncutxt = 0
	end if

	ntext = ntext + 1
	if( ncutxt .lt. ntext ) then

C ***
C ***      Get new memory
C ***
           i = ncutxt + chunk
	   status = lib$get_vm( 4*i, np_xt )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_yt )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_sizet )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_loct )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_colt )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_coort )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_text_a )
	   if( .not.status ) call lib$signal( %val(status) )
	   status = lib$get_vm( 2*i, np_text_l )
	   if( .not.status ) call lib$signal( %val(status) )

           do i = 1, ntext - 1
C ***
C ***         copy the old data to the new location
C ***
              nxt(i) = xt(i) 
	      nyt(i) = yt(i) 
	      nsizet(i) = sizet(i) 
	      nloct(i) = loct(i) 
	      ncolt(i) = colt(i)
	      ncoort(i) = coort(i)
	      ntext_addr(i) = text_addr(i)
	      ntext_len(i) = text_len(i)
	   end do

           if( ncutxt .gt. 0 ) then
C ***
C ***         Free the old memory
C ***
	      status = lib$free_vm( 4*ncutxt, p_xt )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_yt )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_sizet )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_loct )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_colt )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_coort )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncutxt, p_text_a )
	      if( .not.status ) call lib$signal( %val(status) )
	      status = lib$free_vm( 2*ncutxt, p_text_l )
	      if( .not.status ) call lib$signal( %val(status) )

           end if

C ***
C ***      Fix pointers
C ***      
	   p_xt = np_xt
	   p_yt = np_yt
	   p_sizet = np_sizet
	   p_loct = np_loct
	   p_colt = np_colt
	   p_coort = np_coort
	   p_text_a = np_text_a
	   p_text_l = np_text_l

           ncutxt = ncutxt + chunk	! remember what we've got
	end if

        xt(ntext) = x
	yt(ntext) = y
	sizet(ntext) = size
	loct(ntext) = loc
	colt(ntext) = icol
	coort(ntext) = icoor

	text_addr(ntext) = %loc(text)  ! hope this doesn't return the address of the descriptor
	text_len(ntext) = len(text)


	end
